/********************************************************************************
 * E N T I T Y 									*
 ********************************************************************************
 * File: EntityGlobalMode.cpp							*
 *                                                                         	*
 * Copyright (C) A.R.Hardie-Bick 2004-2005					*
 *										*
 * This program is free software; you can redistribute it and/or modify		*
 * it under the terms of the GNU General Public License as published by		*
 * the Free Software Foundation; either version 2 of the License, or (at	*
 * your option) any later version.						*
 *										*
 * This program is distributed in the hope that it will be useful, but		*
 * WITHOUT ANY WARRANTY; without even the implied warranty of			*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU		*
 * General Public License for more details.					*
 *										*
 * You should have received a copy of the GNU General Public License		*
 * along with this program; if not, write to the Free Software			*
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.			*
 *******************************************************************************/

#include "EntityGlobalMode.h"

EntityGlobalMode::EntityGlobalMode(EntityPanel *panel_, EntityGlobal *global_, EntityFlash *flash_, EntitySysexMode *sysexMode_) : EntityMode (panel_) {

	sysexMode = sysexMode_;
	global = global_;
	flash = flash_;

	firstEdit = true;

	globalPara = global->getCurrentParameter();	
}

Mode EntityGlobalMode::enter() {

	panel->setEncoderAcceleration(globalPara->getAcceleration());
		
	panel->setLEDs(PANEL01_LED_EDIT);
	
	panel->blink(LED_BLINK, LED_BLINK_TIME, LED_BLINK_TIME);

	ledBlink = true;
	
	if(!global->fastEdit()) panel->setPotForModulation(PANEL01_POT_CTRL1);
        else panel->setPotForData(PANEL01_POT_CTRL1);
	
	processEvents();

	panel->setPotForModulation(PANEL01_POT_CTRL1);
	panel->clear();
	
	return rMode;
}	
	
Status EntityGlobalMode::incrementValue(int i) {

	return globalPara->incValue(i);
}

void EntityGlobalMode::edit() { rMode = RETURN; }

void EntityGlobalMode::editHold() {  

	panel->clear();
	
	switch(globalPara->save()) {

		case SAVE_CONFIG	:	rMode = SAVE_GLOBAL; break;
						
		case MIDI_SAVE_SOUND	:	sysexMode->saveActiveSoundToMidi(false); rMode = RETURN; break;
		case MIDI_SAVE_BANK	:	sysexMode->saveCurrentBankToMidi(); rMode = RETURN; break;
		case MIDI_SAVE_ALL	:	sysexMode->saveAllToMidi(); rMode = RETURN; break;
		case MIDI_SAVE_CONFIG 	:	sysexMode->saveGlobalToMidi(false); rMode = RETURN; break;

		default 		:	TRACE("ERROR: Unkown code in global save\n");
	}	
}

void EntityGlobalMode::partUp() { }
void EntityGlobalMode::partDown() { }
void EntityGlobalMode::pageUp() { }
void EntityGlobalMode::pageDown() { }

void EntityGlobalMode::paramUp() { 

	if(global->parameterInc(+1)) {
		
		globalPara = global->getCurrentParameter();
		displayUpdate();
		panel->repeat(AUTO_PARAM_UP, 330);
	}
}

void EntityGlobalMode::paramDown() { 

	if(global->parameterInc(-1)) {
		
		globalPara = global->getCurrentParameter();
		displayUpdate();
		panel->repeat(AUTO_PARAM_DOWN, 330);
	}
}

void EntityGlobalMode::pot1Event(int p) {

	if(global->setParameterByPot(p)) {

		globalPara = global->getCurrentParameter();
		displayUpdate();
	}
}

void EntityGlobalMode::editShift() {

	if(!global->toggleFastEdit()) panel->setPotForModulation(PANEL01_POT_CTRL1);
        else panel->setPotForData(PANEL01_POT_CTRL1);
}

void EntityGlobalMode::valueUp() { 

	panel->repeat(AUTO_VALUE_UP, 825, 50);
	incrementEvent();
}

void EntityGlobalMode::valueDown() { 

	panel->repeat(AUTO_VALUE_DOWN, 825, 50);
	decrementEvent();
}

void EntityGlobalMode::shiftParamUp() {	if(global->selectLastParameter()) displayUpdate(); }
void EntityGlobalMode::shiftParamDown() { if(global->selectFirstParameter()) displayUpdate(); }
void EntityGlobalMode::shiftValueUp() {	if(globalPara->clearDefault()) displayValueUpdate(); }
void EntityGlobalMode::shiftValueDown() { if(globalPara->setDefault()) displayValueUpdate(); }

void EntityGlobalMode::autoEvent(int event) {

	switch(event) {

		case AUTO_VALUE_UP 	:	incrementEvent(); break;
		case AUTO_VALUE_DOWN 	:	decrementEvent(); break;

		case AUTO_PARAM_UP	:	paramUp(); break;
		case AUTO_PARAM_DOWN 	:	paramDown(); break;
					
		case LED_BLINK 		:	if((ledBlink = !ledBlink)) panel->setLEDs(PANEL01_LED_EDIT);
						else panel->clearLEDs(PANEL01_LED_EDIT);
						break;
	}
}
	
void EntityGlobalMode::editPageDown() { // Deliberately obscure key combination for tuning calibration

	GlobalPara* newGlobalPara = global->getCalibrateTuningPara();
	
	if(newGlobalPara != globalPara) {		
	
		globalPara = newGlobalPara;
		panel->setEncoderAcceleration(globalPara->getAcceleration());
		displayUpdate();
	}
}

void EntityGlobalMode::displayUpdate() {

	panel->clear();
	panel->print(0, 0, globalPara->getName());
	
	displayValueUpdate();
	
	panel->setEncoderAcceleration(globalPara->getAcceleration());
}

void EntityGlobalMode::displayValueUpdate() {

	if(globalPara->save()) return;
	
	int len; 
	char *s = globalPara->getValStr(len);

	int spare = PANEL_LCD_MAX_LINE_LEN - len;
	int pos = spare / 2;

	if(len < 16) {
	
		char blank[32]; int x;
		
		// Clear leading characters

		for(x = 0; x < pos; x++) blank[x] = 32;
		blank[x] = 0; panel->print(1, 0, blank);

		// Clear trailing characters

		for(x = 0; x < (spare-pos); x++) blank[x] = 32;
		blank[x] = 0; panel->print(1, pos+len, blank);
	}

	panel->print(1, pos, s);
}	


