/********************************************************************************
 * E N T I T Y 									*
 ********************************************************************************
 * File: EntityLevelMeter.cpp							*
 *                                                                         	*
 * Copyright (C) A.R.Hardie-Bick 2004-2005					*
 *										*
 * This program is free software; you can redistribute it and/or modify		*
 * it under the terms of the GNU General Public License as published by		*
 * the Free Software Foundation; either version 2 of the License, or (at	*
 * your option) any later version.						*
 *										*
 * This program is distributed in the hope that it will be useful, but		*
 * WITHOUT ANY WARRANTY; without even the implied warranty of			*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU		*
 * General Public License for more details.					*
 *										*
 * You should have received a copy of the GNU General Public License		*
 * along with this program; if not, write to the Free Software			*
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.			*
 *******************************************************************************/

#include "EntityLevelMeter.h"

EntityLevelMeter::EntityLevelMeter() { 

	ovldL = 0; ovldR = 0;
	peakL = 0; peakR = 0;
	peakTimeL = 0; peakTimeR = 0;
	active = false;
}

char *EntityLevelMeter::draw(int inL, int inR) {

	// Level inputs are in range 0-16, where 16 is saturation

	const int sb = 16; 		// Number of significant bits displayed (incl overload)
	
	// Timings assume this function is called every 100mS

	const int ovldTime = 30;	// Hold time for overload indicator
	const int peakDecay = 15;	// Hold time for peak segment

	// - - - - - - - - - - - - - - - - - - - - - - - - - 

	int x = 0;
		
	if(inL > inR) {
		
		while(x < inR) line[x++] = '=';
		while(x < inL) line[x++] = 3;
		while(x < sb) line[x++] = 32;
		
		if(inL >= sb) ovldL = ovldTime; 
	}
		
	if(inR > inL) {
		
		while(x < inL) line[x++] = '=';
		while(x < inR) line[x++] = 4;
		while(x < sb) line[x++] = 32;
		
		if(inR >= sb) ovldR = ovldTime;
	}
		
	if(inR == inL) {
		
		while(x < inL) line[x++] = '=';
		while(x < sb) line[x++] = 32;
		
		if(inL >= sb) ovldL = ovldR = ovldTime;
	}

	if(peakL < inL) {

		peakL = inL;
		peakTimeL = peakDecay;
	}

	if(peakR < inR) {

		peakR = inR;
		peakTimeR = peakDecay;
	}

	if(peakL > inL) {
		
		line[peakL-1] = (peakL <= inR || peakL == peakR) ? '=' : 3;

		if(!peakTimeL--) {
			
			peakL = inL;
			peakTimeL = 0;
		}
	}
	
	if(peakR > inR) {
		
		line[peakR-1] = (peakR <= inL || peakR == peakL) ? '=' : 4;

		if(!peakTimeR--) { 

			peakR = inR;
			peakTimeR = 0;
		}
	}
	
	if(ovldL) { 
		
		line[sb-1] = ovldR ? '+' : 5; 
		ovldL--; 
	}
	
	if(ovldR) { 
		
		line[sb-1] = ovldL ? '+' : 6; 
		ovldR--; 
	}
		
	line[sb] = 0;

	return line;
}

bool EntityLevelMeter::isActive()   { return active; }

void EntityLevelMeter::deActivate() { active = false; }

void EntityLevelMeter::activate()   { active = true; }
	

