/********************************************************************************
 * E N T I T Y 									*
 ********************************************************************************
 * File: EntityMipsMode.cpp							*
 *                                                                         	*
 * Copyright (C) A.R.Hardie-Bick 2004-2005					*
 *										*
 * This program is free software; you can redistribute it and/or modify		*
 * it under the terms of the GNU General Public License as published by		*
 * the Free Software Foundation; either version 2 of the License, or (at	*
 * your option) any later version.						*
 *										*
 * This program is distributed in the hope that it will be useful, but		*
 * WITHOUT ANY WARRANTY; without even the implied warranty of			*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU		*
 * General Public License for more details.					*
 *										*
 * You should have received a copy of the GNU General Public License		*
 * along with this program; if not, write to the Free Software			*
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.			*
 *******************************************************************************/

#include "EntityMipsMode.h"

EntityMipsMode::EntityMipsMode(EntityPanel *panel_, EntityGlobal *global_) : EntityMode(panel_) { 

	global = global_;
	
	/* For periodically writing to SYNQ */	
	repeater = new EntityRepeat;
	repeater->create("SYNQ", "MIPP", 100);
}

Mode EntityMipsMode::enter() {

	repeater->activate(READ_DSP_MIPS, 1, 1000);
	sendMessage(synq, RESET_DSP_DEBUG_ADDRESS);
	debugCounter = 0;

	if(global->encrCtr1()) panel->setEncoderForModulation();

	processEvents();

	panel->setEncoderForData();
	repeater->clear();
	return rMode;
}

void EntityMipsMode::edit() { rMode = RETURN; }

void EntityMipsMode::partUp() { 

	sendMessage(synq, RESET_DSP_DEBUG_ADDRESS);
	debugCounter = 0;
}

void EntityMipsMode::partDown() { 

	panel->repeat(AUTO_GROUP_DOWN, 500, 50);
	sendMessage(synq, READ_DSP_DEBUG_DATA);
};

void EntityMipsMode::autoEvent(int event) {

	switch(event) {

		case AUTO_TOTAL_MIPS : displayTotalUpdate(); break;
		case AUTO_FRAG_MIPS : displayFragUpdate(); break;
		case AUTO_DEBUG_DATA : traceDebugData(); break;
		case AUTO_GROUP_DOWN : sendMessage(synq, READ_DSP_DEBUG_DATA); break;
	}
}

void EntityMipsMode::traceDebugData() {

	// (define the DSP source address and increment in dsp/main.asm
	// search for "Select memory space, address and increment")

	int data = ((mBuffer[1] << 16) & 0xff0000) | ((mBuffer[2] << 8) & 0xff00) | (mBuffer[3] & 0xff);
	float fdata = (float) data / (float) 0x7fffff;
	
	TRACE("count: %i\t value: $%.6X %i %f\n", debugCounter++, data, data, fdata);
}

void EntityMipsMode::displayTotalUpdate() {

	char 	line[PANEL_LCD_MAX_LINE_LEN + 7];
	
	int mips = getMipsFromMessage();

	sprintf(line, "%2i.%-.2i", mips / 100, mips % 100);
	panel->print(0, 11, line);
}

void EntityMipsMode::displayFragUpdate() {

	char 	line[PANEL_LCD_MAX_LINE_LEN + 7];
	
	int mips = getMipsFromMessage();

	sprintf(line, "%3i.%-.2i", mips / 100, mips % 100);
	panel->print(1, 10, line);
}

void EntityMipsMode::displayUpdate() {

	panel->print(0, 0, "MIPS total      ");
	panel->print(1, 0, "fragment        ");
}

int EntityMipsMode::getMipsFromMessage() {

	int c = ((unsigned char)mBuffer[1] << 16)
	      + ((unsigned char)mBuffer[2] << 8)
	      +  (unsigned char)mBuffer[3];

	float Count = (float) c;
	
	/* Counts are half number of dsp clock cycles used per n cycles */

	const float TotalCycles = DSP_CLOCK_RATE * DSP_N_CYCLES / DSP_SAMPLE_RATE;
	float Mips = (10000.0 * Count) / TotalCycles; 				// Total MIPS with 2dp

	return (int) Mips;
}
	

