/********************************************************************************
 * E N T I T Y 									*
 ********************************************************************************
 * File: EntityMode.cpp								*
 *                                                                         	*
 * Copyright (C) A.R.Hardie-Bick 2004-2005					*
 *										*
 * This program is free software; you can redistribute it and/or modify		*
 * it under the terms of the GNU General Public License as published by		*
 * the Free Software Foundation; either version 2 of the License, or (at	*
 * your option) any later version.						*
 *										*
 * This program is distributed in the hope that it will be useful, but		*
 * WITHOUT ANY WARRANTY; without even the implied warranty of			*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU		*
 * General Public License for more details.					*
 *										*
 * You should have received a copy of the GNU General Public License		*
 * along with this program; if not, write to the Free Software			*
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.			*
 *******************************************************************************/

#include "EntityMode.h"

int  EntityMode::editPending = 0;
int  EntityMode::shiftPending = 0;
bool EntityMode::ledBlink = 0;

EntityMode::EntityMode(EntityPanel *panel_) {

	panel = panel_;

	ctrq = getQueueId("CTRQ");
	synq = getQueueId("SYNQ");

	panel->clearLEDs(PANEL01_LED_SHIFT);
}

void EntityMode::processEvents() {

	panel->resetEncoder();
	
	encoderMoveTimeOut = 0;
	shiftPending = 0;
	editPending = 0;

	displayUpdate();
	
	do {	rMode = SELF;

		getMessage(ctrq);

		switch(mBuffer[0]) {

			case PANEL_ENCODER_DATA : 
				
				{	int i = mBuffer[2];
					if(i > 0) incrementEvent(i);
					else if(i < 0) decrementEvent(-i);

				} break;

			case PANEL_POTENTIOMETER :

				switch(mBuffer[1]) {

					case PANEL01_POT_VOLUME :	pot0Event(mBuffer[2]); break;
					case PANEL01_POT_CTRL1 :  	pot1Event(mBuffer[2]); break;
					case PANEL01_POT_CTRL2 :  	pot2Event(mBuffer[2]); break;
					case PANEL01_POT_CTRL3 :	pot3Event(mBuffer[2]); break;

				} break;						

			case PANEL_KEYPAD :

				buttonEvent();			// *** user-definable function

				panel->clearRepeat();
				keyStates = (mBuffer[4]<<24) | (mBuffer[5]<<16);
					
				if(mBuffer[1] == KEY_ON) {

					buttonOnEvent();	// *** user-definable function
				
					int key = (mBuffer[2]<<24) | (mBuffer[3]<<16);
				
					if(editPending) { 
						
						switch(key) {

							// EDIT + other key at same time
	
							case PANEL01_KEY_SHIFT : 	editShift(); break;
							case PANEL01_KEY_GROUP_UP : 	editGroupUp(); break;
							case PANEL01_KEY_GROUP_DOWN : 	editGroupDown(); break;
							case PANEL01_KEY_PAGE_UP : 	editPageUp(); break;
							case PANEL01_KEY_PAGE_DOWN : 	editPageDown(); break;
							case PANEL01_KEY_VALUE_UP :	editValueUp(); break;
							case PANEL01_KEY_VALUE_DOWN :	editValueDown(); break;
							case PANEL01_KEY_PART_UP : 	editPartUp(); break;		
							case PANEL01_KEY_PART_DOWN :	editPartDown(); break;
							case PANEL01_KEY_PARAM_UP :	editParamUp(); break;
							case PANEL01_KEY_PARAM_DOWN :	editParamDown(); break;
						}
					
						editPending = 0;
					}
					
					else if(shiftPending) { 
						
						switch(key) {

							// SHIFT + other key at same time
	
							case PANEL01_KEY_EDIT : 	shiftEdit(); break;
							case PANEL01_KEY_GROUP_UP : 	shiftGroupUp(); break;
							case PANEL01_KEY_GROUP_DOWN : 	shiftGroupDown(); break;
							case PANEL01_KEY_PAGE_UP : 	shiftPageUp(); break;
							case PANEL01_KEY_PAGE_DOWN : 	shiftPageDown(); break;
							case PANEL01_KEY_VALUE_UP :	shiftValueUp(); break;
							case PANEL01_KEY_VALUE_DOWN :	shiftValueDown(); break;
							case PANEL01_KEY_PART_UP : 	shiftPartUp(); break;		
							case PANEL01_KEY_PART_DOWN :	shiftPartDown(); break;
							case PANEL01_KEY_PARAM_UP :	shiftParamUp(); break;
							case PANEL01_KEY_PARAM_DOWN :	shiftParamDown(); break;
						}
					
						shiftPending++; // If any key was used with shift, inhibit shift latch 
					}

					else {	
						
						switch(key) {

							// EDIT and SHIFT keys activate on release (so they
							// can be used for simultaneous key combinations)
	
							case PANEL01_KEY_EDIT : 	editPending = 1;
										       	
											panel->repeat(
													
												EDIT_KEY_HOLD, 
												EDIT_KEY_HOLD_TIME); 
										
											break;
							
							case PANEL01_KEY_SHIFT : 	shiftPending = 1; 
											break;
										
							// Other keys activate immediately on press
	
							case PANEL01_KEY_GROUP_UP : 	groupUp(); break;
							case PANEL01_KEY_GROUP_DOWN : 	groupDown(); break;
							case PANEL01_KEY_PAGE_UP : 	pageUp(); break;
							case PANEL01_KEY_PAGE_DOWN : 	pageDown(); break;
							case PANEL01_KEY_VALUE_UP :	valueUp(); break;
							case PANEL01_KEY_VALUE_DOWN :	valueDown(); break;
							case PANEL01_KEY_PART_UP : 	partUp(); break;		
							case PANEL01_KEY_PART_DOWN :	partDown(); break;
							case PANEL01_KEY_PARAM_UP :	paramUp(); break;
							case PANEL01_KEY_PARAM_DOWN :	paramDown(); break;
						}
					}
				}

				else if(mBuffer[1] == KEY_OFF) {

					buttonOffEvent();	// *** general key-off function for derived classes
				
					int key = (mBuffer[2]<<24) | (mBuffer[3]<<16);
					
					switch(key) {

						case PANEL01_KEY_GROUP_UP :	groupUpKeyUp(); break;
						case PANEL01_KEY_GROUP_DOWN :	groupDownKeyUp(); break;
										
						case PANEL01_KEY_EDIT :		if(editPending) { editPending = 0; edit(); } break;
						case PANEL01_KEY_SHIFT :	if(shiftPending == 1) shift(); shiftPending = 0; break;
					}
				}

				break;

			case SYSEX_MODE : 	rMode = SYSEX; break;
			case SOUND_MODE : 	rMode = SOUND; break;
	
			case EDIT_KEY_HOLD :	panel->clearRepeat(); editHold(); editPending = 0; break;
						
			default : 		autoEvent(mBuffer[0]); break;

		}
		
	} while(rMode == SELF);
			
	panel->clearRepeat();
	panel->clearBlink();
	panel->clearDelay();
	panel->clearRequest();
}

void EntityMode::incrementEvent(int i) {

	decrementEnable = TRUE;	

	if(!incrementEnable) {

		rtems_clock_get(RTEMS_CLOCK_GET_TICKS_SINCE_BOOT, &ticks);
		timeInterval = ticks - oldTicks;
		oldTicks = ticks;				

		encoderMoveTimeOut -= (int) timeInterval;
		if(encoderMoveTimeOut < 0) incrementEnable = TRUE;
							
		if(timeInterval > (rtems_unsigned32) encoderStillTimeOut) {
			incrementEnable = TRUE;
		}
	}

	if(incrementEnable) {

		switch(incrementValue(i)) {

			case PARAMETER_CHANGED :

				displayValueUpdate();

				break;

			case PARAMETER_THRESHOLD :
							
				incrementEnable = FALSE;

				/* Time till reactivate if moving (510) */
				encoderMoveTimeOut = 510;

				/* Still time till reactivate (230) */
				encoderStillTimeOut = 230;
							
				rtems_clock_get(RTEMS_CLOCK_GET_TICKS_SINCE_BOOT, &oldTicks);

				displayValueUpdate();

				break;
								
			case PARAMETER_UNCHANGED :

				break;

			case PARAMETER_EXCEPTION : 
				
				parameterException();	// ie Matrix full

				break;
		}
	}	
}

void EntityMode::decrementEvent(int i) {

	incrementEnable = TRUE;

	if(!decrementEnable) {
						
		rtems_clock_get(RTEMS_CLOCK_GET_TICKS_SINCE_BOOT, &ticks);
		timeInterval = ticks - oldTicks;
		oldTicks = ticks;				

		encoderMoveTimeOut -= (int) timeInterval;
		if(encoderMoveTimeOut < 0) decrementEnable = TRUE;
							
		if(timeInterval > (rtems_unsigned32) encoderStillTimeOut) {
			decrementEnable = TRUE;
		}
	}	

	if(decrementEnable) {

		switch(incrementValue(-i)) {

			case PARAMETER_CHANGED :

				displayValueUpdate();

				break;

			case PARAMETER_THRESHOLD :
							
				decrementEnable = FALSE;

				/* Time till reactivate if moving (510) */
				encoderMoveTimeOut = 510;

				/* Still time till reactivate (230) */
				encoderStillTimeOut = 230;
							
				rtems_clock_get(RTEMS_CLOCK_GET_TICKS_SINCE_BOOT, &oldTicks);
				
				displayValueUpdate();

				break;
				
			case PARAMETER_UNCHANGED :

				break;
				
			case PARAMETER_EXCEPTION :

				parameterException();
				
				break;
		}
	}	
}

Status EntityMode::incrementValue(int) { 
	
	return PARAMETER_UNCHANGED; 
}

void EntityMode::displayValueUpdate() { }
void EntityMode::parameterException() { }
void EntityMode::buttonEvent() { }
void EntityMode::buttonOnEvent() { }
void EntityMode::buttonOffEvent() { }

void EntityMode::editHold() { rMode = SAVE; }

void EntityMode::edit()	{ rMode = RETURN; }

void EntityMode::shift() { }
void EntityMode::partUp() { rMode = RETURN; }
void EntityMode::partDown() { rMode = RETURN; }

void EntityMode::groupUp() { 

	sm[0] = NOTE_EVENT;
	sm[1] = 69;
	sm[2] = 90;
	
	sendMessage(synq, sm, 3); 
}

void EntityMode::groupUpKeyUp() { 
	
	sm[0] = NOTE_EVENT;
	sm[1] = 69;
	sm[2] = 0;
	
	sendMessage(synq, sm, 3); 
}

void EntityMode::groupDown() { 

	sm[0] = NOTE_EVENT;
	sm[1] = 40;
	sm[2] = 90;
	
	sendMessage(synq, sm, 3); 
}

void EntityMode::groupDownKeyUp() { 
	
	sm[0] = NOTE_EVENT;
	sm[1] = 40;
	sm[2] = 0;
	
	sendMessage(synq, sm, 3); 
}

void EntityMode::pageUp() { rMode = RETURN; }
void EntityMode::pageDown() { rMode = RETURN; }
void EntityMode::paramUp() { rMode = RETURN; }
void EntityMode::paramDown() { rMode = RETURN; }
void EntityMode::valueUp() { rMode = RETURN; }
void EntityMode::valueDown() { rMode = RETURN; }

void EntityMode::editShift() { }
void EntityMode::editPartUp() { }

void EntityMode::editPartDown() { rMode = MIPS; }

void EntityMode::editPageUp() { }
void EntityMode::editPageDown() { }	
void EntityMode::editGroupUp() { }
void EntityMode::editGroupDown() { }	
void EntityMode::editParamUp() { }
void EntityMode::editParamDown() { }
void EntityMode::editValueUp() { }
void EntityMode::editValueDown() { }

void EntityMode::shiftEdit() { rMode = GLOBAL; }

void EntityMode::shiftPartUp() { }
void EntityMode::shiftPartDown() { }
void EntityMode::shiftPageUp() { }
void EntityMode::shiftPageDown() { }	
void EntityMode::shiftGroupUp() { }
void EntityMode::shiftGroupDown() { }	
void EntityMode::shiftParamUp() { }
void EntityMode::shiftParamDown() { }
void EntityMode::shiftValueUp() { }
void EntityMode::shiftValueDown() { }

void EntityMode::pot0Event(int p) { }
void EntityMode::pot1Event(int p) { }
void EntityMode::pot2Event(int p) { }
void EntityMode::pot3Event(int p) { }

void EntityMode::autoEvent(int e) { }
Mode EntityMode::enter() { return SELF; }
