/********************************************************************************
 * E N T I T Y 									*
 ********************************************************************************
 * File: EntityMode.h								*
 *                                                                         	*
 * Copyright (C) A.R.Hardie-Bick 2004-2005					*
 *										*
 * This program is free software; you can redistribute it and/or modify		*
 * it under the terms of the GNU General Public License as published by		*
 * the Free Software Foundation; either version 2 of the License, or (at	*
 * your option) any later version.						*
 *										*
 * This program is distributed in the hope that it will be useful, but		*
 * WITHOUT ANY WARRANTY; without even the implied warranty of			*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU		*
 * General Public License for more details.					*
 *										*
 * You should have received a copy of the GNU General Public License		*
 * along with this program; if not, write to the Free Software			*
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.			*
 *******************************************************************************/

#ifndef __ENTITYMODE_H__
#define __ENTITYMODE_H__

#include <rtems.h>
#include <chameleon.h>
#include <panel.h>
#include <string.h>

#include "EntityPanel.h"
#include "EntityStatus.h"
#include "EntityMessageHandler.h"
#include "EntityDefines.h"

#define EDIT_KEY_HOLD_TIME 1000	// EDIT key hold time, typically for initiating save operation

enum Mode { SELF = 0, SOUND, EDIT, GLOBAL, SAVE, STACK, NUMBER, NAME, INPUT, OUTPUT, MIPS, SYSEX, SAVE_GLOBAL, RETURN };

class EntityMode : public EntityMessageHandler {
	
protected:

	EntityPanel*	panel;

	rtems_id	ctrq;	// For receiving events from the panel
	rtems_id	synq;	// For sending events to the synth
	char		sm[8];  // Synth message packet
	
	Mode		rMode;
	int		keyStates;

	static int	editPending;
	static int	shiftPending;
	static bool	ledBlink;
	
	rtems_interval	ticks;
	rtems_interval	oldTicks;
	rtems_interval	timeInterval;

	bool		incrementEnable;
	bool		decrementEnable;
	
	int		encoderMoveTimeOut;
	int		encoderStillTimeOut;

	/* These functions are defined in the base class */
	
	void		processEvents();
	void		incrementEvent(int=1);
	void		decrementEvent(int=1);

	/* The remaining functions are intended to be defined in derived classes */
	
	virtual void	displayUpdate() = 0;
	virtual void	displayValueUpdate();

	virtual Status	incrementValue(int);
	virtual void 	parameterException();

	virtual void	buttonEvent();
	virtual void	buttonOnEvent();
	virtual void	buttonOffEvent();

	virtual void	edit();
	virtual void	shift();
	virtual void	partUp();
	virtual void	partDown();
	virtual void	pageUp();
	virtual void	pageDown();
	virtual void	groupUp();
	virtual void	groupDown();
	virtual void	paramUp();
	virtual void	paramDown();
	virtual void	valueUp();
	virtual void	valueDown();
	
	virtual void	groupUpKeyUp();
	virtual void	groupDownKeyUp();
	
	virtual void	editShift();
	virtual void	editPartUp();
	virtual void	editPartDown();
	virtual void	editPageUp();
	virtual void	editPageDown();
	virtual void	editGroupUp();
	virtual void	editGroupDown();
	virtual void	editParamUp();
	virtual void	editParamDown();
	virtual void	editValueUp();
	virtual void	editValueDown();
	
	virtual void	shiftEdit();
	virtual void	shiftPartUp();
	virtual void	shiftPartDown();
	virtual void	shiftPageUp();
	virtual void	shiftPageDown();
	virtual void	shiftGroupUp();
	virtual void	shiftGroupDown();
	virtual void	shiftParamUp();
	virtual void	shiftParamDown();
	virtual void	shiftValueUp();
	virtual void	shiftValueDown();

	virtual void	editHold();	
	virtual void	autoEvent(int);

	virtual void	pot0Event(int);
	virtual void	pot1Event(int);
	virtual void	pot2Event(int);
	virtual void	pot3Event(int);
	
public:

	virtual Mode 	enter();
	EntityMode(EntityPanel*);
};

	
	
















#endif /* __ENTITYMODE_H__ */
