/********************************************************************************
 * E N T I T Y 									*
 ********************************************************************************
 * File: EntityPanel.h								*
 *                                                                         	*
 * Copyright (C) A.R.Hardie-Bick 2004-2005					*
 *										*
 * This program is free software; you can redistribute it and/or modify		*
 * it under the terms of the GNU General Public License as published by		*
 * the Free Software Foundation; either version 2 of the License, or (at	*
 * your option) any later version.						*
 *										*
 * This program is distributed in the hope that it will be useful, but		*
 * WITHOUT ANY WARRANTY; without even the implied warranty of			*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU		*
 * General Public License for more details.					*
 *										*
 * You should have received a copy of the GNU General Public License		*
 * along with this program; if not, write to the Free Software			*
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.			*
 *******************************************************************************/

#ifndef __ENTITYPANEL_H__
#define __ENTITYPANEL_H__

#include <rtems.h>
#include <chameleon.h>
#include <panel.h>
#include <math.h>

#include "EntityProcess.h"
#include "EntityRepeat.h"
#include "EntityMessages.h"

class EntityPanel : public EntityProcess {

	int			panelHandle;
	rtems_id		ctrq;
	rtems_id		synq;

	int			encoderAccumulator;
	int			acceleration;
	rtems_signed8		encoder;
	rtems_unsigned8		eidx;
	bool 			encoderMod;

	rtems_interval 		timeInterval;
	rtems_interval 		oldTicks;
	rtems_interval 		ticks;

	int			sigmoidSize;
	int			*sigmoid;

	bool			potMod[4];
	bool			potVal[4];
	rtems_unsigned8		potValue;
	rtems_unsigned8		pidx;

	char			pot3Assignment;
	
	rtems_unsigned32	keyBits;
	rtems_unsigned32	changedKeyBits;
	rtems_unsigned32	oldKeyBits;

	rtems_unsigned32	ledStates;

	EntityRepeat		*repeater1;
	EntityRepeat		*repeater2;
	EntityRepeat		*repeater3;
	EntityRepeat		*repeater4;
	
	// - - - - Encoder Debounce - - - - - 

	rtems_signed8		dbEncoder;
	rtems_interval 		dbTimeInterval;
	rtems_interval 		dbOldTicks;
	int			dbSign;
	int 			dbcps;

	// - - - - - - - - - - - - - - - - - 
	
public:

	EntityPanel();
	~EntityPanel();
	
	void 		Run();

	/* Panel output */
	
	bool		clear();			// Clear display
	bool		clearLine(int);			// Clear line 0 or 1
	bool		print(int, int, char*);		// Write to display
	
	bool		clearLEDs(int=0, int=0, int=0, int=0, int=0);
	bool		setLEDs(int=0, int=0, int=0, int=0, int=0);

	/* Selecting data or modulation for panel pots */
	
	void		setPotsForModulation(int, int=-1, int=-1, int=-1);
	void		setPotsForData(int, int=-1, int=-1, int=-1);
	void		setPotsForModulation();
	void		setPotsForData();
	void		setPotForModulation(int);
	void		setPotForData(int);

	/* For overriding Pot3 during fast edits with matrix source = CTR1 or 2 */

	void		transmitPot3EventsAs(char panelPot);
	
	/* For switching encoder between edit and fine control for CTR1 */
	
	void		setEncoderForData();
	void		setEncoderForModulation();
	
	/* Autorepeaters - general purpose, to put autorepeated events on queues */
	
	bool		request(char message, int delay, int repeat = 0);	// To put events on SYNQ
	bool		repeat(char message, int delay, int repeat = 0);	// To put events on CTRQ	
	bool		blink(char message, int delay, int repeat = 0);		//          "
	bool		delay(char message, int delay, int repeat = 0);		//          "  
	
	void		clearAllRepeaters();

	bool		clearRequest();
	bool		clearRepeat();					
	bool		clearBlink();
	bool		clearDelay();
	
	/* Request updates to set mode context */

	void		requestPotUpdates();
	void		requestPot1Update();

	/* Encoder behaviour */

	void		setEncoderAcceleration(int);	// arg is percentage
	void		resetEncoder();
};

#endif /* __ENTITYPANEL_H__ */
