/********************************************************************************
 * E N T I T Y 									*
 ********************************************************************************
 * File: EntityStackMode.cpp							*
 *                                                                         	*
 * Copyright (C) A.R.Hardie-Bick 2004-2005					*
 *										*
 * This program is free software; you can redistribute it and/or modify		*
 * it under the terms of the GNU General Public License as published by		*
 * the Free Software Foundation; either version 2 of the License, or (at	*
 * your option) any later version.						*
 *										*
 * This program is distributed in the hope that it will be useful, but		*
 * WITHOUT ANY WARRANTY; without even the implied warranty of			*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU		*
 * General Public License for more details.					*
 *										*
 * You should have received a copy of the GNU General Public License		*
 * along with this program; if not, write to the Free Software			*
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.			*
 *******************************************************************************/

#include "EntityStackMode.h"

EntityStackMode::EntityStackMode(EntityPanel *panel_, EntityGlobal *global_, ParaStructure *paras_) : EntityMode(panel_) { 

	panel = panel_;
	paras = paras_;
	global = global_;

	stackParameter = paras->getParaByName("stack");
}

Mode EntityStackMode::enter(EntitySound* sound_) {

	/* Set voice assigner appropriately for stack edit (polyphonic, sustain, 
	 * no stack) also echoing note-on events to CTRQ */
	sendMessage(synq, START_STACK_EDIT);
	
	sound = sound_;
	
	panel->setLEDs(PANEL01_LED_EDIT);
	panel->clearLEDs(PANEL01_LED_SHIFT, PANEL01_LED_CTRL1, PANEL01_LED_CTRL2, PANEL01_LED_CTRL3);

	noteCounter = 0;

	processEvents();

	/* Cancel echo for note-on events & reset voice assignment with possible newly defined stack */
	sendMessage(synq, END_STACK_EDIT);	
	
	return rMode;
}

void EntityStackMode::midiNoteOn() {

	if(noteCounter >= MAX_NOTE_STACK) return; // Ignore any extra notes

	if(!noteCounter) { // First note received is root note

		rootNote = mBuffer[1];	// Midi note number
		noteCounter++;
		
		// Clear existing stack, and display, for user feedback
		
		for(int x = 0; x < MAX_NOTE_STACK; x++) {
			
			sound->loadValue(stackParameter, CLEAR_STACK_NOTE, x);
		}
		
		displayUpdate();
		return;
	}

	/* Stack note received */

	int n = (int)mBuffer[1] - rootNote;
	sound->loadValue(stackParameter, n, noteCounter-1);
	displayStackNote(noteCounter-1);
	noteCounter++;

	/* If stack has new stacked notes, automatically 
	 * set the base parameter to "On" (sensible UI) */
	sound->changeValue(stackParameter, 1);
	
	return;	
}

void EntityStackMode::autoEvent(int event) {

	switch(event) {

		case MIDI_NOTE_ON: 	midiNoteOn(); break;

		default:		break;
	}
}
	
void EntityStackMode::displayUpdate() {

	panel->print(0, 0, "     STACK      ");
	panel->print(1, 0, "                ");

	for(int x = 0; x < MAX_NOTE_STACK-1; x++) {

		displayStackNote(x);
	}
}

void EntityStackMode::displayStackNote(int x) {

	char s[4];
	
	int t = sound->getValue(stackParameter, x);
	
	if(t == CLEAR_STACK_NOTE) sprintf(s, " --");
	else {
		sprintf(s, "%3d", t);

		if(t < 0) {	/* Use custom '-' sign */
		
			if(s[0] == '-') s[0] = 7;
			else if(s[1] == '-') s[1] = 7;
		}
	}
	
	panel->print(1, x * 3, s);
}

void EntityStackMode::displayValueUpdate() { }	

void EntityStackMode::partUp() { noteCounter = 0; }
void EntityStackMode::partDown() { noteCounter = 0; }

void EntityStackMode::groupUp() { }
void EntityStackMode::groupDown() { }

void EntityStackMode::pageUp() { }
void EntityStackMode::pageDown() { }

void EntityStackMode::paramUp() { }
void EntityStackMode::paramDown() { }

void EntityStackMode::valueUp() { }
void EntityStackMode::valueDown() { }

void EntityStackMode::edit() { rMode = EDIT; }

