/********************************************************************************
 * E N T I T Y 									*
 ********************************************************************************
 * File: EntitySynth.h								*
 *                                                                         	*
 * Copyright (C) A.R.Hardie-Bick 2004-2005					*
 *										*
 * This program is free software; you can redistribute it and/or modify		*
 * it under the terms of the GNU General Public License as published by		*
 * the Free Software Foundation; either version 2 of the License, or (at	*
 * your option) any later version.						*
 *										*
 * This program is distributed in the hope that it will be useful, but		*
 * WITHOUT ANY WARRANTY; without even the implied warranty of			*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU		*
 * General Public License for more details.					*
 *										*
 * You should have received a copy of the GNU General Public License		*
 * along with this program; if not, write to the Free Software			*
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.			*
 *******************************************************************************/

#ifndef __ENTITYSYNTH_H__
#define __ENTITYSYNTH_H__

#include <rtems.h>
#include <panel.h>
#include <math.h>

#include "EntityProcess.h"
#include "EntityMessages.h"
#include "EntityGlobal.h"
#include "EntitySound.h"
#include "EntityMidi.h"
#include "EntityDsp.h"
#include "EntityDefines.h"
#include "EntityVoiceAssigner.h"

class MeterLevel {

	char lut[4096];

public:
	
	MeterLevel();
	char convert(int);	
};

class EntitySynth : public EntityProcess {

	// Synth event vector LUT

	typedef 	void (EntitySynth::*Vector)();
	Vector		event[N_SYNTH_EVENT_TYPES];

	// Synth event vectors
	
	void		PanelPotentiometer();
	void		PanelEncoderMod();
	void		ParameterSelect();
	void		ParameterModify();
	void		MidiNoteOn();
	void		MidiNoteShift();
	void		MidiNoteOff();
	void		MidiSustain();
	void		MidiController();
	void		MidiPitch();
	void		MidiVolume();
	void		MidiPan();
	void		MidiExpression();
	void		MidiAllSoundOff();
	void		MidiAllNotesOff();
	void		MidiReset();
	void		GlobalTune();
	void		ReadInputLevels();
	void		ReadOutputLevels();
	void		ClearDSPLevels();
	void		NoteEvent();
	void		AllNotesOff();
	void		NewSoundReady();
	void		ReadDSPMIPS();
	void		ReadDSPDebugData();
	void		ResetDSPDebugAddress();
	void		MidiNRPNHiResUpdate();
	void		MidiNRPNLoResUpdate();
	void		MidiNRPNIncrement();
	void		MidiNRPNDecrement();
	void		ProgramChange();
	void		MidiClock();
	void		MidiLFOSyncSustainOn();
	void		MidiLFOSyncSustainOff();
	void		MidiLFOSyncPhase();
	
	void		ResetVoiceAssign();
	void		StartStackEdit();
	void		EndStackEdit();

	void		NullEvent();
	
	// Non-vector methods

	EntitySound		*getSoundFromStack();	// For transferring new sounds to dsp
	bool			moreSoundsOnStack();
	
	void			updateNRPN(Parameter*, int value, int source);
	void			incrementNRPN(Parameter*, int value, int source);
	void			updateDSPParaIndex(Parameter*, int source);
	
	// Other
	
	EntityVoiceAssigner 	*assign;
	EntityGlobal		*global;
	ParaStructure		*paras;
	EntityMidi		*midi;
	EntityDsp		*dsp;

	EntitySound		*activeSound;
	EntitySound		*pendingSound;
	
	Parameter		*paraForEdit;
	int			sourceForEdit;

	int			dspParaIndex;
	
	rtems_id		ctrq;
	rtems_id		synq;
	rtems_id		sstq;
	
	int			em;			// Encoder modulation (as CTR1)
	int			note;			// Latest midi note
	int			pan;			// Midi pan ( 14 bit value)
	int			volume;			// Midi volume (14 bit value)
	int			expression;		// Midi expression ( " )
	
	void			loadNewSound();		// Transfer new sound to dsp
	int			pitchShift(int);	// Translate midi pitchwheel values
	int			kflw(int midiNote);	// Calculate KFLW from note
	
	MeterLevel		meterLevel;		// Conversion for meter levels
	
public:

	EntitySynth(EntityGlobal*, ParaStructure*, EntityMidi*, EntityDsp*, EntityVoiceAssigner*);
	
	void 			Run();
};

#endif /* __ENTITYSYNTH_H__ */
