/********************************************************************************
 * E N T I T Y 									*
 ********************************************************************************
 * File: EntityVoiceAssigner.h							*
 *                                                                         	*
 * Copyright (C) A.R.Hardie-Bick 2004-2005					*
 *										*
 * This program is free software; you can redistribute it and/or modify		*
 * it under the terms of the GNU General Public License as published by		*
 * the Free Software Foundation; either version 2 of the License, or (at	*
 * your option) any later version.						*
 *										*
 * This program is distributed in the hope that it will be useful, but		*
 * WITHOUT ANY WARRANTY; without even the implied warranty of			*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU		*
 * General Public License for more details.					*
 *										*
 * You should have received a copy of the GNU General Public License		*
 * along with this program; if not, write to the Free Software			*
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.			*
 *******************************************************************************/

#ifndef __ENTITYVOICEASSIGNER_H__
#define __ENTITYVOICEASSIGNER_H__

#include "EntityDefines.h"
#include "EntityMessages.h"
#include "EntityGlobal.h"
#include "EntitySound.h"
#include "EntityDsp.h"

class VoiceListItem {

protected:

	VoiceListItem	*up;
	VoiceListItem	*down;

	signed char	voiceIndex;	// Index of voice if present on this machine, else -1
	signed char	note;		// Note to which voice is assigned

	signed char	VDIF;		// Precalculated voice difference
	signed char	STDF;		// Precalculated stack difference
	
public:

	inline void 	extract() { up->down = down; down->up = up; };

	inline void	insertAbove(VoiceListItem *here) { up = here->up; down = here; up->down = this; down->up = this; };
	inline void	insertBelow(VoiceListItem *here) { up = here; down = here->down; up->down = this; down->up = this; };

	inline void	moveAbove(VoiceListItem *here) { extract(); insertAbove(here); };
	inline void	moveBelow(VoiceListItem *here) { extract(); insertBelow(here); };

	inline void	setUp(VoiceListItem *up_) { up = up_; };
	inline void	setDown(VoiceListItem *down_) { down = down_; };
	
	inline VoiceListItem	*getUp() { return (VoiceListItem*) up; };
	inline VoiceListItem	*getDown() { return (VoiceListItem*) down; };

	VoiceListItem() 	{ up = 0; down = 0; };
	VoiceListItem(int i) 	{ up = 0; down = 0; voiceIndex = i; };

	/* All in one function for speed during assignment reset */
	void initialiseAbove(VoiceListItem *here_, signed char voiceIndex_, signed char VDIF_, signed char STDF_);

	/* Move voice stack, starting with "this", linked down to lastVoice, below "here" */
	void moveVoiceStackBelow(VoiceListItem *here, VoiceListItem *lastVoice);
	
	inline signed char 	getVoiceIndex() { return voiceIndex; };
	inline signed char 	setVoiceIndex(signed char i) { return voiceIndex = i; };
	inline signed char	getNote() { return note; };
	inline signed char	setNote(signed char n) { return note = n; };
	inline signed char	getVDIF() { return VDIF; };
	inline signed char	getSTDF() { return STDF; };
};

class NoteListItem {

protected:

	NoteListItem	*up;
	NoteListItem	*down;

	bool		on;		// true if note is played or sustained
	signed char	noteIndex;	// so note knows its own array index

	int		nVoices;	// Number of voices currently assigned to this note
	VoiceListItem	*voices;	// Top of linked list of voices assigned to this note
	
public:

	inline void 	extract() { up->down = down; down->up = up; };

	inline void	insertAbove(NoteListItem *here) { up = here->up; down = here; up->down = this; down->up = this; };
	inline void	insertBelow(NoteListItem *here) { up = here; down = here->down; up->down = this; down->up = this; };

	inline void	moveAbove(NoteListItem *here) { extract(); insertAbove(here); };
	inline void	moveBelow(NoteListItem *here) { extract(); insertBelow(here); };

	inline void	setUp(NoteListItem *up_) { up = up_; };
	inline void	setDown(NoteListItem *down_) { down = down_; };
	
	inline NoteListItem	*getUp() { return (NoteListItem*) up; };
	inline NoteListItem	*getDown() { return (NoteListItem*) down; };

	NoteListItem() 		{ up = 0; down = 0; };

	inline bool		getOn() { return on; };
	inline bool		setOn(bool s) { return on = s; }; 
	
	inline VoiceListItem*	getVoices() { return voices; };
	inline void		setVoices(int nVoices_, VoiceListItem *voices_) { nVoices = nVoices_; voices = voices_; };
	inline int		getnVoices() { return nVoices; };
	inline int		setnVoices(int n) { return nVoices = n; };
	inline signed char	setNoteIndex(signed char ni) { return noteIndex = ni; };
	inline signed char	getNoteIndex() { return noteIndex; };
};

enum DspWaitAction {

	No_wait_for_dsp,
	Wait_for_dsp	// On first note of new sound, wait for DSP to complete internal update
};
	
class EntityVoiceAssigner : public EntityMessageHandler {

protected:

	EntityDsp	*dsp;
	ParaStructure	*paras;
	EntityGlobal	*global;
	
	rtems_id	ctrq;
	
	/* The current sound, provided in the first note-on argument after a sound is selected 
	 * or its voice assignment properties have been changed (for example by editing) */

	EntitySound	*sound;		

	/* Parameters controlling voice assignment are stored in this context structure */

	struct Context {

		bool	polyphonic;	// True if polyphonic voice assignment required
		int	*stacklist;	// List of note transpositions for stacked voices
		int	nStacked;	// Minimum value is 1
		bool	echo;		// Echo note-on events to editor
		bool	sustain;	// Updated from midi (or always on in stackEdit)
	};

	Context		play;			// The normally active context for voice assignment
	Context		stackEdit;		// Context active when editing the voice stack
	Context 	*context;		// Pointer to either of the above
	Context		*contextPending;	// Context to switch to on next note-on (or null)
	
	// Unswitched context
	
	int		machineIndex;		// Index (ie, 0 or 1) of current machine
	int		nMachines;		// Number of cascaded machines
	
	// Voice linked lists
	
	VoiceListItem	*voices[MAX_VOICES];
	VoiceListItem	voices_[MAX_VOICES];
	
	VoiceListItem	*topOfVoicesOn,	*bottomOfVoicesOn, *topOfVoicesOff, *bottomOfVoicesOff;

	// Note linked lists
	
	NoteListItem	*notes[N_MIDI_NOTES];
	NoteListItem	notes_[N_MIDI_NOTES];
	
	NoteListItem	*topOfNotesOn, *bottomOfNotesOn, *topOfNotesOff, *bottomOfNotesOff;

	enum NoteAction {

		Note_on,	// Reset envelopes in voices assigned to note and apply new velocity
		Note_on_smooth,	// Restart envelopes, but don't zero their levels
		Note_shift	// Shift pitch of voices assigned to note
	};

	// For accessing parameters in a sound

	Parameter	*portamento; 	
	Parameter	*polyphonic;
	Parameter	*stack;

	/* For precalculating VDIF & STDF */
	signed char VDIFlist[MAX_VOICES];
	signed char STDFlist[MAX_VOICES];
	
	/* Reset all data structures according to context, setting all notes and voices off */
	void	reset(EntitySound* sound_);	
	
	/* Deassign any voices assigned to a note */
	void	voicesOff(NoteListItem* n);	
	
	/* New note event, specifying type of voice assignment (Note_on or Note_shift) */
	void 	newNote(int note, int velocity, NoteAction action);

	/* Note shift, for unlimited monophonic revert-to-most-recent-note */
	void	noteShift(int note);
	
	void	sustainReleasePolyphonic();	// Straightforward sustain release processing
	void	sustainReleaseMonophonic();	// Implements revert-to-most-recent-still-on

	inline void	notrace() { return; };
	void		trace();		// Debug tool
	
public:

	/* Update assignment parameters from sound, reset assignment, assign note */
	void 	noteOn(int note, int velocity, EntitySound *sound, DspWaitAction = No_wait_for_dsp);
	
	/* Assign voices to note-on, selecting voices according to portamento (for polyphonic portamento) */
	void 	noteOn(int note, int velocity);	
	
	void	noteOff(int note);		// Put note's voices in release (unless sustained)
	void	allNotesOff();			// Put all notes in release phase
	void	allSoundOff();			// Mute all voices
	bool	sustain(bool s);		// set/clear sustain

	/* Cause assigner to reset on next note-on event. This is used when editing an assignment parameter,
	 * such as "polyphonic", "stack" (the basic on/off para), and also the global cascade parameter */
	void	resetVoiceAssign();	

	/* Start/end stack edit, in which sustain and no stacking are enforced, and note-ons are echoed to editor */
	void	startStackEdit();
	void	endStackEdit();	
	
	EntityVoiceAssigner(EntityGlobal*, ParaStructure*, EntityDsp*);
};

#endif /* __ENTITYVOICEASSIGNER_H__ */
