;*******************************************************************************
; C H A M E L E O N    S. D. K.                                                *
;*******************************************************************************
;  $Archive:: /Chameleon.sdk/include/dsp/dsp_equ.asm                           $
;     $Date:: 14/05/02 17:10                                                   $
; $Revision:: 5                                                                $
;------------------------------------------------------------------------------*
; This file is part of the Chameleon Software Development Kit                  *
;                                                                              *
; Copyright (C) 2001-2002 Soundart                                             *
; www.soundart-hot.com                                                         *
; support@soundart-hot.com                                                     *
;*******************************************************************************


;*****************************************************************************
;
;	EQUATES for interrupts
;
;*****************************************************************************

intequ  ident   1,0

	if	@DEF(IntVecBase)
	;leave user definition as is.
	else
IntVecBase	EQU	$0
	endif

;------------------------------------------------------------------------
; Non-Maskable interrupts
;------------------------------------------------------------------------

VecHwReset		EQU	IntVecBase+$00   ; Hardware RESET
VecStackErr		EQU	IntVecBase+$02   ; Stack Error
VecIllegalInst		EQU	IntVecBase+$04   ; Illegal Instruction
VecDebug		EQU	IntVecBase+$06   ; Debug Request	 
VecTrap			EQU	IntVecBase+$08   ; Trap
VecNonMaskableInt	EQU  	IntVecBase+$0A   ; Non Maskable Interrupt

;------------------------------------------------------------------------
; Interrupt Request Pins
;------------------------------------------------------------------------

VecIrqA			EQU	IntVecBase+$10   ; IRQA
VecIrqB			EQU	IntVecBase+$12   ; IRQB
VecIrqC			EQU	IntVecBase+$14   ; IRQC
VecIrqD			EQU	IntVecBase+$16   ; IRQD

;------------------------------------------------------------------------
; DMA Interrupts
;------------------------------------------------------------------------

VecDma0			EQU	IntVecBase+$18   ; DMA Channel 0
VecDma1			EQU	IntVecBase+$1A   ; DMA Channel 1
VecDma2			EQU	IntVecBase+$1C   ; DMA Channel 2
VecDma3			EQU	IntVecBase+$1E   ; DMA Channel 3
VecDma4			EQU	IntVecBase+$20   ; DMA Channel 4
VecDma5			EQU	IntVecBase+$22   ; DMA Channel 5

;------------------------------------------------------------------------
; Timer Interrupts
;------------------------------------------------------------------------

VecTimer0Comp		EQU	IntVecBase+$24   ; TIMER 0 compare
VecTimer0Ofl		EQU	IntVecBase+$26   ; TIMER 0 overflow
VecTimer1Comp		EQU	IntVecBase+$28   ; TIMER 1 compare
VecTimer1Ofl		EQU	IntVecBase+$2A   ; TIMER 1 overflow
VecTimer2comp		EQU	IntVecBase+$2C   ; TIMER 2 compare
VecTimer2Ofl		EQU	IntVecBase+$2E   ; TIMER 2 overflow

;------------------------------------------------------------------------
; ESSI Interrupts
;------------------------------------------------------------------------

VecEssi0RxData		EQU	IntVecBase+$30   ; ESSI0 Receive Data
VecEssi0RxExSt		EQU	IntVecBase+$32   ; ESSI0 Receive Data With Exception Status
VecEssi0RxLastSlot	EQU	IntVecBase+$34   ; ESSI0 Receive last slot
VecEssi0TxData		EQU	IntVecBase+$36   ; ESSI0 Transmit data
VecEssi0TxExSt		EQU	IntVecBase+$38   ; ESSI0 Transmit Data With Exception Status
VecEssi0TxLastSlot	EQU	IntVecBase+$3A   ; ESSI0 Transmit last slot
VecEssi1RxData		EQU	IntVecBase+$40   ; ESSI1 Receive Data
VecEssi1RxExSt		EQU	IntVecBase+$42   ; ESSI1 Receive Data With Exception Status
VecEssi1RxLastSlot	EQU	IntVecBase+$44   ; ESSI1 Receive last slot
VecEssi1TxData		EQU	IntVecBase+$46   ; ESSI1 Transmit data
VecEssi1TxExSt		EQU	IntVecBase+$48   ; ESSI1 Transmit Data With Exception Status
VecEssi1TxLastSlot	EQU	IntVecBase+$4A   ; ESSI1 Transmit last slot

;------------------------------------------------------------------------
; SCI Interrupts
;------------------------------------------------------------------------

VecSciRxData		EQU	IntVecBase+$50   ; SCI Receive Data 
VecSciRxExSt		EQU	IntVecBase+$52   ; SCI Receive Data With Exception Status
VecSciTxData		EQU	IntVecBase+$54   ; SCI Transmit Data
VecSciIdleLine		EQU	IntVecBase+$56   ; SCI Idle Line
VecSciTimer		EQU	IntVecBase+$58   ; SCI Timer

;------------------------------------------------------------------------
; HOST Interrupts
;------------------------------------------------------------------------

VecHostRxDataFull	EQU	IntVecBase+$60   ; Host Receive Data Full
VecHostTxDataEmpty	EQU	IntVecBase+$62   ; Host Transmit Data Empty
VecHostCommandDefault	EQU	IntVecBase+$64   ; Default Host Command

;------------------------------------------------------------------------
; INTERRUPT ENDING ADDRESS
;------------------------------------------------------------------------

VecIntEndAddr		EQU	IntVecBase+$FF   ; last address of interrupt vector space



;*****************************************************************************
;
;	EQUATES for I/O registers and ports
;
;*****************************************************************************


;------------------------------------------------------------------------
;
;	EQUATES for I/O Port Programming
;
;------------------------------------------------------------------------

;	Register Addresses

HDDR		EQU	$FFFFC9		; Host port GPIO direction Register
HDR		EQU	$FFFFC8		; Host port GPIO data Register
PCRC		EQU	$FFFFBF		; Port C Control Register
PRRC		EQU	$FFFFBE		; Port C Direction Register
PDRC		EQU	$FFFFBD		; Port C GPIO Data Register
PCRD		EQU	$FFFFAF		; Port D Control register
PRRD		EQU	$FFFFAE		; Port D Direction Data Register
PDRD		EQU	$FFFFAD		; Port D GPIO Data Register
PCRE		EQU	$FFFF9F		; Port E Control register
PRRE		EQU	$FFFF9E		; Port E Direction Register
PDRE		EQU	$FFFF9D		; Port E Data Register
OGDB		EQU	$FFFFFC		; OnCE GDB Register 

;------------------------------------------------------------------------
;
;	EQUATES for Host Interface
;
;------------------------------------------------------------------------

;	Register Addresses
 
HCR		EQU	$FFFFC2		; Host Control Register
HSR		EQU	$FFFFC3		; Host Status Register
HPCR		EQU	$FFFFC4		; Host Polarity Control Register
HBAR		EQU	$FFFFC5		; Host Base Address Register
HRX		EQU	$FFFFC6		; Host Receive Register
HTX		EQU	$FFFFC7		; Host Transmit Register

;	HCR bits definition

HCR_HRIE	EQU	$0		; Host Receive interrupts Enable
HCR_HTIE	EQU	$1		; Host Transmit Interrupt Enable
HCR_HCIE	EQU	$2		; Host Command Interrupt Enable
HCR_HF2		EQU	$3		; Host Flag 2
HCR_HF3		EQU	$4		; Host Flag 3

;	HSR bits definition

HSR_HRDF	EQU	$0		; Host Receive Data Full
HSR_HTDE	EQU	$1		; Host Receive Data Empty
HSR_HCP		EQU	$2		; Host Command Pending
HSR_HF0		EQU	$3		; Host Flag 0
HSR_HF1		EQU	$4		; Host Flag 1

;	HPCR bits definition

HPCR_HGEN	EQU	$0		; Host Port GPIO Enable
HPCR_HA8EN	EQU	$1		; Host Address 8 Enable
HPCR_HA9EN	EQU	$2		; Host Address 9 Enable
HPCR_HCSEN	EQU	$3		; Host Chip Select Enable
HPCR_HREN	EQU	$4		; Host Request Enable
HPCR_HAEN	EQU	$5		; Host Acknowledge Enable
HPCR_HEN	EQU	$6		; Host Enable
HPCR_HOD	EQU	$8		; Host Request Open Drain mode
HPCR_HDSP	EQU	$9		; Host Data Strobe Polarity
HPCR_HASP	EQU	$A		; Host Address Strobe Polarity
HPCR_HMUX	EQU	$B		; Host Multiplexed bus select
HPCR_HD_HS	EQU	$C		; Host Double/Single Strobe select
HPCR_HCSP	EQU	$D		; Host Chip Select Polarity
HPCR_HRP	EQU	$E		; Host Request Polarity
HPCR_HAP	EQU	$F		; Host Acknowledge Polarity


;------------------------------------------------------------------------
;
;	EQUATES for Serial Communications Interface (SCI)
;
;------------------------------------------------------------------------

;	Register Addresses

STXH		EQU	$FFFF97		; SCI Transmit Data Register (high)
STXM		EQU	$FFFF96		; SCI Transmit Data Register (middle)
STXL		EQU	$FFFF95		; SCI Transmit Data Register (low)
SRXH		EQU	$FFFF9A		; SCI Receive Data Register (high)
SRXM		EQU	$FFFF99		; SCI Receive Data Register (middle)
SRXL		EQU	$FFFF98		; SCI Receive Data Register (low)
STXA		EQU	$FFFF94		; SCI Transmit Address Register
SCR		EQU	$FFFF9C		; SCI Control Register
SSR		EQU	$FFFF93		; SCI Status Register
SCCR		EQU	$FFFF9B		; SCI Clock Control Register

;	SCI Control Register Bit Flags

SCR_WDS		EQU	$7		; Word Select Mask (WDS0-WDS3)
SCR_WDS0	EQU	0		; Word Select 0
SCR_WDS1	EQU	1		; Word Select 1
SCR_WDS2	EQU	2		; Word Select 2
SCR_SSFTD	EQU	3		; SCI Shift Direction 
SCR_SBK		EQU	4		; Send Break
SCR_WAKE	EQU	5		; Wakeup Mode Select
SCR_RWU		EQU	6		; Receiver Wakeup Enable
SCR_WOMS	EQU	7		; Wired-OR Mode Select
SCR_SCRE	EQU	8		; SCI Receiver Enable
SCR_SCTE	EQU	9		; SCI Transmitter Enable
SCR_ILIE	EQU	10		; Idle Line Interrupt Enable
SCR_SCRIE	EQU	11		; SCI Receive Interrupt Enable
SCR_SCTIE	EQU	12		; SCI Transmit Interrupt Enable
SCR_TMIE	EQU	13		; Timer Interrupt Enable
SCR_TIR		EQU	14		; Timer Interrupt Rate
SCR_SCKP	EQU	15		; SCI Clock Polarity
SCR_REIE	EQU	16		; SCI Error Interrupt Enable (REIE)

;	SCI Status Register Bit Flags

SSR_TRNE	EQU	0		; Transmitter Empty
SSR_TDRE	EQU	1		; Transmit Data Register Empty
SSR_RDRF	EQU	2		; Receive Data Register Full
SSR_IDLE	EQU	3		; Idle Line Flag
SSR_OR		EQU	4		; Overrun Error Flag 
SSR_PE		EQU	5		; Parity Error
SSR_FE		EQU	6		; Framing Error Flag
SSR_R8		EQU	7		; Received Bit 8 (R8) Address

;	SCI Clock Control Register 

SCCR_CD		EQU	$FFF		; Clock Divider Mask (CD0-CD11)
SCCR_COD	EQU	12		; Clock Out Divider
SCCR_SCP	EQU	13		; Clock Prescaler
SCCR_RCM	EQU	14		; Receive Clock Mode Source Bit
SCCR_TCM	EQU	15		; Transmit Clock Source Bit

;------------------------------------------------------------------------
;
;	EQUATES for Synchronous Serial Interface (SSI)
;
;------------------------------------------------------------------------

;	Register Addresses Of SSI0 

TX00		EQU	$FFFFBC		; SSI0 Transmit Data Register 0
TX01		EQU	$FFFFBB		; SSI0 Transmit Data Register 1
TX02		EQU	$FFFFBA		; SSI0 Transmit Data Register 2
TSR0		EQU	$FFFFB9		; SSI0 Time Slot Register
RX0		EQU	$FFFFB8		; SSI0 Receive Data Register
SSISR0		EQU	$FFFFB7		; SSI0 Status Register
CRB0		EQU	$FFFFB6		; SSI0 Control Register B
CRA0		EQU	$FFFFB5		; SSI0 Control Register A
TSMA0		EQU	$FFFFB4		; SSI0 Transmit Slot Mask Register A
TSMB0		EQU	$FFFFB3		; SSI0 Transmit Slot Mask Register B
RSMA0		EQU	$FFFFB2		; SSI0 Receive Slot Mask Register A
RSMB0		EQU	$FFFFB1		; SSI0 Receive Slot Mask Register B

;	Register Addresses Of SSI1								

TX10		EQU	$FFFFAC		; SSI1 Transmit Data Register 0
TX11		EQU	$FFFFAB		; SSI1 Transmit Data Register 1
TX12		EQU	$FFFFAA		; SSI1 Transmit Data Register 2
TSR1		EQU	$FFFFA9		; SSI1 Time Slot Register
RX1		EQU	$FFFFA8		; SSI1 Receive Data Register
SSISR1		EQU	$FFFFA7		; SSI1 Status Register
CRB1		EQU	$FFFFA6		; SSI1 Control Register B
CRA1		EQU	$FFFFA5		; SSI1 Control Register A
TSMA1		EQU	$FFFFA4		; SSI1 Transmit Slot Mask Register A
TSMB1		EQU	$FFFFA3		; SSI1 Transmit Slot Mask Register B
RSMA1		EQU	$FFFFA2		; SSI1 Receive Slot Mask Register A
RSMB1		EQU	$FFFFA1		; SSI1 Receive Slot Mask Register B

;	SSI Control Register A Bit Flags

CRA_PM		EQU	$FF		; Prescale Modulus Select Mask (PM0-PM7)
CRA_PSR		EQU	11		; Prescaler Range
CRA_DC		EQU	$1F000		; Frame Rate Divider Control Mask (DC0-DC7)
CRA_ALC		EQU	18		; Alignment Control (ALC)
CRA_WL		EQU	$380000		; Word Length Control Mask (WL0-WL7)
CRA_SSC1	EQU	22		; Select SC1 as TR #0 drive enable (SSC1)

;	SSI Control Register B Bit Flags							

CRB_OF		EQU	$3		; Serial Output Flag Mask
CRB_OF0		EQU	0		; Serial Output Flag 0				 
CRB_OF1		EQU	1		; Serial Output Flag 1				 
CRB_SCD		EQU	$1C		; Serial Control Direction Mask
CRB_SCD0	EQU	2		; Serial Control 0 Direction			 
CRB_SCD1	EQU	3		; Serial Control 1 Direction			
CRB_SCD2	EQU	4		; Serial Control 2 Direction			
CRB_SCKD	EQU	5		; Clock Source Direction
CRB_SHFD	EQU	6		; Shift Direction					 
CRB_FSL		EQU	$180		; Frame Sync Length Mask (FSL0-FSL1)
CRB_FSL0	EQU	7		; Frame Sync Length 0
CRB_FSL1	EQU	8		; Frame Sync Length 1
CRB_FSR		EQU	9		; Frame Sync Relative Timing
CRB_FSP		EQU	10		; Frame Sync Polarity
CRB_CKP		EQU	11		; Clock Polarity
CRB_SYN		EQU	12		; Sync/Async Control 
CRB_MOD		EQU	13		; SSI Mode Select
CRB_TE		EQU	$1C000		; SSI Transmit enable Mask 
CRB_TE2		EQU	14		; SSI Transmit #2 Enable
CRB_TE1		EQU	15		; SSI Transmit #1 Enable				
CRB_TE0		EQU	16		; SSI Transmit #0 Enable				
CRB_RE		EQU	17		; SSI Receive Enable 
CRB_TIE		EQU	18		; SSI Transmit Interrupt Enable
CRB_RIE		EQU	19		; SSI Receive Interrupt Enable
CRB_TLIE	EQU	20		; SSI Transmit Last Slot Interrupt Enable 
CRB_RLIE	EQU	21		; SSI Receive Last Slot Interrupt Enable 
CRB_TEIE	EQU	22		; SSI Transmit Error Interrupt Enable 
CRB_REIE	EQU	23		; SSI Receive Error Interrupt Enable

;	SSI Status Register Bit Flags

SSISR_IF	EQU	$3		; Serial Input Flag Mask		 
SSISR_IF0	EQU	0		; Serial Input Flag 0
SSISR_IF1	EQU	1		; Serial Input Flag 1
SSISR_TFS	EQU	2		; Transmit Frame Sync Flag
SSISR_RFS	EQU	3		; Receive Frame Sync Flag 
SSISR_TUE	EQU	4		; Transmitter Underrun Error FLag		
SSISR_ROE	EQU	5		; Receiver Overrun Error Flag
SSISR_TDE	EQU	6		; Transmit Data Register Empty 
SSISR_RDF	EQU	7		; Receive Data Register Full

;	SSI Transmit Slot Mask Register A

TSMA_SSTSA	EQU	$FFFF		; SSI Transmit Slot Bits Mask A (TS0-TS15)

;	SSI Transmit Slot Mask Register B

TSMB_SSTSB	EQU	$FFFF		; SSI Transmit Slot Bits Mask B (TS16-TS31)

;	SSI Receive Slot Mask Register A

RSMA_SSRSA0	EQU	$FFFF		; SSI Receive Slot Bits Mask A (RS0-RS15)
 
;	SSI Receive Slot Mask Register B

RSMB_SSRSB	EQU	$FFFF		; SSI Receive Slot Bits Mask B (RS16-RS31)

;------------------------------------------------------------------------
;
;	EQUATES for Exception Processing							 
;
;------------------------------------------------------------------------

;	Register Addresses

IPRC		EQU	$FFFFFF		; Interrupt Priority Register Core
IPRP		EQU	$FFFFFE		; Interrupt Priority Register Peripheral

;	Interrupt Priority Register Core (IPRC) 

IPRC_IAL	EQU	$7		; IRQA Mode Mask
IPRC_IAL0	EQU	0		; IRQA Mode Interrupt Priority Level (low)
IPRC_IAL1	EQU	1		; IRQA Mode Interrupt Priority Level (high)
IPRC_IAL2	EQU	2		; IRQA Mode Trigger Mode
IPRC_IBL	EQU	$38		; IRQB Mode Mask
IPRC_IBL0	EQU	3		; IRQB Mode Interrupt Priority Level (low)
IPRC_IBL1	EQU	4		; IRQB Mode Interrupt Priority Level (high)
IPRC_IBL2	EQU	5		; IRQB Mode Trigger Mode
IPRC_ICL	EQU	$1C0		; IRQC Mode Mask
IPRC_ICL0	EQU	6		; IRQC Mode Interrupt Priority Level (low)
IPRC_ICL1	EQU	7		; IRQC Mode Interrupt Priority Level (high)
IPRC_ICL2	EQU	8		; IRQC Mode Trigger Mode
IPRC_IDL	EQU	$E00		; IRQD Mode Mask
IPRC_IDL0	EQU	9		; IRQD Mode Interrupt Priority Level (low)
IPRC_IDL1	EQU	10		; IRQD Mode Interrupt Priority Level (high)
IPRC_IDL2	EQU	11		; IRQD Mode Trigger Mode
IPRC_D0L	EQU	$3000		; DMA0 Interrupt priority Level Mask
IPRC_D0L0	EQU	12		; DMA0 Interrupt Priority Level (low)
IPRC_D0L1	EQU	13		; DMA0 Interrupt Priority Level (high)
IPRC_D1L	EQU	$C000		; DMA1 Interrupt Priority Level Mask
IPRC_D1L0	EQU	14		; DMA1 Interrupt Priority Level (low)
IPRC_D1L1	EQU	15		; DMA1 Interrupt Priority Level (high)
IPRC_D2L	EQU	$30000		; DMA2 Interrupt priority Level Mask
IPRC_D2L0	EQU	16		; DMA2 Interrupt Priority Level (low)
IPRC_D2L1	EQU	17		; DMA2 Interrupt Priority Level (high)
IPRC_D3L	EQU	$C0000		; DMA3 Interrupt Priority Level Mask
IPRC_D3L0	EQU	18		; DMA3 Interrupt Priority Level (low)
IPRC_D3L1	EQU	19		; DMA3 Interrupt Priority Level (high)
IPRC_D4L	EQU	$300000		; DMA4 Interrupt priority Level Mask
IPRC_D4L0	EQU	20		; DMA4 Interrupt Priority Level (low)
IPRC_D4L1	EQU	21		; DMA4 Interrupt Priority Level (high)
IPRC_D5L	EQU	$C00000		; DMA5 Interrupt priority Level Mask
IPRC_D5L0	EQU	22		; DMA5 Interrupt Priority Level (low)
IPRC_D5L1	EQU	23		; DMA5 Interrupt Priority Level (high)

;	Interrupt Priority Register Peripheral (IPRP)

IPRP_HPL	EQU	$3		; Host Interrupt Priority Level Mask
IPRP_HPL0	EQU	0		; Host Interrupt Priority Level (low)
IPRP_HPL1	EQU	1		; Host Interrupt Priority Level (high)
IPRP_S0L	EQU	$C		; SSI0 Interrupt Priority Level Mask
IPRP_S0L0	EQU	2		; SSI0 Interrupt Priority Level (low)
IPRP_S0L1	EQU	3		; SSI0 Interrupt Priority Level (high)
IPRP_S1L	EQU	$30		; SSI1 Interrupt Priority Level Mask
IPRP_S1L0	EQU	4		; SSI1 Interrupt Priority Level (low)
IPRP_S1L1	EQU	5		; SSI1 Interrupt Priority Level (high)
IPRP_SCL	EQU	$C0		; SCI  Interrupt Priority Level  Mask
IPRP_SCL0	EQU	6		; SCI  Interrupt Priority Level  (low)
IPRP_SCL1	EQU	7		; SCI  Interrupt Priority Level  (high)
IPRP_T0L	EQU	$300		; TIMER Interrupt Priority Level Mask
IPRP_T0L0	EQU	8		; TIMER Interrupt Priority Level (low)
IPRP_T0L1	EQU	9		; TIMER Interrupt Priority Level (high)
IPRP_EPL	EQU	$C00		; EFCOP Interrupt Priority Level Mask
IPRP_EPL0	EQU	10		; EFCOP Interrupt Priority Level (low)
IPRP_EPL1	EQU	11		; EFCOP Interrupt Priority Level (high)



;------------------------------------------------------------------------
;
;	EQUATES for TIMER
;
;------------------------------------------------------------------------


;	Register Addresses Of TIMER 0

TCSR0		EQU	$FFFF8F		; TIMER0 Control/Status Register
TLR0		EQU	$FFFF8E		; TIMER0 Load Reg 
TCPR0		EQU	$FFFF8D		; TIMER0 Compare Register
TCR0		EQU	$FFFF8C		; TIMER0 Count Register

;	Register Addresses Of TIMER 1

TCSR1		EQU	$FFFF8B		; TIMER1 Control/Status Register
TLR1		EQU	$FFFF8A		; TIMER1 Load Reg 
TCPR1		EQU	$FFFF89		; TIMER1 Compare Register
TCR1		EQU	$FFFF88		; TIMER1 Count Register

;	Register Addresses Of TIMER 2

TCSR2		EQU	$FFFF87		; TIMER2 Control/Status Register
TLR2		EQU	$FFFF86		; TIMER2 Load Reg 
TCPR2		EQU	$FFFF85		; TIMER2 Compare Register
TCR2		EQU	$FFFF84		; TIMER2 Count Register

;	Register Addresses Of TIMER Prescaler

TPLR		EQU	$FFFF83		; TIMER Prescaler Load Register
TPCR		EQU	$FFFF82		; TIMER Prescaler Count Register


;	Timer Prescaler Register Bit Flags								

TPLR_PS		EQU	$600000		; Prescaler Source Mask
TPLR_PS0	EQU	21
TPLR_PS1	EQU	22


;	Timer Control/Status Register Bit Flags								

TCSR_TE		EQU	0		; Timer Enable 
TCSR_TOIE	EQU	1		; Timer Overflow Interrupt Enable
TCSR_TCIE	EQU	2		; Timer Compare Interrupt Enable
TCSR_TC		EQU	$F0		; Timer Control Mask (TC0-TC3)
TCSR_INV	EQU	8		; Inverter Bit
TCSR_TRM	EQU	9		; Timer Restart Mode 
TCSR_DIR	EQU	11		; Direction Bit
TCSR_DI		EQU	12		; Data Input
TCSR_DO		EQU	13		; Data Output
TCSR_PCE	EQU	15		; Prescaled Clock Enable
TCSR_TOF	EQU	20		; Timer Overflow Flag
TCSR_TCF	EQU	21		; Timer Compare Flag 

;	Timer Control Bits

TCSR_TC0	EQU	4		; Timer Control 0
TCSR_TC1	EQU	5		; Timer Control 1
TCSR_TC2	EQU	6		; Timer Control 2
TCSR_TC3	EQU	7		; Timer Control 3

;------------------------------------------------------------------------
;
;	EQUATES for Direct Memory Access (DMA) 
;
;------------------------------------------------------------------------

;	Register Addresses Of DMA

DSTR		EQU	$FFFFF4		; DMA Status Register
DOR0		EQU	$FFFFF3		; DMA Offset Register 0
DOR1		EQU	$FFFFF2		; DMA Offset Register 1
DOR2		EQU	$FFFFF1		; DMA Offset Register 2
DOR3		EQU	$FFFFF0		; DMA Offset Register 3

;	Register Addresses Of DMA0

DSR0		EQU	$FFFFEF		; DMA0 Source Address Register
DDR0		EQU	$FFFFEE		; DMA0 Destination Address Register 
DCO0		EQU	$FFFFED		; DMA0 Counter
DCR0		EQU	$FFFFEC		; DMA0 Control Register 

;	Register Addresses Of DMA1

DSR1		EQU	$FFFFEB		; DMA1 Source Address Register
DDR1		EQU	$FFFFEA		; DMA1 Destination Address Register 
DCO1		EQU	$FFFFE9		; DMA1 Counter
DCR1		EQU	$FFFFE8		; DMA1 Control Register

;	Register Addresses Of DMA2

DSR2		EQU	$FFFFE7		; DMA2 Source Address Register
DDR2		EQU	$FFFFE6		; DMA2 Destination Address Register 
DCO2		EQU	$FFFFE5		; DMA2 Counter
DCR2		EQU	$FFFFE4		; DMA2 Control Register

;	Register Addresses Of DMA3

DSR3		EQU	$FFFFE3		; DMA3 Source Address Register
DDR3		EQU	$FFFFE2		; DMA3 Destination Address Register 
DCO3		EQU	$FFFFE1		; DMA3 Counter
DCR3		EQU	$FFFFE0		; DMA3 Control Register

;	Register Addresses Of DMA4

DSR4		EQU	$FFFFDF		; DMA4 Source Address Register
DDR4		EQU	$FFFFDE		; DMA4 Destination Address Register 
DCO4		EQU	$FFFFDD		; DMA4 Counter
DCR4		EQU	$FFFFDC		; DMA4 Control Register 

;	Register Addresses Of DMA5

DSR5		EQU	$FFFFDB		; DMA5 Source Address Register
DDR5		EQU	$FFFFDA		; DMA5 Destination Address Register 
DCO5		EQU	$FFFFD9		; DMA5 Counter
DCR5		EQU	$FFFFD8		; DMA5 Control Register


;	DMA Status Register bits

DSTR_DTD	EQU	$3F		; Channel Transfer Done Status MASK 
DSTR_DTD0	EQU	0		; DMA Channel Transfer Done Status 0
DSTR_DTD1	EQU	1		; DMA Channel Transfer Done Status 1
DSTR_DTD2	EQU	2		; DMA Channel Transfer Done Status 2
DSTR_DTD3	EQU	3		; DMA Channel Transfer Done Status 3
DSTR_DTD4	EQU	4		; DMA Channel Transfer Done Status 4
DSTR_DTD5	EQU	5		; DMA Channel Transfer Done Status 5
DSTR_DACT	EQU	8		; DMA Active State
DSTR_DCH	EQU	$E00		; DMA Active Channel Mask (DCH0-DCH2)
DSTR_DCH0	EQU	9		; DMA Active Channel 0
DSTR_DCH1	EQU	10		; DMA Active Channel 1
DSTR_DCH2	EQU	11		; DMA Active Channel 2

;	DMA Control Register

DCR_DSS		EQU	$3		; DMA Source Space Mask (DSS0-Dss1)
DCR_DSS0	EQU	0		; DMA Source Memory space 0
DCR_DSS1	EQU	1		; DMA Source Memory space 1	
DCR_DDS		EQU	$C		; DMA Destination Space Mask (DDS-DDS1)
DCR_DDS0	EQU	2		; DMA Destination Memory Space 0
DCR_DDS1	EQU	3		; DMA Destination Memory Space 1
DCR_DAM		EQU	$3f0		; DMA Address Mode Mask (DAM5-DAM0)
DCR_DAM0	EQU	4		; DMA Address Mode 0
DCR_DAM1	EQU	5		; DMA Address Mode 1
DCR_DAM2	EQU	6		; DMA Address Mode 2
DCR_DAM3	EQU	7		; DMA Address Mode 3
DCR_DAM4	EQU	8		; DMA Address Mode 4
DCR_DAM5	EQU	9		; DMA Address Mode 5
DCR_D3D		EQU	10		; DMA Three Dimensional Mode
DCR_DRS		EQU	$F800		; DMA Request Source Mask (DRS0-DRS4)
DCR_DCON	EQU	16		; DMA Continuous Mode
DCR_DPR		EQU	$60000		; DMA Channel Priority
DCR_DPR0	EQU	17		; DMA Channel Priority Level (low)
DCR_DPR1	EQU	18		; DMA Channel Priority Level (high)
DCR_DTM		EQU	$380000		; DMA Transfer Mode Mask (DTM2-DTM0)
DCR_DTM0	EQU	19		; DMA Transfer Mode 0
DCR_DTM1	EQU	20		; DMA Transfer Mode 1
DCR_DTM2	EQU	21		; DMA Transfer Mode 2
DCR_DIE		EQU	22		; DMA Interrupt Enable bit
DCR_DE		EQU	23		; DMA Channel Enable bit 


;------------------------------------------------------------------------
;
;	EQUATES for Phase Locked Loop (PLL) 
;
;------------------------------------------------------------------------

;	Register Addresses Of PLL

PCTL		EQU	$FFFFFD		; PLL Control Register

;	PLL Control Register

PCTL_MF		EQU	$FFF		; Multiplication Factor Bits Mask (MF0-MF11)
PCTL_DF		EQU	$7000		; Division Factor Bits Mask (DF0-DF2)
PCTL_XTLR	EQU	15		; XTAL Range select bit
PCTL_XTLD	EQU	16		; XTAL Disable Bit
PCTL_PSTP	EQU	17		; STOP Processing State Bit 
PCTL_PEN	EQU	18		; PLL Enable Bit
PCTL_PCOD	EQU	19		; PLL Clock Output Disable Bit
PCTL_PD		EQU	$F00000		; PreDivider Factor Bits Mask (PD0-PD3)

;------------------------------------------------------------------------
;
;	EQUATES for BIU 
;
;------------------------------------------------------------------------

;	Register Addresses Of BIU

BCR		EQU	$FFFFFB		; Bus Control Register
DCR		EQU	$FFFFFA		; DRAM Control Register
AAR0		EQU	$FFFFF9		; Address Attribute Register 0 
AAR1		EQU	$FFFFF8		; Address Attribute Register 1 
AAR2		EQU	$FFFFF7		; Address Attribute Register 2 
AAR3		EQU	$FFFFF6		; Address Attribute Register 3 
IDR		EQU	$FFFFF5		; ID Register

;	Bus Control Register

BCR_BA0W	EQU	$1F		; Area 0 Wait Control Mask (BA0W0-BA0W4)
BCR_BA1W	EQU	$3E0		; Area 1 Wait Control Mask (BA1W0-BA14)
BCR_BA2W	EQU	$1C00		; Area 2 Wait Control Mask (BA2W0-BA2W2)
BCR_BA3W	EQU	$E000		; Area 3 Wait Control Mask (BA3W0-BA3W3)
BCR_BDFW	EQU	$1F0000		; Default Area Wait Control Mask (BDFW0-BDFW4)
BCR_BBS		EQU	21		; Bus State
BCR_BLH		EQU	22		; Bus Lock Hold
BCR_BRH		EQU	23		; Bus Request Hold

;	DRAM Control Register

DCR_BCW		EQU	$3		; In Page Wait States Bits Mask (BCW0-BCW1)
DCR_BRW		EQU	$C		; Out Of Page Wait States Bits Mask (BRW0-BRW1)
DCR_BPS		EQU	$300		; DRAM Page Size Bits Mask (BPS0-BPS1)
DCR_BPLE	EQU	11		; Page Logic Enable
DCR_BME		EQU	12		; Mastership Enable
DCR_BRE		EQU	13		; Refresh Enable
DCR_BSTR	EQU	14		; Software Triggered Refresh
DCR_BRF		EQU	$7F8000		; Refresh Rate Bits Mask (BRF0-BRF7)
DCR_BRP		EQU	23		; Refresh prescaler

;	Address Attribute Registers

AAR_BAT		EQU	$3		; External Access Type and Pin Definition Bits
					;	Mask (BAT0-BAT1)
AAR_BAAP	EQU	2		; Address Attribute Pin Polarity
AAR_BPEN	EQU	3		; Program Space Enable
AAR_BXEN	EQU	4		; X Data Space Enable
AAR_BYEN	EQU	5		; Y Data Space Enable
AAR_BAM		EQU	6		; Address Muxing
AAR_BPAC	EQU	7		; Packing Enable
AAR_BNC		EQU	$F00		; Number of Address Bits to Compare Mask 
AAR_BAC		EQU	$FFF000		; Address to Compare Bits Mask (BAC0-BAC11)


;------------------------------------------------------------------------
;
;	EQUATES for SR and OMR 
;
;------------------------------------------------------------------------

;	control and status bits in SR

SR_CP		EQU	$c00000		; mask for CORE-DMA priority bits in SR
SR_CA		EQU	0		; Carry
SR_V		EQU	1		; Overflow	 
SR_Z		EQU	2		; Zero
SR_N		EQU	3		; Negative	 
SR_U		EQU	4		; Unnormalized
SR_E		EQU	5		; Extension	
SR_L		EQU	6		; Limit
SR_S		EQU	7		; Scaling Bit 
SR_I0		EQU	8		; Interrupt Mask Bit 0
SR_I1		EQU	9		; Interrupt Mask Bit 1
SR_S0		EQU	10		; Scaling Mode Bit 0
SR_S1		EQU	11		; Scaling Mode Bit 1
SR_SC		EQU	13		; Sixteen_Bit Compatibility
SR_DM		EQU	14		; Double Precision Multiply
SR_LF		EQU	15		; DO-Loop Flag
SR_FV		EQU	16		; DO-Forever Flag
SR_SA		EQU	17		; Sixteen-Bit Arithmetic
SR_CE		EQU	19		; Instruction Cache Enable
SR_SM		EQU	20		; Arithmetic Saturation
SR_RM		EQU	21		; Rounding Mode
SR_CP0		EQU	22		; bit 0 of priority bits in SR
SR_CP1		EQU	23		; bit 1 of priority bits in SR

;	control and status bits in OMR

OMR_CDP		EQU	$300		; mask for CORE-DMA priority bits in OMR
OMR_MA		EQU	0		; Operating Mode A
OMR_MB		EQU	1		; Operating Mode B
OMR_MC		EQU	2		; Operating Mode C
OMR_MD		EQU	3		; Operating Mode D
OMR_EBD		EQU	4		; External Bus Disable bit in OMR
OMR_SD		EQU	6		; Stop Delay				 
OMR_MS		EQU	7		; Memory Switch bit in OMR
OMR_CDP0	EQU	8		; bit 0 of priority bits in OMR
OMR_CDP1	EQU	9		; bit 1 of priority bits in OMR
OMR_BEN		EQU	10		; Burst Enable 
OMR_TAS		EQU	11		; TA Synchronize Select
OMR_BRT		EQU	12		; Bus Release Timing
OMR_ATE		EQU	15		; Address Tracing Enable bit in OMR.
OMR_XYS		EQU	16		; Stack Extension space select bit in OMR.
OMR_EUN		EQU	17		; Extended stack UNderflow flag in OMR.
OMR_EOV		EQU	18		; Extended stack OVerflow flag in OMR.
OMR_WRP		EQU	19		; Extended WRaP flag in OMR.
OMR_SEN		EQU	20		; Stack Extension Enable bit in OMR.

