;********************************************************************************
;* E N T I T Y 									*
;********************************************************************************
;* File: dsp/initialise.asm							*
;*                                                                         	*
;* Copyright (C) A.R.Hardie-Bick 2004-2005                                      *
;*										*
;* This program is free software; you can redistribute it and/or modify		*
;* it under the terms of the GNU General Public License as published by		*
;* the Free Software Foundation; either version 2 of the License, or (at	*
;* your option) any later version.						*
;*										*
;* This program is distributed in the hope that it will be useful, but		*
;* WITHOUT ANY WARRANTY; without even the implied warranty of			*
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU		*
;* General Public License for more details.					*
;*										*
;* You should have received a copy of the GNU General Public License		*
;* along with this program; if not, write to the Free Software			*
;* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.			*
;********************************************************************************

	org	P(2):			; Use off chip memory

Initialise

	ORI	#$03,MR			; Disable interrupts

	MOVE			#0,X0
	MOVE			X0,Y:flags
	MOVE			X0,Y:iSignals
	MOVE			X0,Y:rSignals
	MOVE			X0,Y:iFlipFlops
	MOVE			X0,Y:rFlipFlops

	BCLR	#readyFlag,Y:flags	; Clear for host to set (using host command)

	MOVE			X:KFFFFFF,X0

	MOVE			X0,M0
	MOVE			X0,M1
	MOVE			X0,M2
	MOVE			X0,M3
	MOVE			X0,M4
	MOVE			X0,M5
	MOVE			X0,M6
	MOVE			X0,M7

	MOVE			#stack,R7	; Using R7 as data stack ptr throughout program

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; Operating Mode Register (see "Core Configuration", 4-17, DSP56303UM)
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

; DSP Memory Config

	BCLR	#SR_CE,SR		; Clear cache enable
	BSET	#OMR_MS,OMR		; 2K + 3K + 3K
	BSET	#SR_CE,SR		; 1K cache + 1K + 3K + 3K
	BSET	#OMR_BEN,OMR		; Set cache burst mode

	NOP
	NOP
	NOP	; 6 NOPs necessary after memory reconfiguration
	NOP	; (see "Core Configuration", 4-17, DSP56303UM)
	NOP
	NOP

; Core-DMA Priority (CDP1, CDP0, reset condition = 11, Core has priority)

	; ******* DMA PRIORITY STRUCTURE *******
	;	
	; Audio DMA has highest priority     (3)
	; Core has next highest priority     (2)
	; Host data BLOCK transfer is lowest (0)
	
	; DMA priorities are individually compared with core

	BCLR	#OMR_CDP1,OMR
	BCLR	#OMR_CDP0,OMR

	; Core has DMA priority 2 (second highest)

	BSET	#SR_CP1,SR
	BCLR	#SR_CP0,SR

	; See audio DMA config and host long interrupts
	; for individual DMA channel configurations
	
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; Receive LUT data from host
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	BCLR	#HCR_HF2,X:HCR	; Clear Host Interface flags
	BCLR	#HCR_HF3,X:HCR		

	BSET	#HCR_HCIE,X:HCR	; Enable host command interrupts

	ANDI	#$FC,MR		; Enable interrupts
	
	BSR	WaitForHost	; Wait for host to transfer LUT seed data

	ORI	#$03,MR		; Disable interrupts

; Build miscellaneous LUTs - - - - - - - - - - - - - - - - - - - - - - -

; These are LUTs whose control parameters are things like brightness, 
; Attack time, LFO delay, etc, whose precision does not need to be 
; that high, so a direct lookup into a small non-interpolated LUT 
; in DRAM can be used. The DSP DRAM has a 1K word page size, making
; this the preferred maximum size for all small LUTs.

	MOVE			#exp1LUT129,R1	; source
	MOVE			#exp1LUT1K,R2	; destination
	JSR	BLUT129toLUT1K

	MOVE			#exp2LUT129,R1
	MOVE			#exp2LUT1K,R2
	JSR	BLUT129toLUT1K

	MOVE			#sqrtLUT129,R1
	MOVE			#sqrtLUT1K,R2
	JSR	BLUT129toLUT1K

	MOVE			#panLUT129,R1
	MOVE			#panLUT1K,R2
	JSR	BLUT129toLUT1K

; Overdrive amplitude LUT

	MOVE			#fltOvLUT129,R1
	MOVE			#fltOvLUT64K,R2
	JSR	LUT129toLUT64K

; Resonance compensation LUTs

	MOVE			#res12dB257,R1
	MOVE			#res12dB64K,R2
	JSR	LUT257toLUT64K

; Filter pitch correction LUTs

	MOVE			#fpc12dB257,R1
	MOVE			#fpc12dB64K,R2
	JSR	LUT257toLUT64K	

; Build Pitch-to-Frequency LUT (64K) - - - - - - - - - - - - - - - - - - - - - - - - - 

	; P2F for oscillators, and as precursor for P2I LUT

	MOVE			#P2Fc,R1
	MOVE			#P2F64K,R3

	DO	#256,_p2f2

	MOVE			#P2Ff,R2
	MOVE			Y:(R1)+,X0

	DO	#256,_p2f1

	TFR	X0,B		Y:(R2)+,Y0	
	MPYuu	X0,Y0,A
	ADDL	A,B
	ASR	B
	RND	B
	NOP

	MOVE			B1,Y:(R3)+
_p2f1
	NOP
_p2f2
	NOP

; Build Pitch to Integration LUT - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	MOVE			#F2Ia,R1	; F2I coarse LUT
	MOVE			#F2Ib,R2	;  "  medium
	MOVE			#F2Ic,R3	;  "  fine

	MOVE			#P2F64K,R4	; Source frequency data
	MOVE			#P2I64K,R5	; Destination feedback data

	MOVE			#F2IbShift,R6	; -> Y: F2IbShift, F2IcShift (LUT block denormalisation shift)

	MOVE			#>65536,X0	; 64K pitch values

	DO	X0,_p2i		; (some DRAM page thrashing is unavoidable)

	; The frequency values are expressed as fractions of Fs

	MOVE			Y:(R4)+,A	; Get frequency value (Fc) from P2F LUT

	; Convert frequency to integrator feedback

	EXTRACTU		#>$008028,A,B	; Extract 8 MS bits
	NOP
	MOVE			B0,N1

	EXTRACTU		#>$008020,A,B	; Extract 8 middle bits
	NOP
	MOVE			B0,N2

	EXTRACTU		#>$008018,A,B	; Extract 8 LS bits
	NOP
	MOVE			B0,N3

	MOVE			Y:(R1+N1),X1	; 			F2Ia[a]
	TFR	X1,A		Y:(R2+N2),X0	; 			F2Ib[b]

	MOVE			Y:(R6)+,Y0	;			F2IbShift

	; Effectively the following calculation is performed:
	;
	;   a = $FF & (Fc >> 16)
	;   b = $FF & (Fc >> 8)
	;   c = $FF & Fc
	;
	;   K = F2Ia[a] * (1 - F2Ib[b]) * (1 - F2Ic[c])
	;
	;   I = F2Ia[a]			
	;   J = I - (I.F2Ib[b] >> F2IbShift)
	;   K = K - (J.F2Ic[c] >> F2IcShift)
	;
	;   where K is the feedback and F2IbShift and F2IcShift
	;   are the block-normalising shifts for F2Ib and F2Ic.

	MOVE			#0,A2		; For unsigned arithmetic

	MPYuu	X0,X1,B				; I.F2Ib[b]
	ASR	Y0,B,B				; >> F2IbShift
	ASR	B		Y:(R6)-,Y0	;			[+P] F2IcShift
	SUB	B,A				; J = I - (I.F2Ib[b] >> F2IbShift)

	MOVE			Y:(R3+N3),Y1	;			F2Ic[c]
	MOVE			A10,L:gp	;			J
	MOVE			L:gp,X

	MPYuu	Y1,X0,B
	DMACuu	Y1,X1,B				; J.F2Ic[c]
	ASR	Y0,B,B				; >> F2IcShift
	SUBL	B,A		X:K7FFFFF,B	; K = J - (J.F2Ic[c] >> F2IcShift)

	ASR	#2,A,A
	SUB	A,B				; 1 - K, where K is the feedback
	RND	B

	NOP
	MOVE			B1,Y:(R5)+	; -> P2I64K (stores 1-K)

_p2i	NOP
	
; Build filter coefficient LUTs - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	MOVE			#fltSelectLUT12dB,R1
	MOVE			#fltCoeffLUT12dB,R2
	JSR	InitialiseFilterCoefficientLUT

; Build Sine LUTs (1K, 4K, 256K) - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	; Create first two quadrants of sine4096 (this method accurate to 0.0005%)

	MOVE			#>$003244,Y0	; frequency
	MOVE			#>$7FFFFF,B	; cosine
	MOVE			#>0,A		; sine
	
	MOVE			#sine4096,R0

	DO	#2048,_sine1

	 MOVE			B,X1
	 MAC	Y0,X1,A		A,Y:(R0)+	; sine += freq x cosine		; sine
	 NOP
	 MOVE			A,X0
	 MAC	-Y0,X0,B			; cosine -= freq x sine
	 NOP

_sine1	NOP

	; Invert Q1, Q2 of sine4096 -> Q3, Q4

	MOVE			#sine4096,R0
	MOVE			#sine4096+2048,R1

	NOP
	NOP
	NOP

	DO	#2048,_sine2

	 MOVE			Y:(R0)+,A
	 NEG			A
	 NOP
	 MOVE			A,Y:(R1)+

_sine2	NOP

	; Contract & copy sine4096 to sine1024

	MOVE			#sine4096,R0
	MOVE			#sine1024,R1
	MOVE			#4,N0	
	NOP

	DO	#1024,_sine3

	 MOVE			Y:(R0)+N0,Y0
	 MOVE			Y0,Y:(R1)+

_sine3	NOP

	; Expand and interpolate sine4096 to sine256K

	MOVE			#sine4096,R0
	MOVE			#sine256K,R1
	MOVE			#4095,M0
	
	MOVE			#>4096,X0
	DO	X0,_sine5

	 MOVE			Y:(R0)+,X0	; base
	 MOVE			Y:(R0),B	; next
	 MOVE			#0.015625,Y0	; 1/64 = fractional increment for interpolation	
	 SUB	X0,B				; Difference
	 CLR	A				; reset increment counter
	 MOVE			B,X1		; delta
	
	 ; linearly interpolate between the steps of the sine4096 table

	 DO	#64,_sine4	; 256K / 4K = 64

	  TFR	X0,B		A,Y1		; factor	; base
	  NOP
	  MACR	X1,Y1,B				; base + delta x factor
	  ADD	Y0,A				; next factor increment
	  MOVE			B,Y:(R1)+

_sine4	 NOP

_sine5	NOP	

	MOVE			#$FFFFFF,M0

; Build sine LUT in on-chip RAM for fast linear interpolation

	MOVE			#sine4096,R0
	MOVE			#sine512,R1
	MOVE			#8,N0	
	NOP

	; First populate the base values in X: memory

	DO	#512,_sine6

	 MOVE					Y:(R0)+N0,Y0
	 MOVE			Y0,X:(R1)+

_sine6	NOP

	; Then store the differences - note that these are stored as half 
	; their actual value, to simplify interpolation calculation

	MOVE			#sine512,R0
	MOVE			#511,M0
	
	NOP
	NOP
	NOP

	MOVE			X:(R0)+,X0

	DO	#512,_sine7

	 MOVE			X:(R0)-,A
	 SUB	X0,A
	 ASR	A
	 NOP
 	 MOVE					A,Y:(R0)+	; diff/2
	 MOVE			X:(R0)+,X0

_sine7	NOP
	
	MOVE			#$FFFFFF,M0

; Simulate oscillator sine-wave access to check code and LUT functionality

	; Read results in Y:soundCheck to verify smooth sine quadrant

	MOVE			#soundCheck,R2
	MOVE			X:Ksine512,A	; Osc sine LUT

	DO	#512,_scLoop

	MOVE			A1,R1
	MOVE			A0,Y0
	NOP
	NOP

	; Linear interpolation

	MOVE			L:(R1),X	; Base values in X:, deltas/2 in Y:
	MPYsu	X0,Y0,B
	ADD	X1,B
	NOP
	MOVE			B,Y:(R2)+

	; - - - 

	MOVE			#>0,Y1
	MOVE			#0.125,Y0	; Increment by small steps to test LI
	ADD	Y,A
	NOP
_scLoop

	NOP

; Oscillator LUT shift control word - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			#>$02100F,X0
	MOVE			X0,Y:oscPhzCtrl	; See oscillator code

; Build Dynamic Limiter LUT - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; This LUT is approximately a 1/x LUT for x > 1, and unity for x <= 1. However, a soft 
; transition to the curve occurs for 0.8 <= x <= 1.2. The maximum value of x has to be
; determined in accordance with the size of the extension register. In this case, the
; extension register (A2 or B2) is 8 bits, hence the maximum value of x is 256.
;
; The curve is given by:
;
;	x = 0..8:
;	
; 	    y = 1
;	    if x > 0.8 then y = cos(1.468(x - 0.8))
;	    if x > 1.2 then y = 1/x
;
; For a sine LUT of 256K, the following relationships are useful:
;
; 	sin(a) = sin256K[a/(2pi.32)]			(2^23 -> 2^18 = /32)
;	cos(a) = sin256K[(1/128)+a/(2pi.32)]
;	cos(1.468a) = sin256K[(1/128)+1.468a/(2pi.32)]
; 
; For 0 <= x <= 256, to get sufficient resolution around 0.8 < x < 1.2, 256 points
; should correspond to unity range. In addition, when the LUT is in use, linear 
; interpolation should be applied. For most normal situations, the look-up will
; be for values of x < 1.2, so the quality for these values is critical. The
; LUT itself is therefore 256.256 = 64K in size, and since linear interpolation
; is used, the actual number will be 65537, although the top value is never
; reached in practice.
;
; In a 64K LUT, x < 0.8 corresponds to LUT[n] up to n = 204
;               x < 1.2      "       "     "  "   " n = 306
;
; thereafter the curve is 1/x, which is 256/address
;
; As a final measure, the correction factor across the entire LUT is
; multiplied by a factor of 0.98, in order to guarantee no clipping.

	MOVE			#0,R1
	MOVE			#lim256LUT64K,N1
	MOVE			#>0.98,X0

	DO	#205,_lim0p8				; 0 <= x < 0.8, y = 1 	(y = 0.98, with margin)

	MOVE			X0,Y:(R1+N1)
	MOVE			(R1)+
_lim0p8	
	MOVE			#sine256K,R0
	MOVE			#>$010000,X0		; 1/128
	MOVE			#0.007301233,X1		; 1.468/(2pi.32)
	MOVE			#0.8,Y0

	DO	#307-205,_lim1p2			; 0.8 < x < 1.2, y = cos(1.468(x - 0.8))
	
	MOVE			R1,A
	ASL	#15,A,A					; addr -> x
	SUB	Y0,A					; x - 0.8
	NOP
	MOVE			A,Y1
	MPYR	X1,Y1,A					; 1.468 (x - 0.8) / (2pi.32)
	ADD	X0,A					; + 1/128 (to get cosine)
	NOP
	MOVE			A,N0
	NOP
	NOP
	NOP
	MOVE			Y:(R0+N0),Y1		; y = cos
	MPYR	#0.98,Y1,A				; apply safety margin
	NOP
	MOVE			A,Y:(R1+N1)
	MOVE			(R1)+
_lim1p2
	
	MOVE			#>256,Y0
	MOVE			#>65537-307,X0

	DO	X0,_lim256				; 1.2 < x <= 256, y = 1/x

	AND	#$FE,CCR				; Clear carry bit C (quotient sign bit)
	MOVE			R1,X1			; Address ("x")
	MOVE			Y0,A			; Number to be divided = 256 ("1")
	DO	#24,_limdv				; Form a 24bit quotient
	DIV	X1,A					; Quotient in A0, remainder in A1
_limdv	NOP
	MOVE			A0,Y1			; 1/x
	MPYR	#0.98,Y1,A				; apply margin
	NOP
	MOVE			A,X:(R1+N1)		
	MOVE			(R1)+
_lim256	

; Noise generator intialisation - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; Fill buffer in DRAM with PDF-shaped noise. This can then be read out randomly 
; to quickly generate noise with the same PDF.

	; Initialise noise PDF LUT 

	MOVE			#pdfLUT129,R1
	MOVE			#pdfLUT1K,R2
	JSR	BLUT129toLUT1K

	; Copy PDF LUT to chip RAM for this initialisation, to avoid DRAM page jumps

	MOVE			#pdfLUT1K-512,R4		; Source of distribution function LUT
	MOVE			#initSpaceY,R1			; This area is free during initialisation

	NOP
	NOP

	MOVE					X:(R4)+,A
	MOVE					X:(R4)+,B

	DO	#1024/4,_pdfCopy

	MOVE			A,Y:(R1)+	X:(R4)+,A	; Copy PDF LUT (DRAM as X:)
	MOVE			B,Y:(R1)+	X:(R4)+,B
	MOVE			A,Y:(R1)+	X:(R4)+,A
	MOVE			B,Y:(R1)+	X:(R4)+,B
	
_pdfCopy

	; Initialise fast R250 rng in chip RAM using linear congruential method

	MOVE			#r250area,R1			; This area is free during initialisation
	MOVE     		X:rng,A    		      	; Get random seed (uninitialised memory location)
	LSL  	A   		X:KnoiseMask,Y0  		; 1st step of 1st random number

	DO	#250,_r250Init

	LSL	A
	NOP
	EOR  	Y0,A    	A1,Y1
	TCS  	Y1,A						; Random number
	NOP
	MOVE	   		A1,X:(R1)+			; Store in R250 array
	
_r250Init

	; Loop for writing noise samples to noise buffer

	MOVE			#noise1M,R0			; Destination start address for noise
	MOVE			#>1048576,Y0			; 1M noise samples
	MOVE			#soundData+512,R1		; (LUT copied from pdfLUT1K)
	MOVE			#>512,X1			; Shift for (bipolar) LUT offset

	; initialise r250 rng address registers

	MOVE			#r250area,R2
	MOVE			#r250area+103,R3
	MOVE			#249,M2
	MOVE			#249,M3

	NOP
	NOP
	NOP

	MOVE			X:(R3)+,A			; End-of-loop pipelined operations
	MOVE			X:(R2)+,X0

	DO	Y0,_noiseInitLoop

	; generate two 24 bit random numbers using R250

	EOR	X0,A		X:(R3)+,B
	MOVE			X:(R2)-,Y0
	EOR	Y0,B		A1,X0
	MPYR	X0,X1,A		A1,X:(R2)+			; [+P] address PDF LUT using 1st random number
	MOVE			B1,X:(R2)+
	MOVE			A1,N1
	MOVE			B1,Y1				;			2nd random number

	; generate PDF noise using LUT distribution method

	NOP
	NOP
	MOVE			Y:(R1+N1),X0			; 			PDF LUT access
	MPYR	X0,Y1,B		X:(R3)+,A            		; PDF noise product	[+P] R250 next iteration
	MOVE			X:(R2)+,X0			;			[+P]  "
	MOVE			B,Y:(R0)+			; 			Store noise sample

_noiseInitLoop

	MOVE			A1,X:rng

	MOVE			X:KFFFFFF,M2
	MOVE			X:KFFFFFF,M3

; Real Time Controller (RTC) filter initialisation - - - - - - - - - - - - - - - - - - - - -

_ptc		equ	0.020		; Parameter filter time constant (transition time in seconds)
_pstc		equ	0.012		; Pitch shift (midi is time coarse) filter time constant
_ptcEncr	equ	0.050		; Parameter time constant for encoder (as modulator)

	; This is called parameter filtering, but really it's for filtering values, such as those
	; generated by the pots and/or midi, that change in real time, and have low precision
	; or large steps. This includes filtering the currently edited sound parameter (when
	; this is a continuous value, not a switch, like waveform selection). 

	MOVE			#Ctr1ParaFilt,R0
	MOVE			#_ptc,A
	JSR	ParaFilt_Initialise

	MOVE			#Ctr2ParaFilt,R0
	MOVE			#_ptc,A
	JSR	ParaFilt_Initialise

	MOVE			#Ctr3ParaFilt,R0
	MOVE			#_ptc,A
	JSR	ParaFilt_Initialise

	MOVE			#MmodParaFilt,R0
	MOVE			#_ptc,A
	JSR	ParaFilt_Initialise

	MOVE			#MvolParaFilt,R0
	MOVE			#_ptc,A
	JSR	ParaFilt_Initialise

	MOVE			#MPanParaFilt,R0
	MOVE			#_ptc,A
	JSR	ParaFilt_Initialise

	MOVE			#EditParaFilt,R0
	MOVE			#_ptc,A
	JSR	ParaFilt_Initialise

	MOVE			#PsftParaFilt,R0
	MOVE			#_pstc,A
	JSR	ParaFilt_Initialise

; Sound-related initialisation - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			#soundData,R0
	CLR	A		#soundTransferBuffer,R1

	DO	#soundDataSize,_loop1			; Clear all sound data

	MOVE			A,Y:(R0)+
	MOVE			A,Y:(R1)+

_loop1	NOP


	MOVE			#matrixSources,R0
	CLR	A		#matrixPoPSources,R1
	
	DO	#nMatrixSources,_loop2			; Clear all matrix sources

	MOVE			A,X:(R0)+
	MOVE			A,X:(R1)+
	
_loop2	NOP

	CLR	A		#soundStructure,R0	; Clear all matrix destinations	
	DO	#nDirect+nPoPs,_loop3
	MOVE			A,X:(R0)+
_loop3
	CLR	A		#interpolatedParas,R0	; Clear all interpolated parameters
	DO	#nInterpolatedParas,_loop4
	MOVE			A,L:(R0)+
_loop4
	
	CLR	A		#NISampleParas,R0	; Clear all non-interpolated parameters
	DO	#NISampleParasPerVoice*N_VOICES_MEM,_loop5
	MOVE			A,L:(R0)+
_loop5

	MOVE			A,Y:globalMmod		; Clear all matrix controller inputs
	MOVE			A,Y:globalCtr1
	MOVE			A,Y:globalCtr2
	MOVE			A,Y:globalCtr3
	MOVE			A,Y:globalMidiPan
	MOVE			A,Y:globalMasterVol
	MOVE			A,Y:globalMasterTune
	MOVE			A,Y:globalPitchShift

	MOVE			A,Y:maxInL
	MOVE			A,Y:maxInR
	MOVE			A,Y:maxOutL
	MOVE			A,Y:maxOutR

	; Clear output limiter

	MOVE			A,Y:limMaxLR

; Reset parameter generator state machines

	; Midi sync (LFO5 & 6) - - - - - - - -

	BCLR	#midiSyncSustain,Y:flags

	; Voice matrix sources - - - - - - - - 

	CLR	A		#vMatrixSources,R0

	DO	#vMatrixStateSize*N_VOICES_MEM,_vmstate

	MOVE			A,X:(R0)+

_vmstate

	; Voice states - - - - - - - - - - - - 

	CLR	A		#voiceState,R0

	DO	#voiceStateSize*N_VOICES_MEM,_vstate

	MOVE			A,Y:(R0)+

_vstate

	; ENV phase subroutine addresses

	MOVE			#vENVstate+ENV_phase+0*ENVstateSize,R0
	MOVE			#voiceStateSize,N0
	MOVE			#vENVstate+ENV_phase+1*ENVstateSize,R1
	MOVE			#voiceStateSize,N1

	MOVE			#>ENVreleasePhase,X0

	DO	#N_VOICES_MEM,_env_phases

	MOVE			X0,Y:(R0)+N0
	MOVE			X0,Y:(R1)+N1

_env_phases

	CLR	A		#sLFOs,R0

	DO	#sLFOstateSize*2,_slfos

	MOVE			A,Y:(R0)+
	NOP
_slfos

; Edit-related initialisation

	CLR	A
	BCLR	#editParaValid,Y:flags
	MOVE			A,Y:parameter	; (in case edit data is sent before parameter)

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

; *** DEBUG: Select memory space, address and increment for returning debug data

	MOVE			#>test1,X0		; Address of data to view

	MOVE			#>1,X1				; Address increment (each button press)

;	BCLR			#0,Y:debugSpace			; Clear this bit for X:
	BSET			#0,Y:debugSpace			; Set this bit for Y:

	MOVE			X0,Y:debugAddress
	MOVE			X0,Y:debugStartAddress
	MOVE			X1,Y:debugAddressIncrement

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; The parameter genration vector
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	CLR	A		#>ParameterGenerator,X0
	MOVE			X0,Y:sGenVector

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; Configure Timer 0 for MIPS measurement
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	; Assume TCSR0 is all zeroes (true for Chameleon model 01)
	; This puts timer in mode 0: free running timer

	MOVEP	#0,X:<<TLR0		; This value reloaded whenever timer restarted
	MOVEP	#$FFFFFF,X:<<TCPR0	; High value avoids any automatic reloading

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; Configure audio transmission/reception on DMA channels 0 and 1 using
; double buffering (from Chameleon SDK 'Dspthru' example).
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	; DMA Control Register (DCR) for audio transmitter

	; [23] 		DE 		0 	DMA disabled (until needed)
	; [22] 		DIE 		0 	DMA interrupts disabled
	; [21:19] 	DTM2:DTM0 	101 	word transfer, enabled by DE, but DE not cleared when done
	; [18:17] 	DPR1:DPR0 	11 	DMA priority level 3 (highest)
	; [16] 		DCON 		0 	continuous mode off
	; [15:11] 	DRS4:DRS0 	01011 	peripheral request MDRQ1
	; [10] 		D3D 		0 	non-3D mode
	; [9:7] 	DAM5:DAM3 	100 	destination is linear access, no update
	; [6:4] 	DAM2:DAM0 	000 	source is 2D, using DOR0
	; [3:2] 	DDS1:DDS0 	00 	X memory DMA destination (default)
	; [1:0] 	DSS1:DSS0 	00 	X memory DMA source

	; 0 0 101 11 0 01011 0 100 000 00 00
	; $2E5A00

	BCLR	#CRB_TE0,X:<<CRB0
	BCLR	#CRB_RE,X:<<CRB0

	BCLR	#DCR_DE,X:<<DCR0
	BCLR	#DCR_DE,X:<<DCR1
	MOVEP	#>-(4*N_CYCLES-1),X:<<DOR0
	MOVEP	#>(4*N_CYCLES-1),X:<<DCO0
	MOVEP	#>DACBuffer1,X:<<DSR0
	MOVEP	#>TX00,X:<<DDR0
	MOVEP	#>$2E5A00,X:<<DCR0

	; DMA Control Register (DCR) for audio receiver

	; [23] 		DE 		0 	DMA disabled (until needed)
	; [22] 		DIE 		0 	DMA interrupts disabled
	; [21:19] 	DTM2:DTM0 	101 	word transfer, enabled by DE, but DE not cleared when done
	; [18:17] 	DPR1:DPR0 	11 	DMA priority level 3 (highest)
	; [16] 		DCON 		0 	continuous mode off
	; [15:11] 	DRS4:DRS0 	01010 	peripheral request MDRQ0
	; [10] 		D3D 		0 	non-3D mode
	; [9:7] 	DAM5:DAM3 	001 	destination is 2D access, using DOR1
	; [6:4] 	DAM2:DAM0 	100 	source is linear access, no update
	; [3:2] 	DDS1:DDS0 	00 	X memory DMA destination (default)
	; [1:0] 	DSS1:DSS0 	00 	X memory DMA source

	; 0 0 101 11 0 01010 0 001 100 00 00
	; $2E50C0

	MOVEP	#-(4*N_CYCLES-1),X:<<DOR1
	MOVEP	#>(4*N_CYCLES-1),X:<<DCO1
	MOVEP	#>ADCBuffer1,X:<<DDR1
	MOVEP	#>RX0,X:<<DSR1
	MOVEP	#>$2E50C0,X:<<DCR1

	; Set initial active ADC/DAC buffer to second of two

	MOVE	#>DACBuffer2,X0
	MOVE	X0,X:DACBufferPtr
	MOVE	#>ADCBuffer2,X0
	MOVE	X0,X:ADCBufferPtr

	; Clear audio buffers

	CLR	A

	MOVE	#DACBuffer1,R0
	REP	#(4*N_CYCLES)
	MOVE	A,X:(R0)+

	MOVE	#ADCBuffer1,R0
	REP	#(4*N_CYCLES)
	MOVE	A,X:(R0)+

	; Enable ESSI0 transmitter and receiver 

	MOVEP	A,X:<<TX00
	BSET	#CRB_TE0,X:<<CRB0
	BSET	#CRB_RE,X:<<CRB0

	; Synchronize DMA Transmitter buffers with audio left and right

_sync0

	BRCLR	#SSISR_TDE,X:<<SSISR0,*
	BRSET	#SSISR_TFS,X:<<SSISR0,_sync1
	MOVEP	A,X:<<TX00
	BRCLR	#SSISR_TDE,X:<<SSISR0,*

_sync1

	MOVEP	A,X:<<TX00
		
	; Enable DMA Channel 0 (ESSI0 TX)

	BSET	#DCR_DE,X:<<DCR0

	MOVEP	X:<<RX0,A
	BRCLR	#SSISR_RDF,X:<<SSISR0,*
	BRSET	#SSISR_RFS,X:<<SSISR0,_sync2
	MOVEP	X:<<RX0,A
	BRCLR	#SSISR_RDF,X:<<SSISR0,*

_sync2

	; Enable DMA Channel 1 (ESSI0 RX)

	BSET	#DCR_DE,X:<<DCR1

	; Synchronise code fragment alternation with audio buffer alternation

	BCLR	#skipCodeFragment,Y:flags
	
	; Initialise max recorded benchmark values

	CLR	A
	NOP
	MOVE	A,Y:maxTotalMips
	MOVE	A,Y:maxFragMips
	MOVE	A,Y:totalMips

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	ANDI	#$FC,MR		; Enable interrupts

	BSR	SignalHost	; Tell host DSP is initialised
	RTS

;----------------------------------------------------------------------
;
; 	Initialisation Subroutines
;
;----------------------------------------------------------------------

WaitForHost	; Host and dsp syncronisation

	BSET	#HCR_HF2,X:HCR		; Tell host DSP is ready
	JCLR	#readyFlag,Y:flags,*	; Wait for host (ie to do DMA)
	BCLR	#readyFlag,Y:flags	; (ready for next time)
	BCLR	#HCR_HF2,X:HCR 		; Tell host DSP is busy
	RTS

SignalHost	; DSP acknowledge to host

	BSET	#HCR_HF2,X:HCR
	RTS	

;----------------------------------------------------------------------
; Build filter coefficient LUTs
;----------------------------------------------------------------------
; On entry: 	R1 contains filter selection LUT address (ie fltSelectLUT12dB)
;		R2 contains filter coefficient LUT destination address
;
; Explanation:	Filter characteristics ("capacitor" values, pitch corrections,
;		and resonance corrections) are used to generate a pitch-indexed 
;		LUT with 5 data items per entry (8 including padding) 
;
; Uses:		X,Y,A,B,R1,R2
;
; On exit:	Destination 512K LUT contains 64K pitch-indexable entries of
;		
;		Fa		1st integrator coefficient
;		Fb		2nd	"	"
;		Fc		3rd	"	"
;		Fd		4th	"	"
;		ResC		Resonance correction factor
;		-		unused
;		-		  "
;		-		  "
;		(accessed sequentially, to avoid page thrashing)
;----------------------------------------------------------------------

InitialiseFilterCoefficientLUT

	; fltSelectLUT12dB or fltSelectLUT24dB in R1 
	; fltCoeffLUT12dB or fltCoeffLUT24dB in R2

	MOVE			X:KP2I64K,R5		; Base address half way through P2I LUT 
	MOVE			X:KP2I64K,R6		;		"
	
	MOVE			#6,N1			; to skip back to start of filter LUT
	MOVE			#4,N2			; to skip over unused LUT entries

	MOVE			#-32768,R3		; First pitch value
	MOVE			#>65536,X0	

	DO	X0,_fltCoeffLoop			; One for each pitch value

	; Pitch correction

	MOVE			X:(R1)+,R4		;			Address of filter pitch correction LUT
	MOVE			R3,N4			;			Uncorrected pitch used as index
	MOVE			R3,X:gp1		;			Store for looking up resonance correction
	MOVE			R3,A
	ASL	#8,A,A					; fractional pitch
	MOVE			Y:(R4+N4),X0		;			Look up fractional pitch correction
	ADD	X0,A		X:(R1)+,R4		; Corrected pitch	[+P] resonance LUT, eg res24dB64K

	; Prepare resonance correction

	MOVE			R3,N4			;			index resonance correction by pitch
	MOVE			A,X0			;			Limit corrected pitch

	; Apply pitch offsets for each 6dB stage

	MOVE			X:(R1)+,A		;			pitch offset for Fa (cents x 256)
	ADD	X0,A		X:(R1)+,B		;			  "     "     "  Fb
	ADD	X0,B
	MOVE			A,A
	MOVE			B,B
	ASR	#8,A,A
	ASR	#8,B,B

	MOVE			A,N5
	MOVE			B,N6
	
	NOP
	NOP
	
	MOVE			Y:(R5+N5),Y0		;			Fa
	MOVE			Y:(R6+N6),Y1		;			Fb

	MOVE			X:(R1)+,A		;			pitch offset for Fc
	ADD	X0,A		X:(R1)+,B		;			  "     "     "  Fd
	ADD	X0,B		(R1)-N1			;			go back to start of fltSelectLUTxx
	MOVE			A,A
	MOVE			B,B
	ASR	#8,A,A
	ASR	#8,B,B

	MOVE			A,N5
	MOVE			B,N6
	NOP
	NOP
	MOVE			Y:(R5+N5),X0		;			Fc
	MOVE			Y:(R6+N6),X1		;			Fd

	MOVE			Y:(R4+N4),A1		;			resonance correction

	; Write LUT sequentially, to avoid page thrashing (keep initialisation fast)

	MOVE			Y0,Y:(R2)+		;			-> Fa
	MOVE			Y1,Y:(R2)+		;			-> Fb
	MOVE			X0,Y:(R2)+		;			-> Fc
	MOVE			X1,Y:(R2)+		;			-> Fd
	MOVE			A1,Y:(R2)+N2		;			-> ResC
							;			-> -
							;			-> -
							;			-> -

	MOVE			(R3)+			;			next pitch value
	
_fltCoeffLoop
	
	NOP

	RTS

;----------------------------------------------------------------------
; Build bipolar 1K LUT from 129-step bipolar LUT using LI
;----------------------------------------------------------------------
; On entry: 	R1 contains source 129-step LUT centre address
;		R2 contains destination 1K LUT centre address
;
; Uses:		R1,R2,N2,R3,N3,A,B,X0
;
; On exit:	Destination LUT written, R2,N2,R3,N3,A,B,X0 modified
;----------------------------------------------------------------------

BLUT129toLUT1K

	MOVE		#512,N2		; Offset into bipolar LUT destination
	MOVE		#$800000,R3	; -1
	MOVE		#16384,N3	; numerical range / LUT size = 2 / 1024
	MOVE		R1,A		; Centre address for source 129-setp LUT
	MOVE		(R2)-N2
	MOVE		#>64,X1		; Half of src LUT size (for LUT subroutine)

	DO	#1024,_loop

	MOVE		R3,X0		; Fractional data value to be looked-up
	MOVE		(R3)+N3		; Prepare source data for next iteration
	JSR	LUT
	MOVE		B,Y:(R2)+	; Store looked-up value in destination LUT

_loop	NOP

	RTS

;----------------------------------------------------------------------
; Build 1K LUT from 129-step LUT using LI
;----------------------------------------------------------------------
; On entry: 	R1 contains source 129-step LUT base address
;		R2 contains destination 1K LUT base address
;
; Uses:		R1,R2,N2,R3,N3,A,B,X0
;
; On exit:	Destination LUT written, R2,N2,R3,N3,A,B,X0 modified
;----------------------------------------------------------------------

LUT129toLUT1K

	MOVE		#0,R3		; 0 = first data value (0.999~ = last)
	MOVE		#8192,N3	; numerical range / LUT size = 1 / 1024
	MOVE		R1,A		; Base address for source 129-step LUT
	MOVE		#>128,X1	; Src LUT size (for LUT subroutine)

	DO	#1024,_loop

	MOVE		R3,X0		; Fractional data value to be looked-up
	MOVE		(R3)+N3		; Prepare source data for next iteration
	JSR	LUT
	MOVE		B,Y:(R2)+	; Store looked-up value in destination LUT

_loop	NOP

	RTS

;----------------------------------------------------------------------
; Build 1K LUT from 65-step LUT using LI
;----------------------------------------------------------------------
; On entry: 	R1 contains source 129-step LUT base address
;		R2 contains destination 1K LUT base address
;
; Uses:		R1,R2,N2,R3,N3,A,B,X0
;
; On exit:	Destination LUT written, R2,N2,R3,N3,A,B,X0 modified
;----------------------------------------------------------------------

LUT65toLUT1K

	MOVE		#0,R3		; 0 = first data value (0.999~ = last)
	MOVE		#8192,N3	; numerical range / LUT size = 1 / 1024
	MOVE		R1,A		; Base address for source 65-step LUT
	MOVE		#>64,X1		; Src LUT size (for LUT subroutine)

	DO	#1024,_loop

	MOVE		R3,X0		; Fractional data value to be looked-up
	MOVE		(R3)+N3		; Prepare source data for next iteration
	JSR	LUT
	MOVE		B,Y:(R2)+	; Store looked-up value in destination LUT

_loop	NOP

	RTS

;----------------------------------------------------------------------
; Build 64K LUT from 129-step LUT using LI
;----------------------------------------------------------------------
; On entry:  	R1 contains source 129-step LUT base address
;		R2 contains destination 64K LUT base address
;
; Required:	129 words of spare chip YRAM from Y:initSpaceY
; Uses:		R0-3,N3,A,B,X0
;
; On exit:	Destination LUT written, R0-3,N3,A,B,X0 modified
;----------------------------------------------------------------------

LUT129toLUT64K

	; Copy source LUT to chip RAM to prevent DRAM page thrashing

	MOVE		#initSpaceY,R0	; This area is free during initialisation

	DO	#129,_preLoop

	MOVE		Y:(R1)+,X0
	MOVE		X0,Y:(R0)+

_preLoop	

	; Fill destination LUT by interpolating source LUT in chip RAM

	MOVE		#initSpaceY,A	; Base address for source 129-step LUT
	MOVE		#0,R3		; 0 = first data value (0.999~ = last)
	MOVE		#128,N3		; numerical range / LUT size = 1 / 64K
	MOVE		#>128,X1	; Src LUT size (for LUT subroutine)

	MOVE		#>65536,X0

	DO	X0,_loop

	MOVE		R3,X0		; Fractional data value to be looked-up
	MOVE		(R3)+N3		; Prepare source data for next iteration
	JSR	LUT
	MOVE		B,Y:(R2)+	; Store looked-up value in destination LUT

_loop	NOP

	RTS

;----------------------------------------------------------------------
; Build 64K LUT from 257-step LUT using LI
;----------------------------------------------------------------------
; On entry: 	R1 contains source 257-step LUT base address
;		R2 contains destination 64K LUT base address
;
; Required:	257 words of spare chip YRAM from Y:initSpaceY
; Uses:		R0-3,N3,A,B,X0
;
; On exit:	Destination LUT written, R0-3,N3,A,B,X0 modified
;----------------------------------------------------------------------

LUT257toLUT64K

	; Copy source LUT to chip RAM to prevent DRAM page thrashing

	MOVE		#initSpaceY,R0	; This area is free during initialisation

	DO	#257,_preLoop

	MOVE		Y:(R1)+,X0
	MOVE		X0,Y:(R0)+

_preLoop	

	; Fill destination LUT by interpolating source LUT in chip RAM

	MOVE		#initSpaceY,A	; Base address for source 257-step LUT
	MOVE		#0,R3		; 0 = first data value (0.999~ = last)
	MOVE		#128,N3		; numerical range / LUT size = 1 / 64K
	MOVE		#>256,X1	; Src LUT size (for LUT subroutine)

	MOVE		#>65536,X0

	DO	X0,_loop

	MOVE		R3,X0		; Fractional data value to be looked-up
	MOVE		(R3)+N3		; Prepare source data for next iteration
	JSR	LUT
	MOVE		B,Y:(R2)+	; Store looked-up value in destination LUT

_loop	NOP

	RTS

;----------------------------------------------------------------------
; LUT Lookup
;----------------------------------------------------------------------
; Entry:    X0 contains the value to be converted (-1 or 0..+.9999)
;	    X1 contains the LUT size (ie 64, 128, must be pwr of 2)
;           A contains the LUT base address, i.e. expLUT129
;
; Exit:     B contains the result
;	    All registers preserved except B	    
;
; The input in X0 has a valid range from -1..+1, or 0..+1
; The value in X1 defines the size of the LUT.
; The value in A defines the base address for the LUT.
;
; If the LUT is bipolar, the base address must be half way 
; through the LUT, corresponding to a data input value of zero,
; and the value in X1 defines the size of one half of the LUT.
;
; In all LUTs using this subroutine, there must be (2^n)+1 entries,
; actually stored, where n is the order of the LUT. Typical
; entries are 129, 65, and so on. This is so linear interpolation
; can be performed with the topmost entry in the list. Nevertheless,
; the value supplied in X1, must be 2^n.
;----------------------------------------------------------------------

LUT	PUSHXAR0

	MAC	X0,X1,A					; Generate address
	NOP
	MOVE		      	A1,R0                   ; Copy integer address 
	MOVE			A0,X1                   ; Copy fractional address
	NOP
	NOP
	MOVE			Y:(R0)+,B	        ; Get first data
	MOVE			Y:(R0)+,A		; Get next data
	SUB	B,A					; Calculate data difference
	ASL	B					; pre-compensate mixed multiply
	MOVE			A,X0                    ; Copy difference for multiply
	MACsu	X0,X1,B					; Linear interpolate
	ASR	B					; compensate mixed multiply
	RND	B

	PULLXAR0

	RTS
