;********************************************************************************
;* E N T I T Y 									*
;********************************************************************************
;* File: dsp/parameterGenerator.asm						*
;*                                                                         	*
;* Copyright (C) A.R.Hardie-Bick 2004-2005                                      *
;*										*
;* This program is free software; you can redistribute it and/or modify		*
;* it under the terms of the GNU General Public License as published by		*
;* the Free Software Foundation; either version 2 of the License, or (at	*
;* your option) any later version.						*
;*										*
;* This program is distributed in the hope that it will be useful, but		*
;* WITHOUT ANY WARRANTY; without even the implied warranty of			*
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU		*
;* General Public License for more details.					*
;*										*
;* You should have received a copy of the GNU General Public License		*
;* along with this program; if not, write to the Free Software			*
;* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.			*
;********************************************************************************

	org	P(1):		; On-chip program memory (768 words)

ParameterGenerator

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Register interrupt signals (at this point, interrupts are disabled)
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	; The iSignals flags can be set or reset by interrupts, 
	; so it's necessary to make a copy in rSignals while
	; interrupts are disabled.

	; (At present this is just the midi clock event flag)

	CLR	A		Y:iSignals,X0
	MOVE			X0,Y:rSignals
	MOVE			A,Y:iSignals

	ANDI	#$FC,MR		; Re-enable interrupts

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Register interrupt-driven flipflops (no need to disable interrupts) 
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	; There is no need to disable interrupts for these, because
	; the iFlipFlops are set by interrupts, but only ever cleared
	; in the parameter engine. By making a copy in rFlipFlops
	; at this point, you get a stable version of these flags.

	MOVE			Y:iFlipFlops,Y0
	MOVE			Y0,Y:rFlipFlops

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Update Edited Parameter
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Edited parameter values are filtered at the parameter sample rate, 
; which is why they can't be updated by just using an interrupt.
; If there is a sound pending, it means that any edits are intended 
; for the pending sound, so the edit value must go somewhere in 
; Y:soundTransferBuffer, otherwise its destination is somewhere
; in Y:soundData. If a sound isn't pending, then any product-of-
; product type matrix parameters must have their depth translated by 
; pn = 1 - gn and stored in X:soundPnData, an array of up to 512 
; translated Pn values.

	ORI	#$03,MR		; To ensure parameter edit info is atomic (it is set by interrupts)

	MOVE			Y:paraData,Y0		; Data to write to parameter
	MOVE			Y:parameter,R1		; Offset into Y:soundData or Y:soundTransferBuffer
	MOVE			Y:parameterPn,R2	; Address in X:soundPnData (or fake address if not needed)
	MOVE			Y:flags,R3		; Make a copy of flags, as these are needed several times

	ANDI	#$FC,MR		; Re-enable interrupts

	JCLR	#editParaValid,R3,_noEdit		; ie, if para just changed, but data not ready, or sound just loaded

	CLR	A		#>soundTransferBuffer,X0
	MOVE			#soundData,A1
	BTST	#soundPending,R3
	TCS	X0,A
	MOVE			#EditParaFilt,R0	; Filter object ptr
	MOVE			A,N1			; Base address for parameter

	JCLR	#newPara,R3,_samePara

	; Parameter hasn't been touched before, so precharge filter with stored value

	MOVE			Y:(R1+N1),A		; unedited value
	JSR	ParaFilt_ToValue

	BCLR	#newPara,Y:flags			; (slight non-atomicity, shouldn't cause problem)

_samePara

	TFR	Y0,A					; Edited value
	JSSET	#smoothEdit,R3,ParaFilt_Filter		; Filter continuous paras
	MOVE			A,Y:(R1+N1)		; edited value, possibly filtered
	JSET	#soundPending,R3,_noPn

	; pn = 1, if gn >= 0 pn = 1 - gn

	MOVE			A,X0			; edited value, possibly filtered
	TST	A		X:K7FFFFF,B		; test gn		pn = 1
	SUB	X0,B		IFGE			; if gn >= 0 pn = 1 - gn else pn = 1
	NOP
	MOVE			B,X:(R2)		; pn -> soundPnData[..] or dummyPnData if not a p-of-p depth

_noPn	

_noEdit

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Translate Continuous Controllers for use as Matrix Sources
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			#matrixSources,R1	; Used for direct matrix row computations
	MOVE			#matrixPoPSources,R2	; Used for product-of-product matrix rows

	; Load and save positive CTRx fractional values

	MOVE			Y:globalCtr1,A
	MOVE			#Ctr1ParaFilt,R0
	JSR	ParaFilt_Filter
	MOVE			A,X:(R2)+
	MOVE			B,X:(R1)+

	MOVE			Y:globalCtr2,A
	MOVE			#Ctr2ParaFilt,R0
	JSR	ParaFilt_Filter
	MOVE			A,X:(R2)+
	MOVE			B,X:(R1)+

	MOVE			Y:globalCtr3,A
	MOVE			#Ctr3ParaFilt,R0
	JSR	ParaFilt_Filter
	MOVE			A,X:(R2)+
	MOVE			B,X:(R1)+

	MOVE			Y:globalMmod,A
	MOVE			#MmodParaFilt,R0
	JSR	ParaFilt_Filter
	MOVE			A,X:(R2)+
	MOVE			B,X:(R1)+

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Other control signals that need parameter-rate filtering
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	; Front panel volume

	MOVE			Y:globalMasterVol,A
	MOVE			#MvolParaFilt,R0
	JSR	ParaFilt_Filter
	MOVE			A,Y0
	MPYR	Y0,Y0,A		X:MVOL,X0		; Square the volume		MVOL ->
	SUB	X0,A					; This used directly in audio code, so interpolate
	MOVE			#MPanParaFilt,R0	; at audio rate as well as at parameter rate
	MOVE			A,Y:MVOL		; 				-> dMVOL

	; Midi pan

	MOVE			Y:globalMidiPan,A
	JSR	ParaFilt_Filter
	MOVE			A,Y:filteredMidiPan

	; Midi pitch wheel

	MOVE			Y:globalPitchShift,A	; Value due to midi pitch wheel
	MOVE			#PsftParaFilt,R0
	JSR	ParaFilt_Filter
	MOVE			A,Y:filteredMidiPitchShift

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Generate noise samples
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; These samples are required mainly for audio, but a few are also used
; for certain LFO waveforms, when gaussian noise is required. 
;
; The method here is to read from a large buffer of precalculated noise
; in DRAM. It has to be precalculated, in order to give it the required
; probability density function. However, samples can be read randomly
; without changing the PDF, thereby providing a non-repeating source of
; noise with the required characteristics. The samples are copied to a
; buffer of fast RAM, as a batch process here, where the randomisation  
; and copying can be performed in a single cycle per sample (plus a
; DRAM in-page wait state).
;
; Two random variables are defined: The offset into the source noise data,
; and the address increment that will be used. The increment is chosen
; randomly from the first 32 prime numbers. Increments are kept within 
; a 1K DRAM page (using modulo arithmetic) for each batch of samples,
; to avoid DRAM out-of-page wait states.
;
; The modulus is chosen as 1021, which is a prime, to avoid repetition
; within a page, though this is not guaranteed. You would need to 
; investigate the properties of multiples of 1021 with respect to the 
; first 32 primes, but it's probably okay when the number of samples
; generated in each batch is much less than 1021 (currently it's 256).
;
; Having placed the noise in fast RAM, the same method can then be 
; used to read out samples whenever they are needed. This randomly
; re-uses the same noise samples across voices, and there is unlikely 
; to be any audible or measurable audio artefact arising from this. 
;
; Noise sample re-use is necessary because polyphony requires a much 
; larger number of random noise samples. Since this noise can then be 
; liberally applied, this is probably a reasonable solution. Gaussian 
; dither can be applied wherever required, at a cost of slightly over 
; a single instruction cycle each time (estimated between 1 and 1.3).
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	; Generate two random numbers (using shift register method)

	MOVE        		X:rng,A      	; 			rng state
	LSL     A  		X:KnoiseMask,Y0
	NOP
	EOR     Y0,A  		A1,Y1
	TCS     Y1,A				; first rn

	MOVE			Y:Knoise1M,X1	; 			base address for source noise samples

	LSL     A 		A1,B1		;			first rn
	NOP
	EOR     Y0,A    	A1,Y1
	TCS     Y1,A 	 			; second rn
	LSR	#4,B				;			first rn is now unsigned 20bit offset
	ADD	X1,B		A1,X:rng	; source noise addr	store updated rng state

	; Randomly index a prime

	LSR	#4,A				; use the best bits
	AND	#$1F,A				; rn (0..31)
	MOVE			#primes,R0	;			base address of LUT containing first 32 primes
	MOVE			A1,N0

	; Prepare random read-out

	MOVE			#noise256,R4	; destination address for noise samples
	MOVE			B1,R1		; start location in source noise data
	MOVE			#1020,M1	; circular addressing (modulo 1021) within 1K DRAM page
	MOVE			X:(R0+N0),N1	; prime number increment through noise samples

	MOVE			R4,X0		; reset pointer to noise array
	MOVE			X:K7F,X1	; [+P]

	NOP

	; Fill fast-access noise buffer with randomly selected gaussian noise samples from DRAM

	MOVE					Y:(R1)+N1,A
	MOVE					Y:(R1)+N1,B

	DO	#256/2,_noiseLoop					; Fast noise array has 256 words

	MOVE			A,X:(R4)+	Y:(R1)+N1,A		; -> buffer		randomly selected value ->
	MOVE			B,X:(R4)+	Y:(R1)+N1,B		

_noiseLoop

	; Generate two more random numbers for randomly accessing the noise array in a single cycle
	
	; (A and B already have fresh random numbers from the last cycle of the previous loop)

	AND	#$1F,B				; Primes array has 32 entries
	AND	X1,A				; Noise array has 251 words, so select 0..127
	ADD	X0,A		B1,N0		; + Noise array base address	offset in primes
	NOP
	MOVE			A1,X:noisePtr	; Used to pre-load Rx (NB use A1, not A, AND is 24 bit only operation)
	MOVE			X:KFFFFFF,M1	; [+P] Restore modifier register
	MOVE			X:(R0+N0),X1	; Choose randomly from first 32 primes
	MOVE			X1,X:noiseInc	; Used to pre-load Nx (put 250 (251 = prime) in Mx)

;--------------------------------------------------------------------------
; Sound matrix 
;--------------------------------------------------------------------------

	; Redundancy in the modulation matrix is exploited to reduce computation.

	; In these matrix operations, the contributions of sources that are 
	; common to all voices (CTR1-3 and MMOD) are pre-calculated. The 
	; results will be used as initial values in each voice matrix.

	; Modulation depths

	MOVE			#soundData,R4			; Depths corresponding to first set
	MOVE			#16-(MMOD-CTR1),N4		; To go from g(MMOD) to h(base)

	; Source addresses

	MOVE			#matrixSources+CTR1,R0		; First set of modulation sources
	MOVE			#CTR1-MMOD,N0			; To go from MMOD to CTR1 (-ve)

	; Destinations

	MOVE			#soundStructurePre,R2		; "Pre-calculated Destinations"

	; Matrix

	MOVE					Y:(R4)+,A	; 			First base value
	MOVE			X:(R0)+,X0	Y:(R4)+,Y0	; 	    source	gain

	DO	#nDirect,_smatrix1				; direct sum-of-products matrix

	MAC	X0,Y0,A		X:(R0)+,X0	Y:(R4)+,Y0	; CTR1
	MAC	X0,Y0,A		X:(R0)+,X0	Y:(R4)+,Y0	; CTR2
	MAC	X0,Y0,A		X:(R0)+N0,X0	Y:(R4)+N4,Y0	; CTR3
	MAC	X0,Y0,A						; MMOD

	MOVE			X:(R0)+,X0	Y:(R4)+,Y1	; 	    source	next base value		
	TFR	Y1,A		A,X:(R2)+	Y:(R4)+,Y0	;	    result	gain (g1 in next section)
	
_smatrix1

	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        ; Derivation of "product-of-products" matrix
        ;
        ; 	x'  =  x (1 - g1) + x .g1.s1
        ; 	x'' =  x'(1 - g2) + x'.g2.s2
        ; 	.
        ;	.
        ;
        ; if pn = 1 - gn, then the row result, r, is given by:
        ;
        ;	r = x(p1 + g1.s1)(p2 + g2.s2)...(pn + gn.sn)
        ;
        ; In the special case of gn < 0, pn = 1 is used.

	; Source addresses

	MOVE			#matrixPoPSources+CTR1,R0	; First set of modulation sources

	; Additional modulation depth data

	MOVE			#soundPnData,R3			; precalculated pn = 1 - gn array
	MOVE			#16-(MMOD-CTR1),N3		; To go from q(pn(MMOD)) to r(pn(CTR1))

	; Matrix

	MOVE			A,X1				; Initial value to be multiplied
	MOVE			X:(R0)+,X0			; (next g1 already loaded)

	DO	#nPoPs,_smatrix2				; product-of-products matrix

	MPY	X0,Y0,A	  	X:(R3)+,X0			; s1.g1		p1
	ADD	X0,A	  	X:(R0)+,X0	Y:(R4)+,Y0      ; p1 + s1.g1	s2		g2		CTR2
	MPY	X0,Y0,B   	X:(R3)+,X0			; s2.g2		p2
	ADD	X0,B	  	A,Y1				; p2 + s2.g2	(p1 + s1.g1)
  
	MPY	X1,Y1,A		X:(R0)+,X0	Y:(R4)+,Y0      ; .(p1 + s1.g1)	s3		g3		CTR3
  	MPY	X0,Y0,B		X:(R3)+,X0	B,Y1		; s3.g3		p3		(p2 + s2.g2)	
	ADD	X0,B	 	A,X1				; p3 + s3.g3    .(p1 + s1.g1)			
	
	MPY	X1,Y1,A		X:(R0)+N0,X0	Y:(R4)+N4,Y0 	; MMOD
  	MPY	X0,Y0,B		X:(R3)+N3,X0	B,Y1		
	ADD	X0,B	 	A,X1			
	
	MPY	X1,Y1,A	
	MOVE			B,Y1	
	MOVE			A,X1

	MPY	X1,Y1,A				Y:(R4)+,X1	; .(p6 + s6.g6)			next base value
 
	MOVE			X:(R0)+,X0	Y:(R4)+,Y0	; 		next s1 	next g1
	MOVE			A,X:(R2)+			;		store result

_smatrix2
 
;--------------------------------------------------------------------------
; Per-voice calculations
;--------------------------------------------------------------------------
; In the voice loop, we first calculate the modulation matrix. That gives 
; a full set of parameters that can then be translated. Rather than 
; preserve this full set of parameters for each voice, consuming memory, 
; once translated, these are discarded, and the same memory used by the 
; next voice. 
;--------------------------------------------------------------------------

	MOVE			#voiceState,A0
	MOVE			#vMatrixSources,A1
	MOVE			#vInterpolatedParas,B0
	MOVE			#NISampleParas,B1

	MOVE			A0,X:voiceStateBaseAddress
	MOVE			A1,X:voiceMatrixBaseAddress
	MOVE			B0,X:voiceInterpolatedParasBaseAddress
	MOVE			B1,X:voiceNISampleParasBaseAddress

	DO	#N_VOICES_PROC,paraVoiceLoop

;--------------------------------------------------------------------------
; Note event processing
;--------------------------------------------------------------------------

	MOVE			X:voiceStateBaseAddress,R3	; Y: voice assign states
	MOVE			#2,N3				; for jumping past vFilteredPitch
	MOVE			X:voiceMatrixBaseAddress,R4	; Base of voice-unique matrix sources
	CLR	A		#vMatrixPoPoffset,N4		; To skip between direct & PoP sources

	ORI	#$03,MR		; Disable interrupts

	MOVE			Y:(R3),Y0			; 		iOnOff
	TFR	Y0,A		A,Y:(R3)+			; 		clear iOnOff
	MOVE			Y:(R3)+,X0			; 		iVoice
	MOVE			Y:(R3)+,X1			;		iPitch
	MOVE			Y:(R3)+,B			;		iVelocity

	ANDI	#$FC,MR		; Re-enable interrupts as soon as possible

	MOVE			R3,X:voiceStatePtr		;		now pointing to vPitch
	MOVE			A,Y:vOnOff			;		stable OnOff flag

	; Always copy iPitch to vPitch in case the host has shifted the pitch 

	MOVE			X1,Y:(R3)			;		Y:vPitch	

	; At this point, R3 is pointing to Y:vPitch for this voice

	LUA			(R3+vAssignStateSize+vLFOstateBlockSize-vPitch),R0 ; ENV state base address
	TST	A		#ENVstateSize,N0

	JEQ	_done
	JMI	_off

	; New note on event - - - - - - - - - - - - - - - - - - - - - - -

	; Note that we don't update matrix sources KFLW, KVEL, VDIF & STDF
	; directly at this point. Changes to these values should synchronise
	; with the start of the new envelope attack phase, so the sources
	; are all updated at the same time, ie, at the usual point in the 
	; parameter cycle where the ENV1 and ENV2 sources are updated. There
	; is an exception here, because ENV1&2 can be forced to zero on a
	; staccato new note, but still the attack phase actually starts at 
	; the usual place.

	; The voice number, and VDIF and STDF values, are in Y:iVoice 
	; (voice number is redundant at this point, could be useful)
	:
	; MS byte : mid byte : LS byte
	;   STDF  :   VDIF   :  voice    (all these values are 0..127)

	; Update vVelocity

	MOVE			(R3)+N3				; skip over vFilteredPitch
	TFR	X0,A		B,Y:(R3)+			; vVelocity	

	; Update most recent voice, voice diff and stack diff from Y:iVoice 

	AND	#>$FFFF00,A					; discard LSB 
	ASR	#16,A,B
	AND	#>$FF0000,A
	CLR	B		B0,Y:(R3)+			; [+p]		vVoiceDiff
	MOVE			A,Y:(R3)+			; 		vStackDiff

	; Update envelopes - - - - - - - - - - - - - - - - - - - 

	MOVE			B,Y:(R3)+			; clear vMute, activate voice
	MOVE			X:K2,B	
			
	; Legato for monophonic/portamento playing? This is signified by an iOnOff value of 2.
	; If not, reset the envelope levels to zero, so the attack phase starts quiet,
	; which is the preferred style for polyphonic voice assignment

	MOVE			  (R0)+
	MOVE			Y:(R0)+N0,A			;		ENV1_level
	CMP	Y0,B		Y:(R0)-N0,B			; iOnOff==2 ?	ENV2_level
	CLR	A	IFNE					; if not, clear ENV levels 
	CLR	B	IFNE
	MOVE			A,Y:(R0)+N0
	MOVE			B,Y:(R0)-N0

	MOVE			X:KENVattackPhase,X0		; 	 	Legato attack phase

	MOVE			Y:-(R0),B			; 		[+P] previous envelope phase
	MOVE			X0,Y:(R0)+N0			; 		ENV1_phase
	MOVE			X0,Y:(R0)-N0			; 		ENV2_phase

	; If the new note comes before the old note was released, 
	; then always use legato; else if portamento is zero, use staccato.

	CMP	#>ENVreleasePhase,B
	MOVE			Y:(R3)+,B			; 		get vLegato
	JNE	_done						; Not in release phase, so use legato

	; Envelope was in release phase, use legato or staccato?

	TST	B		X:KENVstaccatoPreAttackPhase1,X0 ; vLegato set?	[+p]
	JNE	_done

	; Staccato: initiate envelope sequence that will optionally reset audio oscillators
	; in a subsequent parameter cycle

	MOVE			X0,Y:(R0)+N0			; ENV1_phase
	MOVE			X0,Y:(R0)-N0			; ENV2_phase

	MOVE			X:voiceMatrixBaseAddress,R4	; Base of voice-unique matrix sources
	MOVE			#vMatrixPoPoffset,N4		; To skip between direct & PoP sources
	
	; Reset the envelope levels to zero (to take effect on next parameter cycle)
	
	MOVE			     (R0)+				
	MOVE			Y0,Y:(R0)+N0			; ENV1_level
	MOVE			Y0,Y:(R0)-N0			; ENV2_level
	MOVE			     (R0)-

	; Set the voice matrix ENV sources to zero. This ensures that the voice matrix 
	; in the current cycle will use zero envelope values, and the next audio cycle 
	; will incorporate any changes that result, minimising staccato note-on latency.

	MOVE			      (R4)+N4
	MOVE			Y0,X:-(R4)			; vENV2 (direct)
	MOVE			Y0,X:-(R4)			; vENV1 (direct)
	MOVE			     (R4)+N4
	MOVE			Y0,X:(R4)+			; vENV1 (PoP)
	MOVE			Y0,X:(R4)+			; vENV2 (PoP)
	
	JMP	_done

	; Note off event - - - - - - - - - - - - - - - - - - - - - - - - -

_off	MOVE			X:KENVreleasePhase,X0
	MOVE			X0,Y:(R0)+N0			; ENV1_phase
	MOVE			X0,Y:(R0)-N0			; ENV2_phase
_done	

;--------------------------------------------------------------------------
; Oscillator sync pre-processing
;--------------------------------------------------------------------------

	; Here we look at the envelope phase, which has a special value 
	; in the parameter cycle where oscillator synchronisation should
	; occur. We set a flag which can be examined in the oscillator loop.

	MOVE			Y:(R0),A
	CMP	#>ENVstaccatoPreAttackPhase2,A
	CLR	A	IFNE
	NOP
	MOVE			A,Y:oscPhSyn			; zero if no sync

;--------------------------------------------------------------------------
; Voice matrix 
;--------------------------------------------------------------------------
 
	; Here we have already calculated the part of the matrix that is 
	; common to all voices. This includes the base values (the 
	; first of the seventeen values for each parameter). 

	MOVE			#soundStructurePre,R6		; "Pre-calculated Destinations"

	; Source addresses

	MOVE			X:voiceMatrixBaseAddress,R0	; Direct modulation sources (vKFLW etc)
	MOVE			#(vKFLW-vENV2),N0		; To go from vENV2 to vKFLW (-ve)

	; Modulation depths

	MOVE			#soundData+KFLW+1,R4		; Depths corresponding to first set
	MOVE			#17-(ENV2-KFLW),N4		; To go from g(ENV2) to h(KFLW)

	; Destinations

	MOVE			#soundStructure,R2		; Matrix outputs
	NOP

        ; - - - - - - - - - - - - - - - - - - - - - - - - - - - -          
	; Sum-of-products (direct) matrix 
	
	MOVE			X:(R6)+,X1			; 	    First base value
	TFR	X1,A		X:(R0)+,X0	Y:(R4)+,Y0	; 	    source	gain

	DO	#nDirect,_vmatrix1				; direct sum-of-products matrix

	MAC	X0,Y0,A		X:(R0)+,X0	Y:(R4)+,Y0	; KFLW
	MAC	X0,Y0,A		X:(R0)+,X0	Y:(R4)+,Y0	; KVEL
	MAC	X0,Y0,A		X:(R0)+,X0	Y:(R4)+,Y0	; VDIF
	MAC	X0,Y0,A		X:(R0)+,X0	Y:(R4)+,Y0	; STDF
	MAC	X0,Y0,A		X:(R0)+,X0	Y:(R4)+,Y0	; LFO1
	MAC	X0,Y0,A		X:(R0)+,X0	Y:(R4)+,Y0	; LFO2
	MAC	X0,Y0,A		X:(R0)+,X0	Y:(R4)+,Y0	; LFO3
	MAC	X0,Y0,A		X:(R0)+,X0	Y:(R4)+,Y0	; LFO4
	MAC	X0,Y0,A		X:(R0)+,X0	Y:(R4)+,Y0	; LFO5
	MAC	X0,Y0,A		X:(R0)+,X0	Y:(R4)+,Y0	; LFO6
	MAC	X0,Y0,A		X:(R0)+N0,X0	Y:(R4)+N4,Y0	; ENV1
	MAC	X0,Y0,A		X:(R6)+,X1			; ENV2	    Next base value
	
	ASL	#3,A,B						; matrix gain = 8

	MOVE			X:(R0)+,X0			; 	    source
	TFR	X1,A		B,X:(R2)+	Y:(R4)+,Y0	;	    result	gain (g1 in next section)
	
_vmatrix1

	; Source addresses

	LUA			(R0+vMatrixPoPoffset-1),R0	; PoP modulation sources (vKFLW etc)

	; Additional modulation depth data

	MOVE			#soundPnData+KFLW,R3		; precalculated pn = 1 - gn array
	MOVE			#16-(ENV2-KFLW),N3		; To go from q(pn(ENV2)) to r(pn(KFLW))

        ; - - - - - - - - - - - - - - - - - - - - - - - - - - - -          
	; Product-of-Products (PoP) matrix 

	MOVE			A,X1				; Initial value to be multiplied
	MOVE			X:(R0)+,X0			; (next g1 already loaded)

	DO	#nPoPs,_vmatrix2				; Product-of-products matrix 

	MPY	X0,Y0,A	  	X:(R3)+,X0			; s1.g1		p1
	ADD	X0,A	  	X:(R0)+,X0	Y:(R4)+,Y0      ; p1 + s1.g1	s2		g2		KVEL
	MPY	X0,Y0,B   	X:(R3)+,X0			; s2.g2		p2
	ADD	X0,B	  	A,Y1				; p2 + s2.g2	(p1 + s1.g1)
  
	MPY	X1,Y1,A		X:(R0)+,X0	Y:(R4)+,Y0      ; .(p1 + s1.g1)	s3		g3		VDIF
  	MPY	X0,Y0,B		X:(R3)+,X0	B,Y1		; s3.g3		p3		(p2 + s2.g2)	
	ADD	X0,B	 	A,X1				; p3 + s3.g3    .(p1 + s1.g1)			
	
	DUP	8

	MPY	X1,Y1,A		X:(R0)+,X0	Y:(R4)+,Y0 	; STDF, LFO1, LFO2, LFO3, LFO4, LFO5, LFO6, ENV1
  	MPY	X0,Y0,B		X:(R3)+,X0	B,Y1		
	ADD	X0,B	 	A,X1			
	
	ENDM

	MPY	X1,Y1,A		X:(R0)+N0,X0	Y:(R4)+N4,Y0 	; ENV2
  	MPY	X0,Y0,B		X:(R3)+N3,X0	B,Y1		
	ADD	X0,B	 	A,X1			
	
	MPY	X1,Y1,A
	MOVE			B,Y1			
	MOVE			A,X1

	MPY	X1,Y1,A		X:(R6)+,X1			; .(pz + sz.gz)	next pre-calculated value
	ASL	#3,A,A						; matrix gain = 8
	
	MOVE			X:(R0)+,X0	Y:(R4)+,Y0	; 		next s1 	next g1
	MOVE			A,X:(R2)+			;		store result

_vmatrix2

;--------------------------------------------------------------------------
; Translate Matrix Outputs
;--------------------------------------------------------------------------

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Comments marked with a  indicate a translation from a switched user 
; parameter. The modulation matrix treats these as continuous parameters, 
; having the full range of possible values. The matrix output must then be 
; translated back into small integers. This is done using Equation #2 from 
; ../java/switching/Switching.java, where these calculations are documented.
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE		X:voiceInterpolatedParasBaseAddress,R4	; -> X/Y:vInterpolatedParas
	MOVE		#FLTcut,R1				; -> X: Direct matrix outputs
	MOVE		#FLTovdr,R2				; -> X: PoP matrix outputs

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Filter Translations
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	; This is a stereo filter, but the same parameters are used for both channels
	
	; The filter is calculated before the oscillators, because a pre-filter 
	; mix factor is derived from the filter overdrive parameter, and this 
	; must be known when calculating the oscillator output levels.

	; At this point, registers should be set as follows:
	; R1 -> FLTcut (direct matrix output)
	; R2 -> FLTovdr (PoP matrix output)
	; R4 -> vInterpolatedParas

	; Cutoff

	MOVE			X:K7FFFFF,X0		;			1
	LUA			(R4+fltFa),R5		;			-> fltFa
	TFR	X0,B		X:(R1)+,A		;			FLTcut 
	SUBL	B,A		Y:globalMasterTune,X0	; (-1..+1)		(cents.256)

	; Middle C = 60, and this is half way through the LUT. So the base of the LUT is not 
	; note zero, it is: 60 - 32768/400 = note -21.92 (semitones), and this needs to be added:

	ADD	X0,A		X:K8768T256,X0		; + master tuning	(21.92 * 400) << 8
	ADD	X0,A		#3,N5

	; Non-linear coefficients. A look-up table takes care of the slight pitch non-linearity,
	; and translates cent pitch values directly into filter coefficients for each stage.
	; A resonance correction value is also provided, to correct for the variation in 
	; loop gain at different cutoff frequencies.

	MOVE	X:KfltCoeffLUT12dB,X0			; coefficient LUT base address (mid point)

	MOVE			A,B			; 			Limit address
	ASR	#8,B,B					; div by 256 to form address
	LSL	#3,B					; 8 entries per row in LUT
	ADD	X0,B					; + base address of coefficient LUT
	MOVE			#3,N0			;			To skip unused Fc and Fd
	MOVE			B,R0			;			Pitch as address
	NOP
	NOP

	; The filter coefficient LUT stores data in this order: Fa, Fb, Fc, Fd, ResC, x, x, x 
	; In this version, we don't use Fc and Fd, as it's only a 2 stage filter

	MOVE			X:(R5)+,Y0		;			old Fa
	MOVE			Y:(R0)+,A		;			target Fa
	SUB	Y0,A		X:(R5)-,Y0		;			old Fb
	MOVE			Y:(R0)+N0,B		;			target Fb
	SUB	Y0,B		A,Y:(R5)+		;			-> dFa
	MOVE			Y:(R0),Y0		;			[+P] resonance correction factor
	MOVE			B,Y:(R5)+		;			-> dFb

	; Overdrive 

	; The overdrive parameter is used to determine two things: the level of pre-mix
	; supplied to the filter, and also the level of non-linearity in the transfer
	; function (the n in x - 2(nx)). The level of pre-mix only varies signifcantly
	; at low overdrive settings - basically so that it's possible to run the filter
	; fairly linear, even though distortion is preferable in most sounds. The level
	; function is a complex polynomial, so an LUT is used. The factor is then applied
	; to all the pre-mix levels, so that the audio code only does one sum of products.

	; The value of n varies as 0.55 .. 1.0 for overdrive range 0.0 .. 1.0 
	; See java/distortion/Distortion.java 

	MOVE			X:(R2)+,X0		;			FLTovdr
	TFR	X0,A		X:KP45,Y1		;			1 - 0.55
	ASR	#7,A,A
	CLR	A		IFMI			; min=0
	MPYR	X0,X0,B		X:KfltOvLUT64K,R0	; overdrive

	MOVE			A,N0
	MOVE			B,X1			;			overdrive
	MPY	X1,Y1,A		X:KP55,Y1		; 0.45 x overdrive
	ADD	Y1,A		X:(R5),Y1		; n = 0.55 .. 1.0	old fltOn
	SUB	Y1,A					; interpolation diff
	MOVE			X:(R0+N0),X0	
	MOVE			A,Y:(R5)-N5		;			-> dfltOn [+P] -> (fltRs)
	MOVE			X0,X:preMixGain		;			This value used to modify premix levels

	MOVE			X:(R1)+,Y1		;			FLTres
	MPYR	Y0,Y1,A					; frequency-compensated resonance
	CLR	A		IFMI			; min=0

	MOVE			X:(R5),Y0		; 			Rs

	SUB	Y0,A		X:voiceInterpolatedParasBaseAddress,R4		; [+P] -> X/Y:vInterpolatedParas
	MOVE			X:voiceNISampleParasBaseAddress,R6		; [+P] -> XYL:NISampleParas

	MOVE			A,Y:(R5)		;			-> dRs

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Prepare addresses 
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			#O1pitch,R1		; -> X: Direct matrix outputs
	MOVE			#O1level,R2		; -> X: PoP matrix outputs: O1level, O2level, O3level

	MOVE			#2,N3
	MOVE			#O1R-O1L,N4		; (Rx) -> L: O1L, O2L, O3L; (Rx+Nx) -> L: O1R, O2R, O3R

	LUA			(R6+O1freq),R3		; -> XYL: various, from O1freq (see memory.asm)

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Oscillator Translations 
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	DO	#3,_oscTransLoop			; 3 oscillators

	; Pitch conversion

	MOVE			Y:globalMasterTune,B	; 			Master tune pitch offset in cents * 256
	MOVE			X:(R1)-,A		; 			Oxpitch 	
	ADD	B,A		X:KP2F64K,R0		; Add master tune 	Pitch to frequency LUT mid address

	MOVE			X:(R1)+,Y1		;			Oxoctave
	MOVE			A,A			;			Limit result of master tune pitch offset
	ASR	#8,A,A					; Address for P2F
	MOVE			(R1)+			;			-> (Oxoffset)
	MOVE			A1,N0

	CLR	B		X:KP08193125,Y0		;  q' for octave: -10..+10
	MPYR	Y0,Y1,A		#8,B1			;  xq' 		somewhat arbitrary octave offset
	ASR	#16,A,A					;  (xq' >> 16) 
	RND	A		#3,N1			;  k = 0, so add 0.5

	SUB	A,B		Y:(R0+N0),A		; freq right shifts 	P2F lookup

	MOVE			#0,A2			;			(the P2F output is unsigned)
	ASR	B1,A,A					; Oscillator octave shift
	ASL	#8,A,A					; Remove shift offset

	; Add frequency offset

	; To get 80Hz, divide the fsd value by the nyquist frequency and multiply by 80.
	; So divide the value by ns (=24000 @fs48k) and multiply by 80 -> / 300.
	; (Nyquist instead of fs, because phase cycles through -1..+1)
	; (80Hz, because of UI limit = integer 8000)

	MOVE			X:(R1)+N1,Y1		; Oxoffset: 80Hz = $7FFFFF
	MOVE			X:KP003333333,X0	; 1/300
	MPY	X0,Y1,B					; Scale frequency offset
	ADD	B,A		#0.25,Y0		; Add pitch-derived frequency 	[+P] Phase reset value

	; Phase synchronisation (optional reset oscillator phase on staccato new note)

	MOVE			X:(R1)-,B		;			Oxphsyn: User wants phase reset? (-ve -> no)		
	MOVE			A10,L:(R3)-		; 			[-p] store freq for oscillator
	TST	B		L:(R3),A		; Test Oxphsyn		Current oscillator phase value
	BMI	_noRst	

	MOVE			Y:oscPhSyn,B		;			A flag set earlier in the voice loop
	TST	B					; Staccato new note?
	TNE	Y0,A					; Conditionally update phase on new note
	NOP
_noRst	MOVE			A,L:(R3)+N3		;			Possibly a new oscillator phase value
 
	; FM source and depth

	MOVE			X:(R1)-,A		;			Oxfm
	MOVE			X:(R1)-N1,Y1		; 			Oxfmsrc

	MOVE			X:KP03505625,Y0		; 			 q' for 0..8
	MPYR	Y0,Y1,B		X:KModSources,Y0	;  xq'	      		LUT base address
	ASR	#16,B,B					;  xq' >> 16  
	ADD	#4,B					;  then add 4 -> 0..8
	RND	B		(R1)-			;  k = integer, so add 0.5

	LSL	B		#4,N1			; -> 0..16 (LUT has double entries)
	ADD	Y0,B					; + LUT base address
	NOP

	TST	A		B,R0			; Test Oxfm
	CLR	A	IFMI				; Oxfm: >= 0
	MOVE			R6,X0			;			base address for voice samples
	MOVE			A,X:(R3)-N3		; 			Oxzfm

	MOVE			X:(R0)+,A		; 			source address type (relative/absolute)	
	TST	A		X:(R0)+,A		;			source address selection	
	ADD	X0,A	IFNE                   		; Select relative or absolute signal address

	; Phase offset between oscillator pair (user parameter "phase", DSP parameter "shape")

	MOVE			X:-(R1),B		;			Oxphase (matrix output)
	MOVE			A1,X:-(R3)		;			[-P] Oxzfmsrc

	; Reduced matrix gains for phase offset permit phase wraparound

	ASL	#3,B,B					;			User values were divided by eight
	MOVE			X:(R4),Y0		;			Oxzshape (dsp parameter)
	MOVE			B1,A			;			Wrap matrix phase

	; Interpolating between wrapped values is done by comparing their sum and difference.
	; If the sum has a lower magnitude, negate it and use this instead.

	ADD	Y0,A		B1,B			; sum
	SUB	Y0,B		(R1)-			; diff			[+P] -> (Oxbright)	
	CMPM	A,B		X:(R1)+,X0		; compare magnitudes	[+P] Oxbright
	TFR	A,B	IFGT				; use sum if smaller
	NEG	B	IFGT				; and negate it

	; Fixed frequency follower for self phase modulation (brightness)
	
	MOVE			X:K7FFFFF,A		;			nyquist =fs/2
	MOVE			X:(R3+N3),Y0		; 			= osc freq (as used by osc)
	SUB	Y0,A		B,Y:(R4)+		; nyquist - osc freq	[-P] dOxzshape, -> (Oyzshape)
	MOVE			  (R1)+			;			
	MOVE			X:(R1)-,B	A,Y0	;			[+P] Oxwave	nyq-frq
	MPYR	X0,Y0,A		  (R1)+N1		; br follower 		[+P]
	CLR	A	IFMI				; All negative PM values -> zero

	; Select wavetype (saw or square)

	TST	B		X:K1,B
	CLR	B	IFMI

	MOVE			X:(R3)+N3,A	A,Y0	; 	 		(dummy move)	PM
	MOVE			  (R3)+N3		;			->(Oxfrqshift)
	MOVE			B,X:(R3)-		; Scale PM for sawt.	Oxfrqshift = freq shift for osc a (0 or 1)
	MOVE			Y0,Y:(R3)+N3		;			OxPM 

	; Oscillator level and pan

	MOVE			  (R1)+N1
	MOVE			X:(R2)+,X0		;			Oxlevel [+P] -> (Oylevel)
	MOVE			X:(R1)+N1,X1		;			Oxpan  
	
	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
	; Translate Pan 
	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
	; On entry: 	X0 = level (0..1)
	; 	 	X1 = pan (-1..+1)
	;
	; On exit:	A = left gain 
	;		B = right gain
	;
	; Modified:	A,B,X,Y,R0
	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	MPY	X1,#14,A
	MPYR	X0,X0,B		#panLUT1K,R0	; Square the volume		pan power compensation LUT
	TFR	X1,A		A1,N0		; Pan -1..1			pan LUT addr (signed)
	ASR	A		B,X1		; Pan -..			uncompensated squared level factor

	; Calculate left (1..0) and right (0..1) factors from pan

	TFR	A,B		B,Y0		;				squared direct volume
	NEG	A		#.5,Y1		; Pan ..-
	ADD	Y1,B				; Pan 0..1 (right)		
	ADD	Y1,A		Y:(R0+N0),Y1	; Pan 1..0 (left)		pan power compensation factor

	; Compensate levels and calculate left and right gains

	MPYR	X1,Y1,B		B,X1		; Compensate sqrd level		pan right
	MOVE        		A,X0		;                 		pan left
	MOVE        		B,Y0		;                       	compensated squared level

	; Calculate squared level gains

	MPYR	X0,Y0,A		(R4)+  		; left gain (sqrd lvl)		[+P] (nothing to do with this function)
	MPYR	X1,Y0,B		(R4)+ 		; right gain (sqrd lvl)		[+P] ( " )

	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	MOVE			X:preMixGain,X0		;			(from filter overdrive)

	MOVE			A,Y0	X:(R3)+N3,X1	;			left gain	[+P] dummy move
	MPY	X0,Y0,A		B,Y0			;			right gain
	MPY	X0,Y0,B		X:(R4)+N4,X1   		;      	 		OxL
	SUB	X1,A		X:(R4)-N4,X1		; 			OxR
	SUB	X1,B		(R3)+			;					[+P] -> (Oyfreq)			
	MOVE			A,Y:(R4)+N4		;			dOxL
	MOVE			B,Y:(R4)-N4		;			dOxR

	MOVE			(R4)-
	MOVE			(R4)-
	MOVE			(R1)+	

_oscTransLoop

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Prepare addresses
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			#RMlevel-FLTovdr,N2
	MOVE			(R4)+N4
	MOVE			(R4)-
	MOVE			(R4)-			;			-> L: NGLR
	MOVE			(R3)-N3			;			-> RMzsrca
	MOVE			(R1)-N1			;			-> RMsrca
	MOVE			#RMpan-FLTcut,N1

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; At this point:
;	
;	R1->RMsrca
;	R2->NGlevel
;	R3->RMzsrca
;	R4->NGLR
;
; debug	MOVE		R1,Y:test1
; debug	MOVE		R2,Y:test2
; debug	MOVE		R3,Y:test3
; debug	MOVE		R4,Y:test4

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Noise Generator
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	; The noise generator only has a level parameter. The noise supplied
	; to the filter is always stereo gaussian. A minimum noise level is 
	; defined, in order to dither the pre-filter mix even when the
	; noise level is zero.

	MOVE			X:(R2)+,X0		;			NGlevel (from matrix)
	MPYR	X0,X0,A					; level
	MOVE			X:preMixGain,X1
	MOVE			A,Y0
	MPYR	X1,Y0,A		X:KP000001,Y0		; pmg.level		minimum (dither level)
	CMP	Y0,A		X:(R4),X0		;			NGLR
	TMI	Y0,A					; enforce minimum
	SUB	X0,A		X:(R2)+,X0		;			[+P] EXTlevel
	MPYR	X0,X0,B					; [+P] level
	MOVE			A,Y:(R4)+		; [+P] RMsrca scaling	dNGLR [+P] -> (EXLR)

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; External input
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			B,Y0
	MPYR	X1,Y0,A		X:(R4),X0		; pmg.level		EXLR
	SUB	X0,A		X:(R1)+,X0		;			[+P] RMsrca
	MOVE			X:KP03505625,Y0		; [+P]  q' for 0..8
	MPYR	X0,Y0,A		A,Y:(R4)-		; [+P]  xq'		dEXLR

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Ring Modulator
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	; Select sources

	ASR	#16,A,A					;  xq' >> 16  
	RND	A		X:KModSources,Y1        ;  k = 4, so add 0.5
	ADD	#4,A			                ;  then add 4 -> 0..8

	LSL	A               			; -> 0..16 (LUT has double entries)
	ADD	Y1,A					; + LUT base address

	MOVE			X:(R1)+,X0		;			RMsrcb
	MPYR	X0,Y0,A		A,R0			;  xq'		
	ASR	#16,A,A					;  xq' >> 16
	RND	A		(R4)-		        ;  k = 4, so add 0.5	[+P] -> (RML)
	ADD	#4,A			                ;  then add 4 -> 0..8

	LSL	A               R6,X0			; -> 0..16 (LUT has double entries)
	ADD	Y1,A					; + LUT base address

	MOVE			X:(R0)+,B		; 			source address type (relative/absolute)
	TST	B		X:(R0)+,B      		;			source address selection	
	ADD	X0,B	IFNE                   		; Add if address is relative
	
	MOVE			A,R0
	MOVE			B1,X:(R3)+		;			RMzsrca
	MOVE			#5,N3			;			[+P]

	MOVE			X:(R1)-N1,X1		; 			[+P] RMpan	
	
	MOVE			X:(R0)+,B		; 			source address type (relative/absolute)
	TST	B		X:(R0)+,B      		;			source address selection	
	ADD	X0,B	IFNE                   		; Add if address is relative
	
	MOVE			X:(R2)-N2,X0		;			[+P] RMlevel, [+P] -> (FLTovdr)
	MOVE			B1,X:(R3)+N3		;			RMzsrcb

	; Level & Pan

	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
	; Translate Pan 
	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
	; On entry: 	X0 = level (0..1)
	; 	 	X1 = pan (-1..+1)
	;
	; On exit:	A = left gain 
	;		B = right gain
	;
	; Modified:	A,B,X,Y,R0
	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	MPY	X1,#14,A
	MPYR	X0,X0,B		#panLUT1K,R0	; Square the volume		pan power compensation LUT
	TFR	X1,A		A1,N0		; Pan -1..1			pan LUT addr (signed)
	ASR	A		B,X1		; Pan -..			uncompensated squared level factor

	; Calculate left (1..0) and right (0..1) factors from pan

	TFR	A,B		B,Y0		;				squared direct volume
	NEG	A		#.5,Y1		; Pan ..-
	ADD	Y1,B				; Pan 0..1 (right)		
	ADD	Y1,A		Y:(R0+N0),Y1	; Pan 1..0 (left)		pan power compensation factor

	; Compensate levels and calculate left and right gains

	MPYR	X1,Y1,B		B,X1		; Compensate sqrd level		pan right
	MOVE        		A,X0		;                 		pan left
	MOVE        		B,Y0		;                       	compensated squared level

	; Calculate squared level gains

	MPYR	X0,Y0,A		    		; left gain (sqrd lvl)		                 
	MPYR	X1,Y0,B		     		; right gain (sqrd lvl)		                           

	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	MOVE			X:preMixGain,X0
	MOVE			A,Y0
	MPY	X0,Y0,A		B,Y0
	MPY	X0,Y0,B		X:(R4)+N4,X0   		;      	 		RML
	SUB	X0,A		X:(R4)-N4,X0		; 			RMR
	SUB	X0,B		#2,N3			;					(restore inc)
	MOVE			A,Y:(R4)+N4		;			dRML
	MOVE			B,Y:(R4)+N4		;			dRMR

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Mute processing
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	
	MOVE			X:voiceStatePtr,R3	; 			-> Y:vPitch
	MOVE			#vMute-vPitch,N3

	; (Perform address updates unrelated to mute processing)
	
	MOVE			(R4)-
	MOVE			R4,R5
	MOVE			R4,R6			; 			(VLL)

	; Mute processing

	MOVE			Y:(R3+N3),A
	TST	A		X:SNDvolume,A
	CLR	A	IFNE				; If mute is set, volume is zero

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Stereo positioning for output
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			Y:filteredMidiPan,B
	MOVE			X:SNDpan,X0
	ADD	X0,B		X:SNDbalance,R1		; Add midi pan		Balance
	MOVE			X:SNDspread,R2		;			Spread
	MOVE			B,R3			; 			Pan
	MOVE			A,R4			;			Level
	
	MOVE			(R6)+
	MOVE			(R6)+			;			-> (VLR)

	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
	; Translate Spread, Balance, Pan and Level 
	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
	; This function is used to pan a stereo pair. Four parameters define the
	; position: spread, balance, pan, level. Negative values of spread are 
	; valid, and have the effect of swapping a and b positions. The space 
	; between a and b is inversely proportional to |pan|, so that neither 
	; a nor b hits a position limit for values of |pan| less that one. This 
	; ensures that the average of pana and panb is always equal to pan, 
	; regardless of the amount of spread, and changes made in real time 
	; result in a smooth rate of change.
	;
	; Spread is applied according to:
	;
	;	pana = pan + |pan|.spread - spread
	;	panb = pan - |pan|.spread + spread
	;
	; Balance is a straightforward parameter that adjusts the relative 
	; levels of a and b, after the level has been squared:
	;
	;	levela = (1-balance).level
	;	levelb = (1+balance).level
	;
	; pana and panb are then combined with levela and levelb 
	; in the usual pan relationship:
	;
	;	aL = (1-pana).levela.f(pan)
	;	aR = (1+pana).levela.f(pan)
	;	bL = (1-panb).levelb.f(pan)
	;	bR = (1+panb).levelb.f(pan)
	;
	; where f() is the pan power compensation curve. 
	;
	; On entry:	R1 = balance
	; 		R2 = spread
	;		R3 = pan
	;		R4 = level
	;
	;		R5 -> L: aL, bL	(interpolated paras)
	;		R6 -> L: aR, bR
	;
	; On exit:	interpolated paras updated
	;
	; Modified:	A,B,X,Y,R0-R6,N1-N4
	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	; pana = pan + |pan|.spread - spread

	MOVE			R3,A		;				pan
	ABS	A				; |pan|			
	MOVE			R2,Y1		;				spread
	MOVE			A,Y0		; 			       |pan|
	MPYR	Y0,Y1,B		R3,A		;       |pan|.spread	        pan
	ADD	B,A				; pan + |pan|.spread
	SUB	Y1,A				; pan + |pan|.spread - spread

	; panb = pan - |pan|.spread + spread

	NEG	B		R3,X0		;     - |pan|.spread		pan
	ADD	X0,B		X:K7FFFFF,X1	; pan - |pan|.spread		[+p] 1
	ADD	Y1,B		R4,Y0		; pan - |pan|.spread + spread	[+p] level

	MPYR	Y0,Y0,A		A,N1		; [+p]  level			pana
	MOVE			B,N2		; 				panb

	; balance (levela, levelb)
	
	TFR	X1,B		R1,X0		;   1				balance
	SUB	X0,B		A,Y1		;   1-balance			level
	ASR	B		R1,A		; (1-balance)			balance
	ADD	X1,A				;   1+balance			
	ASR	A		B,Y0		; (1+balance)			
	MPYR	Y0,Y1,B		N1,X0		; (1-balance).level		[+P] pana
	MOVE			A,Y0		;
	MPYR	Y0,Y1,A		#panLUT1K,R0	; (1+balance).level		[+P] centre addr

	MOVE			B,N3		;				levela
	MOVE			A,N4		;				levelb

	; aL = (1-pana).levela.f(pan)
	; aR = (1+pana).levela.f(pan)
	
	MOVE			R3,Y0		;				pan
	MPY	Y0,#14,A			; address(pan)
	TFR	X1,B				;   1
	SUB	X0,B		A1,N0		;   1-pana
	ASR	B		X1,A		; (1-pana)			1
	ADD	X0,A		N3,Y1		;   1+pana			levela
	ASR	A		B,Y0		; (1+pana)
	MPYR	Y0,Y1,B		Y:(R0+N0),X0	; (1-pana).levela		f(pan)
	MOVE			A,Y0
	MPYR	Y0,Y1,A		B,Y0		; (1+pana).levela
	MPYR	Y0,X0,B		X0,R3		; (1-pana).levela.f(pan)	f(pan) (temp)
	MOVE			A,Y0
	MPYR	Y0,X0,A		N2,X0		; (1+pana).levela.f(pan)	[+P] panb
	MOVE			B,R1		;				aL
	MOVE			A,R2		;				aR

	; bL = (1-panb).levelb.f(pan)
	; bR = (1+panb).levelb.f(pan)
	
	TFR	X1,B				;   1
	SUB	X0,B				;   1-panb
	ASR	B		X1,A		; (1-panb)			1
	ADD	X0,A		N4,Y1		;   1+panb			levelb
	ASR	A		B,Y0		; (1+panb)
	MPYR	Y0,Y1,B		R3,X0		; (1-panb).levelb		f(pan)
	MOVE			A,Y0
	MPYR	Y0,Y1,A		B,Y0		; (1+panb).levelb
	MPYR	Y0,X0,B				; (1-panb).levelb.f(pan)
	MOVE			A,Y0
	MPYR	Y0,X0,A				; (1+panb).levelb.f(pan)
	MOVE			B,R3		;				bL
	MOVE			A,R4		;				bR

	; Update interpolated parameters	(aL' = old value, aL = new value)

	MOVE			X:(R5),X0	;				aL'
	MOVE			R1,A		;				aL
	SUB	X0,A		X:(R6),X1	;				aR'
	MOVE			R2,B		;				aR
	SUB	X1,B		A,Y:(R5)+	;				-> daL
	MOVE			X:(R5),X0	;				bL'
	MOVE			B,Y:(R6)+	;				-> daR
	MOVE			R3,A		;				bL
	SUB	X0,A		X:(R6),X1	;				bR'
	MOVE			R4,B		;				bR
	SUB	X1,B		A,Y:(R5)+	;				-> dbL
	NOP						
	MOVE			B,Y:(R6)+	;				-> dbR

	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

;--------------------------------------------------------------------------
; Update matrix sources
;--------------------------------------------------------------------------

	MOVE			X:voiceStatePtr,R3		; -> Y:vPitch
	MOVE			X:voiceMatrixBaseAddress,R4	; vKFLW (voice-unique matrix source)
	MOVE			#vMatrixPoPoffset,N4		; To skip between direct & PoP sources

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Portamento
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; The portamento filter output is not updated on the first cycle of 
; a new note, as its coefficient is still defined by the velocity,
; or other note-dependent parameter, of the previous note. So 
; although the pitch may be momentarily wrong, at least it will 
; be correct on the following cycle. Conversely, if the portamento
; filter was applied immediately, a velocity-dependent slow
; portamento following a fast note would be impossible.

	MOVE			Y:vOnOff,A			; +ve->on, -ve->off, 0->no event
	TST	A		X:SNDportamnto,A
	JGT	_noPortamento					; Jump if note on

	; Out += K * (In - Out)

	ASR	#14,A,A
	MOVE			X:Kexp1LUT1K,R0
	MOVE			A,N0
	MOVE			Y:(R3)+,A			; vPitch ("In")
	MOVE			Y:(R3),Y0			; vFilteredPitch ("Out")
	MOVE			X:K7FFFFF,B
	SUB	Y0,A		Y:(R0+N0),X0
	SUB	X0,B		
	MOVE			A,Y1
	TFR	Y0,B		B,X0	
	MAC	X0,Y1,B						; Pitch portamento filter	
	NOP
	MOVE			B,Y:(R3)-			; vFilteredPitch

_noPortamento

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; KFLW (Key follower)
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			X:SNDoctave,Y1			; Matrix output (WARNING: dependency on previous cycle)

	MOVE			X:KP04286875,Y0			;  q' for octave: -5..+5
	MPYR	Y0,Y1,B		(R3)+				;  xq'	
	ASR	#16,B,B						;  xq' >> 16 
	RND	B		Y:filteredMidiPitchShift,X1	;  k = 0, so add 0.5		Value due to midi pitch wheel

	MOVE			X:K4800,X0		; factor for octave (units are  cents)
	MOVE			B,Y0
	MPY	X0,Y0,A		Y:(R3)+,Y0		; vFilteredPitch
	ASL	#31,A,A					; Shift & multiply integer result by 256 (fsd=24 bits, p2flut=16 bits)
	ADD	Y0,A		#.5,X0			; Add in the key follower value (derived from the midi note)
	ADD	X1,A					; Filtered pitch wheel

	ASR	#1,A,B
	ADD	X0,B		A,X:(R4)+N4		; -> 0..1			direct vKFLW
	MOVE			Y:(R3),Y0		; 				[+P] vVelocity
	MOVE			B,X:(R4)-N4		;				PoP vKFLW

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; KVEL (key velocity)
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	TFR	Y0,B		#0.5,Y1
	SUB	Y1,B		(R4)+
	ASL	B		(R4)+N4
	MOVE			Y0,X:(R4)-N4			; PoP vKVEL
	MOVE			B,X:(R4)+			; direct vKVEL
	
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; KVEL Decay
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; Velocity decay is placed after the matrix source update, to ensure 
; that maximum velocity is available even with very short decay times.

	MOVE			X:SNDkveldecy,A
	ASR	#14,A,A
	MOVE			X:Kexp1LUT1K,R0
	MOVE			A,N0
	NOP
	NOP
	MOVE			Y:(R3),Y0			; vVelocity
	MOVE			Y:(R0+N0),X0
	MPYR	X0,Y0,A						
	NOP
	MOVE			A,Y:(R3)+			; vVelocity

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; VDIF & STDF
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			Y:(R3)+,X0			; vVoiceDiff
	TFR	X0,A		#0.5,Y1	
	SUB	Y1,A		Y:(R3)+,X1			; vStackDiff	
	ASL	A		(R4)+N4
	TFR	X1,B		X0,X:(R4)-N4			; VDIF (PoP)
	SUB	Y1,B		A,X:(R4)+N4			; VDIF (direct)
	ASL	B		(R4)+
	MOVE			Y0,X:(R4)-N4			; STDF (PoP)
	MOVE			B,X:(R4)+			; STDF (direct)

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Update Legato flag
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Legato playing style is always asserted if portamento is non-zero. We 
; precalculate this flag here, so it can be examined before the voice matrix  
; is calculated. This is necessary, so that ENV1 & ENV2 outputs can be pulled 
; to zero immediately, and take effect in the new matrix calculation that 
; follows after a staccato note-on event occurs.
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			X:SNDportamnto,A		; Matrix output
	MOVE			Y:KH1000,Y0
	CMPM	Y0,A		Y0,A				; permit small values of portamento to count as zero
	CLR	A	IFLE					; Zero = Staccato, non-zero = legato 
	MOVE			(R3)+				; [-P] skip vMute (processing already performed above)
	MOVE			A,Y:(R3)+			; vLegato

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Calculate LFOs and Envelopes
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			R3,X:voiceStatePtr		; [+P] Record start of LFO1-4 state

	JSR	CalculateVoiceLFOs				; Generate LFO1 - LFO4	
	JSR	ProcessSoundLFOs				; Process LFO5 & LFO6 (delay & level in this voice)
	JSR	CalculateEnvelopes				; Generate ENV1 & ENV2

; If this voice was most recent assigned, copy LFO5 & LFO6 matrix parameters for later use 

	MOVE			Y:vOnOff,A
	MOVEC			LC,X1
	TST	A		Y:mostRecentVoiceIdx,B
	TGT	X1,B						; If note-on signal present, this is most recent assigned
	CMP	X1,B						; Compare LC with LC value of most recent voice
	JNE	_notMostRecent

	; This voice was most recently assigned

	MOVE			B,Y:mostRecentVoiceIdx

	MOVE			X:voiceStatePtr,X0
	MOVE			X0,X:LFO56vLFOStatePtr

	MOVE			#LFO5rate,R0
	MOVE			#LFO56Direct,R1
	
	DO	#8,_lfo56paracopy

	MOVE			X:(R0)+,X0
	MOVE			X0,X:(R1)+

_lfo56paracopy

	MOVE			X:LFO5levl,X0
	MOVE			X0,X:LFO56PoPs
	MOVE			X:LFO6levl,X1
	MOVE			X1,X:LFO56PoPs+1

_notMostRecent

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Prepare pointers and address registers for next voice loop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			(R4)+N4	; skip PoPs to next voice's matrix sources

	CLR	A		X:voiceInterpolatedParasBaseAddress,X0	
	CLR	B		X:voiceNISampleParasBaseAddress,X1
	MOVE			#interpolatedParasPerVoice,A1
	MOVE			#NISampleParasPerVoice,B1

	ADD	X0,A		R3,X:voiceStateBaseAddress
	ADD	X1,B		R4,X:voiceMatrixBaseAddress

	MOVE			A1,X:voiceInterpolatedParasBaseAddress
	MOVE			B1,X:voiceNISampleParasBaseAddress

paraVoiceLoop

;--------------------------------------------------------------------------
; Remaining non-voice-specific parameter calculations
;--------------------------------------------------------------------------

	JSR	CalculateSoundLFOs				; LFO5 & LFO6

	BRA	SampleGenerator

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	MSG	'INFO: ',$400-@CVI(*),' words of on-chip PRAM available'
	
	IF	@CVI(*)>$400
	FAIL	'Program section too large for on-chip ram'
	ENDIF

	org	P(2):		; Off-chip cached memory

ParaGenCacheStart

;----------------------------------------------------------------------
;	Quickfaders
;----------------------------------------------------------------------

FadeForNewSound

	MOVE			#>TransferNewSound,X0
	MOVE			X0,Y:sGenVector

FadeSoundQuickly

	ANDI	#$FC,MR		; Re-enable interrupts (as soon as possible)

	; Zero all parameter deltas so they stay the same

	CLR	A		#interpolatedParas,R0
	DO	#nInterpolatedParas,_loop
	MOVE			A,Y:(R0)+
_loop
	
	; Set the voice output level deltas to fade voice output levels to zero
	
	MOVE			#vInterpolatedParas+VLL,R0
	MOVE			#interpolatedParasPerVoice-3,N0

	DO	#N_VOICES_PROC,_qFadeLoop		

	MOVE			X:(R0)+,A	; VLL
	NEG	A		X:(R0)-,B	; VRL
	NEG	B
	MOVE			A,Y:(R0)+	; dVLL
	MOVE			B,Y:(R0)+	; dVRL

	MOVE			X:(R0)+,A	; VLR
	NEG	A		X:(R0)-,B	; VRR
	NEG	B
	MOVE			A,Y:(R0)+	; dVLR
	MOVE			B,Y:(R0)+N0	; dVRR

_qFadeLoop

	; Mute all voices for subsequent parameter generation
	; (voices will be unmuted when they receive a new note-on signal)

	MOVE			#vAssignState+vMute,R0
	MOVE			#voiceStateSize,N0
	MOVE			X:KFFFFFF,X1

	DO	#N_VOICES_PROC,_muteLoop

	MOVE			X1,Y:(R0)+N0	; vMute = on

_muteLoop

	; Zero envelopes and put them in release phase

	MOVE			#vENVstate+ENV_phase,R0			; ENV1
	MOVE			#voiceStateSize-1,N0

	MOVE			#vENVstate+ENV_phase+ENVstateSize,R1	; ENV2
	MOVE			#voiceStateSize-1,N1

	MOVE			X:KENVreleasePhase,X0
	MOVE			#0,X1

	DO	#N_VOICES_PROC,_clearEnvLoop

	MOVE			X0,Y:(R0)+		; release ENV1
	MOVE			X1,Y:(R0)+N0		; ENV1_level = 0
	
	MOVE			X0,Y:(R1)+		; release ENV2
	MOVE			X1,Y:(R1)+N1		; ENV2_level = 0

_clearEnvLoop

	; Audio

	BRA	SampleGenerator

;----------------------------------------------------------------------
;	Minimimal Parameter Generator
;----------------------------------------------------------------------

MinimalParameterGenerator

	ANDI	#$FC,MR		; Re-enable interrupts (as soon as possible)

	; Tell host that it has successfully minimised DSP operations

	BSET	#HCR_HF3,X:HCR

	; Zero all parameter deltas so they stay the same

	CLR	A		#interpolatedParas,R0
	DO	#nInterpolatedParas,_loop
	MOVE			A,Y:(R0)+
_loop
	; Generate audio samples as usual

	BRA	SampleGenerator
	
;----------------------------------------------------------------------
;	New Sound Processing
;----------------------------------------------------------------------

TransferNewSound

	ANDI	#$FC,MR		; Re-enable interrupts (as soon as possible)

	MOVE			X:DACBufferPtr,R0

	; Write zeros to left and right channels

	CLR	A

	DO	#2*N_CYCLES,_loop1
	MOVE			A,X:(R0)+
_loop1
	
	; Make new sound data live (NB DRAM treated as X: not Y: in this loop)

	MOVE			#soundTransferBuffer,R0
	MOVE			#soundData,R4
	MOVE			X:(R0)+,A
	MOVE			X:(R0)+,B

	DO	#1+soundDataSize/2,_loop2

	MOVE			X:(R0)+,A		A,Y:(R4)+
	MOVE			X:(R0)+,B		B,Y:(R4)+
_loop2

	; Update soundPnData array

 	; After transferring the sound data to live locations, translations are
	; required to translate gn values into pn values for matrix depths of 
	; those parameters whose matrix rows are calculated by product-of-products.

	MOVE			#soundPnData,R0
	MOVE			#soundData+nDirect*(nMatrixSources+1)+1,R4	; (skip base value)
	
	DO	#nPoPs,_loop4
	
	MOVE					Y:(R4)+,B			; gn (skip base value each outer loop)

	DO	#nMatrixSources,_loop3

	; pn = 1, if gn >= 0 pn = 1 - gn
	
	TST	B		X:K7FFFFF,A	B,Y0		; test gn	pn = 1		gn copy
	SUB	Y0,A	IFGE					; if gn >= 0 pn = 1 - gn  else  pn = 1
	NOP
	MOVE			A,X:(R0)+	Y:(R4)+,B	; pn -> soundPnData[..]		next gn (skip final "gn")

_loop3	NOP
_loop4	NOP

	; CTR1-3 may just have been changed for the new sound. Bypass their smoothing filters to avoid squawk.
	
	MOVE			Y:globalCtr1,A
	MOVE			#Ctr1ParaFilt,R0
	JSR	ParaFilt_ToValue
	MOVE			A,X:(R2+CTR1)
	SUB	#0.5,A
	ASL	A
	NOP
	MOVE			A,X:(R1+CTR1)

	MOVE			Y:globalCtr2,A
	MOVE			#Ctr2ParaFilt,R0
	JSR	ParaFilt_ToValue
	MOVE			A,X:(R2+CTR2)
	SUB	#0.5,A
	ASL	A
	NOP
	MOVE			A,X:(R1+CTR2)

	MOVE			Y:globalCtr3,A
	MOVE			#Ctr3ParaFilt,R0
	JSR	ParaFilt_ToValue
	MOVE			A,X:(R2+CTR3)
	SUB	#0.5,A
	ASL	A
	NOP
	MOVE			A,X:(R1+CTR3)

	; Update parameter generation vector, and signal host that transfer is complete

	MOVE			#>ParameterGenerator,X0
	MOVE			X0,Y:sGenVector

	BCLR	#soundPending,Y:flags

	JSR	SignalHost			; Re-enable host communications
	BRA	SwapAudioBuffers		; Then back to MainLoop

;--------------------------------------------------------------------------
; Parameter Filtering Class
;--------------------------------------------------------------------------
; On entry, R0 points to the object's data. This class provides the
; ability to smooth incoming parameter changes to any rate. The 
; parameter sample rate is very high, and this can result in abrupt 
; changes. Objects are declared for each of the incoming modulation 
; sources, such as pots, encoder, pedal, and also for the parameter
; editor. The parameter editor requires extra care, because not all
; sound parameters (ie the switched ones) should be filtered, and 
; also, it is necessary, when a parameter is selected for editing, 
; to ensure the previously edited para reaches its target value.
;
; A parameter transition is defined whenever the incoming filter 
; value is different from the internal target value. The transition
; is defined by these class members:
;
; ParaFilt_base:	the previous target value
; ParaFilt_delta:	the difference between base and target
; ParaFilt_target:	the parameter's current target value
;
; The rate of transition is defined by these class members, that 
; are constant after initialisation:
;
; ParaFilt_nIter:	number of iterations for transition
; ParaFilt_incr:	fractional increment (ie, 0.1 if nIter = 10)
; 
; The state of transition is defined by countdown timer ParaFilt_count,
; which decrements from nIter to zero.
;--------------------------------------------------------------------------

;----------------------------------------------------------------
; Initialise ParaFilt Object
;----------------------------------------------------------------
; A ParaFilt object is initialised with the parameter transition 
; time. This is translated to a number of parameter iterations
; and a fractional increment per iteration by:
; 
; nIterations = Transition Time (s) * Parameter sample rate (Hz)
;
; incrementPerIteration = 1 / nIterations
;
; these are called ParaFilt_nIter and ParaFilt_incr respectively.
;
; On entry:	R0 = object pointer
;		A  = Transition time in seconds (ie 0.02)
;
; Modified:	A,B,X,Y
;----------------------------------------------------------------

ParaFilt_Initialise

	MOVE			A,X0			; Transition time in seconds
	MOVE			#>48000/N_CYCLES,X1	; Parameter sample rate in hertz
	MPYR	X0,X1,A					; No. of iterations (at Para sample rate)
	
	AND	#$FE,CCR				; Clear carry bit C (quotient sign bit)
	MOVE			A,X:(R0+ParaFilt_nIter)
	MOVE			A,X0
	MOVE			#>1,A			; Number to be divided
	DO	#24,_loop				; Form a 24bit quotient
	DIV	X0,A					; Quotient in A0, remainder in A1
_loop	MOVE			A0,X:(R0+ParaFilt_incr)	; 1 / nIterations
	
	MOVE			#0,X0
	MOVE			X0,X:(R0+ParaFilt_count)
	MOVE			X0,X:(R0+ParaFilt_base)
	MOVE			X0,X:(R0+ParaFilt_delta)
	MOVE			X0,X:(R0+ParaFilt_output)
	MOVE			X0,X:(R0+ParaFilt_target)

	RTS

;----------------------------------------------------------------
; Filter iteration
;----------------------------------------------------------------
; On entry:	R0 = object pointer
;		A  = filter input value
;
; On exit:	R0 = unchanged
;		A  = filter output value 
;		B  = bipolar version of filter output 
;
; Modified:	A,B,X,Y
;
; The bipolar version of the output assumes the ordinary output
; is in the range 0..1, in which case the bipolar output is -1..1
;----------------------------------------------------------------

ParaFilt_Filter

	; Is input different from target?

	MOVE		X:(R0+ParaFilt_target),X0
	CMP	X0,A		
	JNE	_newTarget

	; Input matches target, so do any remaining iterations

	MOVE		X:(R0+ParaFilt_count),B
	TST	B	#0.5,Y0
	JNE	_newOutput
	
	TFR	A,B
	SUB	Y0,B
	ASL	B

	RTS

_newOutput	

	; Target hasn't been reached yet, last iteration?

	SUB	#1,B
	JNE	_notAtTarget

	; Done all iterations, so just update counter and load target value

	MOVE		B,X:(R0+ParaFilt_count)
	MOVE		B,X:(R0+ParaFilt_delta)
	MOVE		X:(R0+ParaFilt_target),X0
	MOVE		X0,X:(R0+ParaFilt_base)
	MOVE		X0,X:(R0+ParaFilt_output)

	TFR	X0,B	X0,A
	SUB	Y0,B
	ASL	B

	RTS

_newTarget

	; Target value has changed, so redefine transition and restart iterations

	MOVE		A,X:(R0+ParaFilt_target)

	MOVE		X:(R0+ParaFilt_output),X0
	MOVE		X0,X:(R0+ParaFilt_base)

	SUB	X0,A
	
	MOVE		X:(R0+ParaFilt_nIter),X1
	MOVE		A,X:(R0+ParaFilt_delta)

	TFR	X1,B
	SUB	#1,B

_notAtTarget

	; Iterate through the transition

	MOVE		X:(R0+ParaFilt_nIter),A
	SUB	B,A				; upCounter = nIter-count, so the result here counts up to nIter

	MOVE		B,X:(R0+ParaFilt_count)
	MOVE		X:(R0+ParaFilt_incr),Y0
	MOVE		A,X0			

	MPY	X0,Y0,A				; upCounter * incr, ie 10 * 0.03 = 0.3
	ASL	#23,A,A
	MOVE		X:(R0+ParaFilt_delta),Y0
	MOVE		X:(R0+ParaFilt_base),B
	MOVE		A,X0			; fraction of delta to add to base value this time

	MACR	X0,Y0,B				; frc . delta + base
	
	MOVE		#0.5,Y0

	MOVE		B,X:(R0+ParaFilt_output)
	MOVE		B,A
	
	SUB	Y0,B
	ASL	B

	RTS

;----------------------------------------------------------------
; Other ParaFilt Methods
;----------------------------------------------------------------

ParaFilt_GetValue	; Get the current filtered value

	MOVE		X:(R0+ParaFilt_output),A
	RTS

ParaFilt_ToValue	; Go straight to input value

	MOVE		A,X:(R0+ParaFilt_target)

	MOVE		#0,X0
	MOVE		X0,X:(R0+ParaFilt_count)
	MOVE		X0,X:(R0+ParaFilt_delta)

	MOVE		A,X:(R0+ParaFilt_output)
	MOVE		A,X:(R0+ParaFilt_base)
	
	RTS

ParaFilt_ToTarget	; Go straight to the target value

	MOVE		X:(R0+ParaFilt_target),A

	MOVE		#0,X0
	MOVE		X0,X:(R0+ParaFilt_count)
	MOVE		X0,X:(R0+ParaFilt_delta)

	MOVE		A,X:(R0+ParaFilt_output)
	MOVE		A,X:(R0+ParaFilt_base)
	
	RTS

;--------------------------------------------------------------------------
; LFOs 
;--------------------------------------------------------------------------
; There are 6 LFOs. LFO1-4 are unique in each voice (ie, each voice has its 
; own independant versions of these LFOs). LFO5 and LFO6 are common to all 
; voices. LFO5 and LFO6 are capable of syncing to midi, whereas LFO1-4 are
; not, mainly due to the amount of processing this requires, but also 
; because midi sync would (usually) make each voice LFO identical in 
; output, so you might as well use LFO5 or LFO6 in such an instance.
;
; Sync modes are different for voice LFOs and sound LFOs. It makes sense
; for LFO1-4 to sync to any of LFOs 1-6, but it doesn't make sense to 
; sync LFOs 5 or 6 to any of LFOs 1-4, unless one goes to the trouble
; of taking a vote among all the voices to determine the "average" 
; state for a particular voice LFO. 
;--------------------------------------------------------------------------

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Entry point for generating LFOs 1 to 4
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

CalculateVoiceLFOs

	MOVE			#LFO1rate,R1		; X: base address of direct matrix outputs
	MOVE			#LFO1levl,R2		; X: base address of product-of-product matrix outputs
	MOVE			#vLFOstateSize,N3

; For indexing LFO1-4 when used as sync sources 

	MOVE			X:voiceStateBaseAddress,N6		

; Main LFO processing

	DO	#4,_LFOmainLoop				; For each of four voice LFOs

; Identify LFO synchronisation subroutine

	MOVE			X:(R1+2),Y1		; LFOxsync matrix output

	MOVE			#0.10536875,Y0		;  q' for sync: 0..26
	MPYR	Y0,Y1,B					;  xq' 
	ASR	#16,B,B 				;  xq' >> 16 
	ADD	#13,B					;  + k
	RND	B					;  k = 13, so add 0.5 	

	LSL	B		#>vLFOsyncs,Y0		; Each LUT entry has 2 words	LUT base address
	ADD	Y0,B
 
; Translate LFO rate to frequency and update LFO phase

	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
	; The standard pitch-to-frequency LUT has a minimum frequency
	; which is not as low as one would like for an LFO. A simple 
	; fix is to take the pitch (-1..1), add 1 and saturate, and
	; use this number to factor out the minimum frequency, so 
	; the LFO goes all the way down to 0Hz:
	;
	;   z = Pitch + 1 		(values above 0 limit to 1)
	;   f = P2F(Pitch) + (zK - K)
	;
	;   where K = P2F(-1), the lowest frequency given by P2F
	
	MOVE			X:(R1)+,A		; 			LFOxrate: signed pitch value (-1..+1) from matrix	
	MOVE			B1,R6			;			[-P]
	MOVE			#0.5,B
	ADDL	A,B		X:KP2F64K,R0		; z = Pitch + 1	 	Pitch to frequency LUT mid address

	ASR	#8,A,A					; Address for P2F
	MOVE			B,X0			;			z
	MOVE			X:K806,Y0		;			K (from minimum value in Y:P2F64K) 
	MPY	X0,Y0,B		A1,N0			; zK
	SUB	Y0,B					; zK-K
		
	MOVE			Y:(R3+LFO_phase),X0	; 			Previous LFO phase
	MOVE			Y:(R0+N0),A		; 		 	P2F(P)
	ADD	B,A		X:(R6)+,R0		; f = P2F(P) + (zK-K)	[+P] LFO waveform synchronisation subr.
	ASR	#2,A,A					; LFO rate reduction 

; LFO phase addition

	ADD	X0,A		X:(R6),R6		; LFO phase update	[+P] LFO synchronisation source (offset/abs)
	BCLR	#LFOsHold,Y:flags			;			Default no sample-and-hold
	
; Update phase (for phase sync), or set hold flag (for S&H)
 
	JSR	(R0)					; ie, LFOnoSync, LFOsyncNote, LFOsyncLFO_NSH

; Translate phase into selected LFO waveform

	MOVE			X:(R1)+,Y1		;			LFOxwave

	MOVE			#0.046775,Y0		;  q' for waveform: 0..11
	MPYR	Y0,Y1,B		#>LFOwaves,Y0		;  xq' 		Base address for waveform subroutines
	ASR	#16,B,B 				;  xq' >> 16 
	ADD	#6,B					;  + k + 0.5 (k = 5.5)
	ADD	Y0,B
	MOVE			X:K1,X1			;			[+P] Possible S&H trigger for other LFOs
	MOVE			B1,R6

	MOVE			Y:(R3+LFO_phase),X0	; 			[+p] Get previous (possibly sync'd) phase
	MOVE			A1,Y:(R3+LFO_phase)	; 			[+p] Save new LFO phase (even if sync'd)
	CLR	B		A1,A			; 			[+p] Wrap new phase value

	CMP	X0,A		X:(R6),R0		; Compare previous with current phase, for new cycle detection
	TLT	X1,B					; signal if new cycle started

	MOVE			(R1)+			;			[+P] Skip over LFOxsync (already used)
	MOVE			B1,Y:(R3+LFO_signal)	; (used to send cycle signal to other LFOs)
	
	JSR	(R0)					; Waveform translation subroutine, ie LFOtriangle, LFOsine

; Waveform is now in Y0 (signed, bipolar output) and Y1 (unsigned, unipolar, inverted)
; The reason Y1 is inverted, as well as unipolar, is to facilitate correct level calculation
; for supplying appropriate signals to PofP matrix (ie, for zero level, LFO out should be 1).

; Update delay and calculate amplitude 			; (LFO waveform now in Y0 and Y1)

	MOVE			X:(R1)+,A		; LFOxdel signed delay value (-1..1) from matrix

	ASR	#14,A,A
	MOVE			X:Kexp2LUT1K,R0		; Exponential law for translation 
	MOVE			A,N0
	MOVE			Y:(R3+LFO_delayLevel),A	
	MOVE			Y:vOnOff,B
	TST	B		X:K7FFFFF,B		; test for new note
	CLR	A	IFGT				; Reset delay on new note
	MOVE			Y:(R0+N0),X0

	SUB	X0,B					; use 1-y for LFO delay	increment
	ADD	B,A		            		; Increase level

	MOVE			X:(R2)+,X0		; 			LFOxlevl from PoPs matrix
	MOVE			A,Y:(R3+LFO_delayLevel)	; 			[-p] update delay level (limiting)
	MOVE			A,X1

	MPYR	X0,X1,A					; output level * delay level
	MOVE			X:K7FFFFF,B		; For P of P matrix, if LFO level is zero, LFO out should be 1
	MOVE			A,X1			; LFO amplitude

; Apply amplitude to signed and unsigned versions of LFO waveform (see also notes for LFOformat)

	MPYR	Y0,X1,A					; signed LFO output		[+P]
	MACR   -Y1,X1,B		(R3)+N3			; unsigned LFO output		[+P]

; Sample-and-hold processing

	MOVE			X:(R4)+N4,X0		; get signed previous LFO output
	MOVE			X:(R4)-N4,X1		; get unsigned LFO output

	BTST	#LFOsHold,Y:flags			; hold flag set in sync subroutine, (ie LFOsyncNote_S_H)

	TFR	X0,A	IFCS				
	TFR	X1,B	IFCS

; Store LFO outputs

	MOVE			A,X:(R4)+N4		; store signed
	MOVE			B,X:(R4)-N4		; store unsigned

	MOVE			(R4)+			; Next LFO

_LFOmainLoop

	RTS
	
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; Entry point for generating LFOs 5 and 6
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

CalculateSoundLFOs 

; Midi clock sync pre-processing

	; filteredMCR = LPF(midiClockRate) (over several seconds) 

	; This filter takes the form: Out += K * (In - Out)

	MOVE			Y:midiClockRate,A	; 			In
	MOVE			L:filteredMCR48,B	; 			Out
	SUB	B,A		Y:filteredMCR,X1	; In - Out				[+P]
	MOVE			#.001,X0		;			K
	MOVE			A,L:gp
	MOVE			L:gp,Y
	
	MPYsu	X0,Y0,A					; K * (In - Out)
	DMACss	X0,Y1,A

	ADD	B,A		X:K1,B			; Out += 				[+P]

	; Detect midi clock signal and increment nIterations if no clock
	
	MOVE			Y:nIterations,X0
	MOVE			A,L:filteredMCR48

	BTST	#midiSyncSustain,Y:rFlipFlops
	TCS	X1,A					; Hold filteredMCR during sync sustain
	NOP
	ADD	X0,B		A,Y:filteredMCR		; nIterations++

_syncSustain

	BTST	#midiClockSignal,Y:rSignals
	JCC	_noMidiClock

	; Make persistent flags clear after the current midi clock cycle has been processed

	BCLR	#midiSyncPhaseUpdate,Y:iFlipFlops	; Clears registered flag after this parameter cycle

	; midiClockRate = 1 / nIterations since last clock

	AND	#$FE,CCR				; Clear carry bit C (quotient sign bit)
	MOVE			X:K1,A			; Number to be divided
	DO	#24,_loop				; Form a 24bit quotient
	DIV	X0,A					; Quotient in A0, remainder in A1
_loop	CLR	B					; [+P] reset nIterations
	MOVE			A0,Y:midiClockRate	; 1 / nIterations
	
_noMidiClock
	
	MOVE			B,Y:nIterations

	; Synced LFO updates

_unmodulatedLFO5rate 	equ 	@CVS(N,soundData)+(LFO5rate-soundStructure)*(nMatrixSources+1)
_LFOstep		equ	(LFO6rate-LFO5rate)*(nMatrixSources+1)
	
	MOVE			#_unmodulatedLFO5rate,R1   ; Y: LFO rate base parameter value
	MOVE			#_LFOstep,N1		   ; Step thru LFOs
	
	MOVE			#-2097152,R2		; LFO rate equal or lower indicates midi sync

	MOVE			#sLFOs,R3		; X: base address of sound LFO state variables
	MOVE			#sLFOstateSize,N3

	DO	#2,LFOMidiSyncLoop

	MOVE			Y:(R1)+N1,X0		; unmodulated LFO Rate  
	MOVE			R2,A			; Threshold value for base parameter
	SUB	X0,A		#0,B			; -> 0..18 if synced		zIncr = 0 if no sync
	JLT	LFOnoMidiSync

	JCLR	#midiClockSignal,Y:rSignals,LFOnoMidiClock
	
	; Base address for sync rate parameter LUT

	ASL	#1,A,B					; these 2 steps multiply by three
	ADD	B,A					; index for LFOSyncParas LUT

	MOVE			Y:(R3+LFO_seq),R0	; 				[+p] Seq
	ADD	#>LFOSyncParas,A			; Add LUT base address

    	; LFO has midi sync. Calculate new zIncr (see midiLFOsync.txt)

	; zDiff = zNow - zPrev; zPrev = zNow 		; zPrev = LFO phase at previous clock pulse

	MOVE			Y:(R3+LFO_phase),B	;				zNow
	CLR	A		A,R0			;				[P+] LFOSyncParas LUT
	MOVE			Y:(R3+LFO_zPrev),X0	;				zPrev
	MOVE			B,Y:(R3+LFO_zPrev)	;				zPrev = zNow

	SUB	X0,B		#1,A2			; zDiff = zNow - zPrev 		A = 2.000
	ADD	A,B	IFLT				; Add 2.0 if zNow has wrapped since zPrev
	 
	; zErr = zSync - zDiff				; zSync is the previous phase change required per clock

	MOVE			Y:(R3+LFO_zSync),A 
	LSL	#2,A					; Rescale, LS 2 bits must be zero

	SUB	B,A		X:(R0)+,N4		; zErr				[+P] Base address for sequence
	CLR	A	IFES				; zErr limit (fixes sync rate panic on large sync rate changes) 

	; zErr can be inhibited by any process that needs to arbitrarily set the LFO's phase
	; (otherwise zErr will try to correct the LFO for apparent phase drift)

	MOVE			Y:(R3+LFO_zErrOkay),B	; (NB cannot use this as mask as AND is only 24 bit operation)
	TST	B		X:K7FFFFF,B
	CLR	A	IFEQ				; Clear zErr if it's not okay to use
	MOVE			B,Y:(R3+LFO_zErrOkay)	; Default next cycle zErr okay

	; zSync = LUT[Seq++] + zErr			; New phase change: LUT contains the sequence, eg 85, 85, 86

	MOVE			Y:(R3+LFO_seq),R4
	MOVE			X:(R0)+,Y1		;				[+P] Sequence length
	NOP

	MOVE			X:(R4+N4),B		; ensure zErr not extended	LUT[Seq]
	ASL	#8,B,B					; so that max value of sequence data is 2.0 (from 65536)

	ADD	B,A		(R4)+			; zSync = LUT[Seq++] + zErr	Seq++
	ASR	#2,A,A					; scale result for storage and also for multiply
	MOVE			Y:midiClockRate,X0
	MOVE			A,X1

	; zIncr = zSync * midiClockRate			; Required phase increment per LFO iteration

	MPY	X0,X1,B		R4,A			;				Seq	
	MOVE			X1,Y:(R3+LFO_zSync)

	CMP	Y1,A					; Sequence length vs Seq
	CLR	A	IFGE				; if Seq >= sequence length, Seq = 0
	
	LSL	#2,B					; Rescale, LS 2 bits must be zero to avoid phase drift
	MOVE			A,Y:(R3+LFO_seq)

	; zIncr = sustainPhasePerClock * filteredMCR	; Alternatively use this value during sync sustain

	JCLR	#midiSyncSustain,Y:rFlipFlops,_noSyncSustain

	MOVE			X:(R0)+,X0		;				sustainPhasePerClock
	MOVE			Y:filteredMCR,Y0	;				(unchanged during sync sustain)
	MPY	X0,Y0,B		#0,X0			; Phase incr per iteration
	ASL	#2,B,B					; Compensate for /4 sustainPhasePerClock storage in LUT
	MOVE			X0,Y:(R3+LFO_zErrOkay)	; Inhibit zErr for first cycle after sync sustain is released

_noSyncSustain

	; If midiSyncPhaseUpdate requested, update LFO phase with user value (or zero on midi start)

	JCLR	#midiSyncPhaseUpdate,Y:rFlipFlops,_noPhaseUpdate

	MOVE			Y:globalMidiSyncPhase,A
	SUB	B,A		#0,X0			; Subtract zIncr so next LFO iteration results in target phase
	MOVE			X0,Y:(R3+LFO_zErrOkay)	;				 zero zErr on next midi clock 
	MOVE			A1,Y:(R3+LFO_phase)	

_noPhaseUpdate

LFOnoMidiSync

	MOVE			B,Y:(R3+LFO_zIncr)

LFOnoMidiClock

	MOVE			(R3)+N3

LFOMidiSyncLoop

	; Prepare registers for main LFO processing loop

	MOVE			#LFO56Direct,R1		; X: base address of LFO5 and LFO6 direct matrix outputs
	MOVE			#sLFOs,R3		; Y: base address of LFO5 and LFO6 state variables
	MOVE			#sLFOstateSize,N3

	; NB, currently N6 is not used in any sLFO sync routines

; Main LFO processing

	DO	#2,_LFOmainLoop				; For each of two sound LFOs

; Identify sLFO synchronisation subroutine (sLFOs have different sync modes from vLFOs)

	MOVE			X:(R1+2),Y1		; LFOxsync matrix output

	MOVE			#0.04677500,Y0		;  q' for sync: 0..11
	MPYR	Y0,Y1,B					;  xq' 
	ASR	#16,B,B 				;  xq' >> 16 
	ADD	#6,B					;  + k + 0.5

	LSL	B		#>sLFOsyncs,Y0		; Each LUT entry has 2 words	LUT base address
	ADD	Y0,B
 
; Translate LFO rate to frequency and update LFO phase

	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
	; The standard pitch-to-frequency LUT has a minimum frequency
	; which is not as low as one would like for an LFO. A simple 
	; fix is to take the pitch (-1..1), add 1 and saturate, and
	; use this number to factor out the minimum frequency, so 
	; the LFO goes all the way down to 0Hz:
	;
	;   z = Pitch + 1 		(values above 0 limit to 1)
	;   f = P2F(Pitch) + (zK - K)
	;
	;   where K = P2F(-1), the lowest frequency given by P2F
	
	MOVE			X:(R1)+,A		; 			LFOxrate: signed pitch value (-1..+1) from matrix	
	MOVE			B1,R6			;			[-P]
	MOVE			#0.5,B
	ADDL	A,B		X:KP2F64K,R0		; z = Pitch + 1	 	Pitch to frequency LUT mid address

	ASR	#8,A,A					; Address for P2F
	MOVE			B,X0			;			z
	MOVE			X:K806,Y0		;			K (from minimum value in Y:P2F64K) 
	MPY	X0,Y0,B		A1,N0			; zK
	SUB	Y0,B					; zK-K
		
	MOVE			Y:(R3+LFO_phase),X0	; 			Previous LFO phase
	MOVE			Y:(R0+N0),A		; 		 	P2F(P)
	ADD	B,A		X:(R6)+,R0		; f = P2F(P) + (zK-K)	[+P] LFO waveform synchronisation subr.
	ASR	#2,A,A					; LFO rate reduction 

; Check for midi sync on this LFO 

	MOVE			Y:(R3+LFO_zIncr),B	;			zIncr (freq from midi sync)
	TST	B					; Always zero in voice LFOs
	TNE	B,A					; if midi synced, use zIncr as frequency

; LFO phase addition

	ADD	X0,A		X:(R6),R6		; LFO phase update	[+P] LFO synchronisation source (offset/abs)
	BCLR	#LFOsHold,Y:flags			;			Default no sample-and-hold
	
; Update phase (for phase sync), or set hold flag (for S&H)
 
	JSR	(R0)					; ie, LFOnoSync, LFOsyncNote, LFOsyncLFO_NSH

; Translate phase into selected LFO waveform

	MOVE			X:(R1)+,Y1		;			LFOxwave

	MOVE			#0.046775,Y0		;  q' for waveform: 0..11
	MPYR	Y0,Y1,B		#>LFOwaves,Y0		;  xq' 		Base address for waveform subroutines
	ASR	#16,B,B 				;  xq' >> 16 
	ADD	#6,B					;  + k + 0.5 (k = 5.5)
	ADD	Y0,B
	MOVE			X:K1,X1			;			[+P] Possible signal to other LFOs to S&H etc
	MOVE			B1,R6

	MOVE			Y:(R3+LFO_phase),X0	; 			[+p] Get previous (possibly sync'd) phase
	MOVE			A1,Y:(R3+LFO_phase)	; 			[+p] Save new LFO phase (even if sync'd)
	CLR	B		A1,A			; 			[+p] Wrap new phase value

	CMP	X0,A		X:(R6),R0		; Compare previous with current phase, for new cycle detection
	TLT	X1,B					; signal if new cycle started

	MOVE			(R1)+			;			[+P] Skip over LFOxsync (already used)
	MOVE			B1,Y:(R3+LFO_signal)	; (used to send cycle signal to other LFOs)
	
	JSR	(R0)					; Waveform translation subroutine, ie LFOtriangle, LFOsine
	
	MOVE			(R3)+N3			;			[+P] Next LFO
	MOVE			(R1)+			;			[+P] skip over LFOxdel to LFOyrate

_LFOmainLoop

	RTS
	
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Voice-specific post processing of LFO5 and LFO6
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; LFO5 and LFO6 are calculated once for the sound as a whole. 
; However, their output levels are controlled individually 
; in each voice. Delay is also voice-specific, so that LFO5, 
; can be delayed (its amplitude faded up from zero) when a
; new note is played within a voice, whereas other voices also 
; using LFO5 will have different fade-in levels. Furthermore,
; sound LFOs can be sampled and held by voice events (note-on
; or a signal from LFO1-4).
;
; On entry: 	R1 -> LFO5rate
;		R2 -> LFO5levl
;		R3 -> LFO5_delayLevel
;		R4 -> vLFO5 (voice matrix source)
;		N4 == offset between signed and unsigned sources
; 
; On exit:	LFOx_delayLevel updated for LFO5&6
;		vLFO5 and vLFO6 voice matrix sources updated
;		R1-R4 updated past end of their respective segments
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ProcessSoundLFOs

	MOVE			#LFO5del-LFO5rate,N1
	MOVE			#sLFOs,R5		; Address of LFO5 waveform
	MOVE			#sLFOstateSize,N5	; To skip between LFO5 and LFO6

	DO	#2,_loop

; Note that it is inappropriate to translate a sound LFO sync mode at this point, 
; because the sync mode is determined by the voice matrix for the voice most 
; recently assigned. 

; Create unsigned inverted version of sound waveform

	MOVE			Y:(R5+LFO_data),Y0	;			bipolar LFO signal -1..1
	TFR	Y0,B
	ASR	B		#0.5,X1
	NEG	B					; Invert for correct level operation (see note above)
	ADD	X1,B		(R1)+N1			; LFO signal 0..1	goto LFOxdel

	MOVE			X:(R1)+,A		; LFOxdel signed delay value (-1..1) from matrix

; Update delay and calculate amplitude 			; (LFO waveform now in Y0 and Y1)

	ASR	#14,A,A
	MOVE			X:Kexp2LUT1K,R0		; Exponential law for translation 
	MOVE			A,N0
	MOVE			Y:(R3),A	
	MOVE			Y:vOnOff,B
	TST	B		X:K7FFFFF,B		; test for new note
	CLR	A	IFGT				; Reset delay on new note
	MOVE			Y:(R0+N0),X0

	SUB	X0,B					; use 1-y for LFO delay	increment
	ADD	B,A		            		; Increase level

	MOVE			X:(R2)+,X0		; 			LFOxlevl from PoPs matrix
	MOVE			A,Y:(R3)+		; 			[-p] LFOx_delayLevel (limiting)
	MOVE			A,X1

	MPYR	X0,X1,A					; output level * delay level
	MOVE			X:K7FFFFF,B		; For P of P matrix, if LFO level is zero, LFO out should be 1
	MOVE			A,X1			; LFO amplitude

; Apply amplitude to signed and unsigned versions of LFO waveform (see also notes for LFOformat)

	MPYR	Y0,X1,A					; signed LFO output
	MACR   -Y1,X1,B					; unsigned LFO output

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; Sample-and-hold processing for LFO5 and LFO6 in a voice
;
; 	LFO_shType		Meaning
;
;	$000000			S&H trigger at absolute address
;	$FFFFFF			S&H trigger at relative address

	MOVE			A,X0
	MOVE			B,X1

	MOVE			Y:(R5+LFO_shAddr),A
	MOVE			Y:(R5+LFO_shType),B
	TST	B		X:voiceStateBaseAddress,Y0

	ADD	Y0,A	IFNE				; Use relative address if LFO_shType != 0
	NOP
	MOVE			A1,R0			; Address of sample & hold signal (ie, LFO_signal)
	NOP
	MOVE			(R5)+N5			; [+P] Next LFO
	
	MOVE			X:(R4)+N4,A		; 			signed previous LFO output
	MOVE			Y:(R0),B		; 			trigger sigmal
	TST	B		X:(R4)-N4,B		; Test for trigger 	unsigned previous LFO output

	TFR	X0,A	IFGT				; update with new value on positive trigger
	TFR	X1,B	IFGT

; Store LFO outputs

	MOVE			A,X:(R4)+N4		; store signed
	MOVE			B,X:(R4)-N4		; store unsigned

	MOVE			(R4)+			; Next LFO 

_loop	NOP
	RTS
	
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; LFO Waveforms 
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

LFOtriangle

	MOVE			X:K7FFFFF,Y0
	ADD	Y0,A					; Start at minimum when synchronised
	NOP
	MOVE			A1,A

	ABS	A		#0.5,Y0
	SUB	Y0,A
	ASL	A
	BRA	LFOformat

LFOsine

	ADD	#0.5,A					; Start at minimum when synchronised

	LSR	#6,A
	MOVE			X:Ksine256K,R0
	MOVE			A1,N0
	NOP
	NOP
	NOP
	MOVE			Y:(R0+N0),A

	BRA	LFOformat

LFOsawtooth

	BRA	LFOformat

LFOsawExp

	ASR	A		#0.5,Y0
	ADD	Y0,A
	NOP
	MOVE			A,X0
	MPYR	X0,X0,A

	SUB	Y0,A
	ASL	A

	BRA	LFOformat

LFOsquare

	; Square wave starts on high cycle

	TST	A		X:K7FFFFF,A
	NEG	A	IFGE
	BRA	LFOformat

LFOpulse

	; Pulse starts on high part of cycle

	MOVE		#-0.9375,Y0
	CMP	Y0,A	X:K7FFFFF,A
	NEG	A	IFGE
	BRA	LFOformat

LFOrandom

	MOVE		Y:(R3+LFO_data),A	; previous random number
	BGT	LFOformat			; test for phase wrap since last iteration

	; Generate new random number (two iterations for better quality)

	LSL	A	X:KnoiseMask,Y0
	NOP
	EOR	Y0,A	A1,Y1
	TCS	Y1,A				; first rn

	NOP
	LSL	A	A1,X0
	NOP
	EOR	Y0,A	A1,Y1
	TCS	Y1,A				; second rn

	NOP
	MOVE		A1,A
	
	BRA	LFOformat

LFOgaussian
	
	MOVE		X:noisePtr,R0		; pointer to fast noise sample batch
	MOVE		#250,M0
	MOVE		Y:(R3+LFO_data),A	; previous random number
	BGT	LFOformat			; test for phase wrap since last iteration

	NOP
	MOVE		X:(R0)+,A
	MOVE		R0,X:noisePtr		; update noise pointer
	MOVE		A1,A

	BRA	LFOformat

LFOpink	

	MOVE		Y:(R3+LFO_data),A	; previous output
	BGT	LFOformat			; test for phase wrap since last iteration

	; Phase wrapped, so new value needed

	MOVE		X:noisePtr,R0		; pointer to noise sample
	MOVE		#250,M0

	MOVE		R1,Y:(R7)+		; Pink noise filter uses R1 and R2
	MOVE		R2,Y:(R7)+

	LUA		(R3+LFO_stateA),R1	; Base address for pink noise filter vector

	MOVE		X:(R0)+,A		; Get gaussian noise sample
	MOVE		R0,X:noisePtr		; update noise pointer

	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        ; Pink Noise Filter
        ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        ; Three parallel 6dB sections + direct, delays excluded from direct 
        ; signal path. For further background, see pinknoise.txt.
        ; 
        ; On entry:
        ;
        ; A contains the input sample
        ; R1 -> Y: Da, Db, Dc (filter state vector)
        ;
        ; On exit: 
        ; 
        ; A contains the output sample
        ; Filter samples updated
        ; R1 points to the location after the state vector 
        ; (ie. possibly the next vector of a stereo pair)
        ;
        ; Registers used: 
        ;
        ; A, B, X, Y, R1, R2
        ;
        ; Filter constants (defined in memory.asm):
        ;
        ; R2 -> X: Ka, Kb, Ca, Cb, Cd, Kc, Cc
        ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	ASR	#7,A,A					; Prescale by 1/128	

	MOVE			#pinkConstants,R2
	MOVE			A,X0	Y:(R1)+,B	; 			Input		Da ->
	ADD	X0,B			Y:(R1)-,A	; Feedback a				Db ->
	ADD	X0,A					; Feedback b
	MOVE			B,Y0	X:(R2)+,X1	;			Ia		Ka
	MPYR	Y0,X1,A		A,Y1	X:(R2)+,X1	; a.Ka			Ib		Kb
	MPYR	Y1,X1,B			X:(R2)+,X1	; b.Kb 					Ca

	MPY	Y0,X1,A			A,Y:(R1)+	; Ca.Ia			-> Da
	MOVE				B,Y:(R1)+	;			-> Db

	MOVE				Y:(R1),B	;			Dc ->
	ADD	X0,B			X:(R2)+,X1	; Feedback c				Cb
	MAC	Y1,X1,A			X:(R2)+,X1	; + Cb.Ib				Cd
	MAC	X0,X1,A		B,Y0	X:(R2)+,X1	; + Cd.Input		Ic		Kc
	MPYR	Y0,X1,B			X:(R2)+,X1	; c.Kc					Cc
	MAC	Y0,X1,A					; + Cc.Ic
	MOVE				B,Y:(R1)+	;			-> Dc				
	
	ASL	#10,A,A					; Rescale by 128 and then boost some more

        ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE		Y:-(R7),R2
	MOVE		Y:-(R7),R1

	BRA	LFOformat

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; The next three LFO routines are interpolated waveform generators - each time the phase 
; wraps around, a new target value is generated by the random process. When no wrap occurs,
; the phase is used to interpolate between the previous and most recent random values. 

LFOrandom_S

	BGT	LFOinterpolate			; test for phase wrap since last iteration

	; Generate new random number (two iterations for better quality)

	MOVE		Y:(R3+LFO_target),A	; previous random number
	LSL	A	X:KnoiseMask,Y0
	NOP
	EOR	Y0,A	A1,Y1
	TCS	Y1,A				; first rn

	NOP
	LSL	A	A1,X0
	NOP
	EOR	Y0,A	A1,Y1
	TCS	Y1,A				; second rn

	MOVE		Y:(R3+LFO_target),X0	; Old target becomes new initial value
	MOVE		X0,Y:(R3+LFO_initial)

	MOVE		A1,Y:(R3+LFO_target)	; New pink noise sample is new target
	MOVE		X0,A			; Old target is output value

	BRA	LFOformat

LFOgauss_S
	
	MOVE		X:noisePtr,R0		; pointer to fast noise sample batch
	MOVE		#250,M0
	BGT	LFOinterpolate			; test for phase wrap since last iteration

	MOVE		X:(R0)+,A
	MOVE		R0,X:noisePtr		; update noise pointer

	MOVE		Y:(R3+LFO_target),X0	; Old target becomes new initial value
	MOVE		X0,Y:(R3+LFO_initial)

	MOVE		A1,Y:(R3+LFO_target)	; New pink noise sample is new target
	MOVE		X0,A			; Old target is output value

	BRA	LFOformat

LFOpink_S

	BGT	LFOinterpolate			; test for phase wrap since last iteration

	; Phase wrapped, so new target value needed

	MOVE		X:noisePtr,R0		; pointer to noise sample
	MOVE		#250,M0

	MOVE		R1,Y:(R7)+		; Pink noise filter uses R1 and R2
	MOVE		R2,Y:(R7)+

	LUA		(R3+LFO_stateA),R1	; Base address for pink noise filter vector

	MOVE		X:(R0)+,A		; Get gaussian noise sample
	MOVE		R0,X:noisePtr		; update noise pointer

	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        ; Pink Noise Filter
        ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        ; Three parallel 6dB sections + direct, delays excluded from direct 
        ; signal path. For further background, see pinknoise.txt.
        ; 
        ; On entry:
        ;
        ; A contains the input sample
        ; R1 -> Y: Da, Db, Dc (filter state vector)
        ;
        ; On exit: 
        ; 
        ; A contains the output sample
        ; Filter samples updated
        ; R1 points to the location after the state vector 
        ; (ie. possibly the next vector of a stereo pair)
        ;
        ; Registers used: 
        ;
        ; A, B, X, Y, R1, R2
        ;
        ; Filter constants (defined in memory.asm):
        ;
        ; R2 -> X: Ka, Kb, Ca, Cb, Cd, Kc, Cc
        ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	ASR	#7,A,A					; Prescale by 1/128	

	MOVE			#pinkConstants,R2
	MOVE			A,X0	Y:(R1)+,B	; 			Input		Da ->
	ADD	X0,B			Y:(R1)-,A	; Feedback a				Db ->
	ADD	X0,A					; Feedback b
	MOVE			B,Y0	X:(R2)+,X1	;			Ia		Ka
	MPYR	Y0,X1,A		A,Y1	X:(R2)+,X1	; a.Ka			Ib		Kb
	MPYR	Y1,X1,B			X:(R2)+,X1	; b.Kb 					Ca

	MPY	Y0,X1,A			A,Y:(R1)+	; Ca.Ia			-> Da
	MOVE				B,Y:(R1)+	;			-> Db

	MOVE				Y:(R1),B	;			Dc ->
	ADD	X0,B			X:(R2)+,X1	; Feedback c				Cb
	MAC	Y1,X1,A			X:(R2)+,X1	; + Cb.Ib				Cd
	MAC	X0,X1,A		B,Y0	X:(R2)+,X1	; + Cd.Input		Ic		Kc
	MPYR	Y0,X1,B			X:(R2)+,X1	; c.Kc					Cc
	MAC	Y0,X1,A					; + Cc.Ic
	MOVE				B,Y:(R1)+	;			-> Dc				
	
	ASL	#10,A,A					; Rescale by 128 and then boost some more

        ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE		Y:(R3+LFO_target),X0	; Old target becomes new initial value
	MOVE		X0,Y:(R3+LFO_initial)

	MOVE		A,Y:(R3+LFO_target)	; New pink noise sample is new target
	MOVE		X0,A			; Old target is output value

	MOVE		Y:-(R7),R2
	MOVE		Y:-(R7),R1

	BRA	LFOformat

	; Use phase (-1..1) to interpolate between initial and target values

LFOinterpolate

	MOVE		A1,A			;   -1..1, phase
	ASR	A	#0.5,X0
	ADD	X0,A				; -> 0..1, interpolation
	MOVE		Y:(R3+LFO_target),B
	MOVE		Y:(R3+LFO_initial),X0
	SUB	X0,B	A,Y0			; target-initial	0..1, interpolation		 	
	ASR	B				; shift right to ensure difference <= 1
	TFR	X0,A				; initial
	MOVE		B,Y1	
	MAC	Y0,Y1,A				; Linear interpolation
	MACR	Y0,Y1,A				; (x2 for difference right shift)

	BRA	LFOformat

; LFO waveforms are applied as signed modulators for parameters like pitch, calculated by the 
; direct matrix, but level parameters calculated by the product-of-products matrix require
; modulators that are unipolar, for example, a sinewave in the range 0..1. For this reason,
; LFOs have signed and unsigned versions, generated here, and returned in Y0 and Y1.

; A particular twist is that when the LFO level is less than one, ie 0.25, then the product-of-products
; matrix requires that the LFO signal varies from, ie, 0.75 up to 1. To help facilitate this
; calculation, the unipolar version of the LFO is inverted, so that this can be multiplied by 
; the LFO level and then subtracted from 1 in order to facilitate this correct behaviour.  

LFOformat

	MOVE			A,Y0			;			bipolar LFO signal -1..1
	MOVE			Y0,Y:(R3+LFO_data)	;
	ASR	A		#0.5,X1			; 
	NEG	A					; Invert for correct level operation (see note above)
	ADD	X1,A					; LFO signal 0..1
	MOVE			X:KFFFFFF,M0		; 			Some noise waveforms use Mx
	MOVE			A,Y1			;			unipolar 1..0

	RTS

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; LFO synchronisation 
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; On entry: 	A contains current LFO's phase 
;		R3 points to the current LFO state vector
;		R6 points to the LFO synchronisation source
;		N6 is the voice LFO state base address (0 for sLFOs)
;		(typically the address of another LFO's phase)
;
; On exit: 	A contains LFO phase (possibly unchanged) 
;		R1-R6 are preserved
;
; Midi sync:	Midi sync will attempt to correct for any unexpected
;		phase change. Clearing LFO_zErrOkay prevents this. 
;		The LFO will then lock at the new phase.
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

;--------------------------------------------------------------------------
; Sync algorithms for voice LFOs (LFO1-4)
;--------------------------------------------------------------------------

vLFOnoSync

	RTS

; Restart LFO waveform on new note  - - - - - - - - - - - - - - - - - - - -

vLFOsyncNote

	MOVE			Y:vOnOff,B
	TST	B		X:K800000,X0		; [+P]			Start at waveform minimum
	TGT	X0,A

	RTS

; Sync LFOs in quadrature - - - - - - - - - - - - - - - - - - - - - - - - -

vLFOsyncQuadInVoice

	MOVE			#0.5,Y0			; 90 degrees phase offset
	JMP	vLFOsyncPhaseInVoice	

vLFOsyncQuadInSound

	MOVE			#0.5,Y0			; 90 degrees phase offset
	JMP	vLFOsyncPhaseInSound

; Sync LFOs in antiphase  - - - - - - - - - - - - - - - - - - - - - - - - -

vLFOsyncAntiInSound
	
	MOVE			X:K7FFFFF,Y0		; 180 degrees phase offset
	JMP	vLFOsyncPhaseInSound

vLFOsyncAntiInVoice
	
	MOVE			X:K7FFFFF,Y0		; 180 degrees phase offset
;	JMP	vLFOsyncPhaseInVoice			; [unnecessary]

vLFOsyncPhaseInVoice
	
	MOVE			Y:(R6+N6),A		; 				relative addr vLFO
	ADD	Y0,A					; Calculate sync'd phase
	RTS

vLFOsyncPhaseInSound
	
	MOVE			Y:(R6),A		; 				absolute addr sLFO
	ADD	Y0,A					; Calculate sync'd phase
	RTS

; Sample-and-hold on signal from another LFO - - - - - - - - - - - - - - - -

vLFOsyncLFO_S_H_InVoice

	BTST	#0,Y:(R6+N6)				; Source LFO sent sample signal?
	BCS	_sample
	BSET	#LFOsHold,Y:flags			; Hold if no LFO sample signal

_sample	RTS

vLFOsyncLFO_S_H_InSound

	BTST	#0,Y:(R6)				; Source LFO sent sample signal?
	BCS	_sample
	BSET	#LFOsHold,Y:flags			; Hold if no LFO sample signal

_sample	RTS

; Reset phase on new Note, then Sample-and-Hold from LFO - - - - - - - - - - 

vLFOsyncLFO_NSH_InVoice					; "Note Sample Hold"

	MOVE			Y:vOnOff,B
	TST	B		X:K800000,X0		; 			Start at waveform minimum
	TGT	X0,A
	JMP	vLFOsyncLFO_S_H_InVoice

vLFOsyncLFO_NSH_InSound					; "Note Sample Hold"

	MOVE			Y:vOnOff,B
	TST	B		X:K800000,X0		; 			Start at waveform minimum
	TGT	X0,A
	JMP	vLFOsyncLFO_S_H_InSound

; Sample-and-hold on new note - - - - - - - - - - - - - - - - - - - - - - -

vLFOsyncNote_S_H

	MOVE			Y:vOnOff,B
	TST	B					; New note event just occured?
	BGT	_sample
	BSET	#LFOsHold,Y:flags			; Hold if no new note event
 
_sample	RTS

;--------------------------------------------------------------------------
; Sync algorithms for sound LFOs (LFO5 and LFO6)
;--------------------------------------------------------------------------

; Sync sources for sound LFOs are never voice LFOs, so source addressing is absolute

sLFOnoSync

	CLR	B		#>one,X0		; Always trigger = no S&H
	MOVE			X0,Y:(R3+LFO_shAddr)
	MOVE			B,Y:(R3+LFO_shType)
	RTS

; Sync LFOs in quadrature - - - - - - - - - - - - - - - - - - - - - - - - -

sLFOsyncQuad

	MOVE			#0.5,Y0			; 90 degrees phase offset
	JMP	sLFOsyncPhase	

; Sync LFOs in antiphase  - - - - - - - - - - - - - - - - - - - - - - - - -

sLFOsyncAnti
	
	MOVE			X:K7FFFFF,Y0		; 180 degrees phase offset

sLFOsyncPhase
	
	MOVE			Y:(R6),A
	ADD	Y0,A		#0,X0			; Calculate sync'd phase

	MOVE			X0,Y:(R3+LFO_zErrOkay)	; zErrOkay always zero when tracking phase of another LFO

	CLR	B		#>one,X0		; Always trigger = no S&H
	MOVE			X0,Y:(R3+LFO_shAddr)
	MOVE			B,Y:(R3+LFO_shType)

	RTS

; Sample-and-hold for sound LFOs  - - - - - - - - - - - - - - - - - - - - -

; Sample and hold processing is different for LFO5 and LFO6, because each voice
; can independently sample and hold the LFO5 and LFO6 waveforms. The triggers 
; can be other LFOs in the voice (LFO1-4), for which a voice-relative address is 
; provided. vOnOff (note event) can also be a trigger, but note that this is an
; absolute address in spite of its name. LFO5 and LFO6 are also possible triggers,
; requiring an absolute address. To prevent S&H from occurring, the address of 
; Y:one is given, so that the sample update is effectively triggered every time.
;
; The contents of the LFO_shType state variable are defined in the synchronisation
; subroutine called during generation of LFO5 or LFO6. 
;
; 	LFO_shType		Meaning
;
;	$000000			S&H trigger at absolute address
;	$FFFFFF			S&H trigger at relative address

sLFOsyncLFO_S_H_InVoice

	; Sample & hold from LFO in a voice

	MOVE			X:KFFFFFF,X0		; Value for voice-relative LFO_signal
	MOVE			R6,Y:(R3+LFO_shAddr)
	MOVE			X0,Y:(R3+LFO_shType)
	RTS
	
sLFOsyncLFO_S_H_InSound

	; Sources for this include the LFO_signal in LFO5 or LFO6

	MOVE			#0,X0			; Value for absolute trigger signal
	MOVE			R6,Y:(R3+LFO_shAddr)
	MOVE			X0,Y:(R3+LFO_shType)
	RTS

sLFOsyncNote_S_H	

	; Sample & hold from absolute address Y:vOnOff 

	CLR	B		#>vOnOff,X0
	MOVE			X0,Y:(R3+LFO_shAddr)
	MOVE			B,Y:(R3+LFO_shType)
	RTS

;--------------------------------------------------------------------------
; Envelopes
;--------------------------------------------------------------------------
; On entry: R3 points to the start of ENV1 state in Y: memory
; 	    R4 points to vENV1 of voice matrix sources in X:
;	    N4 is the offset between direct & PoP sources
;--------------------------------------------------------------------------

CalculateEnvelopes
	
	MOVE			Y:(R3),R0		; ENV_phase (ENV1)

	MOVE			#ENV1att,R1		; X: base address of direct matrix outputs
	MOVE			#ENV2att-ENV1att,N1	; (Placed out of order here, to save a few cycles)

	JSR	(R0)					; ENV1

	MOVE			Y:(R3),R0		; ENV_phase (ENV2)

	MOVE			(R1)+N1			; increments for next ENV
	
	MOVE			A,X:(R4)+N4		; (both types of envelope output are unipolar)
	MOVE			A,X:(R4)-N4
	MOVE			    (R4)+

	JSR	(R0)					; ENV2

	MOVE			A,X:(R4)+N4
	MOVE			A,X:(R4)-N4
	MOVE			    (R4)+

	RTS

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Envelope Phases 
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ENVpreAttackPhase

	MOVE			#>ENVattackPhase,X0
	MOVE			X0,Y:(R3)+		; update phase
	MOVE			Y:(R3)+,A		; get new level
	RTS

; The purpose of the two staccato pre-attack phases is to provide ordinary
; attack processing during these phases. However, the second phase is
; detected in the voice parameter loop, and a flag is set, so that
; audio oscillator sync can be performed at the appropriate time.
; This is a form of (very opaque) pipelining, required because the
; immediate cycle following a staccato note-on involves fading the
; envelopes down to zero. Only on the subsequent parameter cycle 
; should oscillators be reset (if the reset parameter requires this).

ENVstaccatoPreAttackPhase1

	MOVE			#>ENVstaccatoPreAttackPhase2,X0
	MOVE			X0,Y:(R3)
	JMP			ENVattackPhase

ENVstaccatoPreAttackPhase2

	MOVE			#>ENVattackPhase,X0
	MOVE			X0,Y:(R3)

ENVattackPhase
	
	MOVE			X:(R1),A		; signed attack value (-1..+1) from matrix	

	ASR	#14,A,A
	MOVE			X:Kexp1LUT1K,R0
	MOVE			A,N0
	NOP
	MOVE			X:K7FFFFF,B
	MOVE			  (R3)+			;				skip over phase 
	MOVE			Y:(R0+N0),X0

	SUB	X0,B		Y:(R3),A		; we use 1-y for env attack 	get envelope level	

	ADD	B,A					; Add attack rate to current level
	MOVE			#>ENVdecayPhase,B	;				possible new phase
	MOVE			A,Y:(R3)+		;				store new level
	BES	ENVnewPhase				; Jump if limit reached
	RTS

ENVdecayPhase

	MOVE			X:(R1+ENVxdec),A	; signed decay value (-1..+1) from matrix	

	ASR	#14,A,A
	MOVE			X:Kexp1LUT1K,R0
	MOVE			A,N0
	MOVE			  (R3)+
	MOVE			Y:(R3)+,A		; 			get envelope level
	MOVE			X:(R1+ENVxsus),B	;			get sustain level (-1..1)	
	MOVE			Y:(R0+N0),X0
	ASR	B		A,Y0
	ADD	#.5,B					; Sustain (0..1)

	; NB the sustain level from the matrix is in the range -1..1 to avoid need for complex checking

	CMP	Y0,B		#>ENVsustainPhase,B	; reached sustain yet?
	BGE	ENVnewPhase				; 
	
	MPYR	X0,Y0,A					; decay
	MOVE			(R3)-	
	MOVE			A,Y:(R3)+		; 			update level

	RTS

ENVsustainPhase

	; Sustain can change dynamically as it is a matrix output, so always update 

	MOVE			X:(R1+ENVxsus),A	;			get sustain level (-1..1)
	ASR	A		#0.5,Y0
	ADD	Y0,A					; rerange sustain
	MOVE			    (R3)+
	MOVE			A,Y:(R3)+		; 			update level (0..1)
	RTS

ENVreleasePhase

	MOVE			X:(R1+ENVxrel),A	; signed release value (-1..+1) from matrix	

	ASR	#14,A,A
	MOVE			X:Kexp1LUT1K,R0
	MOVE			A,N0
	NOP
	MOVE			(R3)+
	MOVE			Y:(R3),Y0		; 			get envelope level
	MOVE			Y:(R0+N0),X0

	MPYR	X0,Y0,A					; decrease level
	NOP
	MOVE			A,Y:(R3)+		; 			update level
	RTS

ENVnewPhase

	MOVE			B,Y:(R3-2)		; update phase
	RTS

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Translate Exponential Parameters 
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; The envelope time parameters (attack, decay and release) use
; the same EXP1 exponential translation, as does Portamento. 
; LFO delay uses EXP2. Accuracy is not crucial, so 1K LUTs are 
; used, without interpolation. 
;
; On entry: 	A contains parameter, range -1..1
;
; On exit:	X0 contains translated data, range 0..1
;
; Used:		A, X0, R0, N0
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; The code has now been optimised, so these subroutines
; are not used. This may be useful as documentation.

EXP1translate

	ASR	#14,A,A
	MOVE			X:Kexp1LUT1K,R0
	MOVE			A,N0
	NOP
	NOP
	NOP
	MOVE			Y:(R0+N0),X0

	RTS

EXP2translate

	ASR	#14,A,A
	MOVE			X:Kexp2LUT1K,R0		; Exponential law for translation 
	MOVE			A,N0
	NOP
	NOP
	NOP
	MOVE			Y:(R0+N0),X0

	RTS

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        MSG     'INFO: ',@CVI(*)-ParaGenCacheStart,' / 1024 words of cache used by ParameterGenerator'
