;********************************************************************************
;* E N T I T Y  								*
;********************************************************************************
;* File: dsp/sampleGenerator.asm						*
;*                                                                         	*
;* Copyright (C) A.R.Hardie-Bick 2004-2005                                      *
;*										*
;* This program is free software; you can redistribute it and/or modify		*
;* it under the terms of the GNU General Public License as published by		*
;* the Free Software Foundation; either version 2 of the License, or (at	*
;* your option) any later version.						*
;*										*
;* This program is distributed in the hope that it will be useful, but		*
;* WITHOUT ANY WARRANTY; without even the implied warranty of			*
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU		*
;* General Public License for more details.					*
;*										*
;* You should have received a copy of the GNU General Public License		*
;* along with this program; if not, write to the Free Software			*
;* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.			*
;********************************************************************************

	org	P(2):		; Use cached external memory

SampleGenerator

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;	Prepare for sample loop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			X:noisePtr,R6
	MOVE			X:noiseInc,N6
	MOVE			#250,M6
	
	DO	#N_CYCLES,EndSampleLoop

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;	Prepare for voice loop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			X:ADCBufferPtr,R0
	MOVE			#NISampleParas,R3	; -> O1zfmsrc/O1a, O1phaseAcc,
	MOVE			#vInterpolatedParas,R4	; -> O1ashape, O2ashape...
	CLR	A		#3,N3
	MOVE			#4,N5

	MOVE			A,L:vAccL		; Zero voice accumulator left
	MOVE			A,L:vAccR		; Zero voice accumulator right

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; 	External Input
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			X:(R0)+,X1		; Left
	MOVE			X:(R0)+,X0		; Right
	TFR	X1,A		Y:maxInL,B
	MAXM	A,B		X,L:external		; X:left Y:right
	TFR	X0,A		Y:maxInR,Y0
	TFR	Y0,B		B,Y:maxInL		; max left for level meter
	MAX	A,B		X1,Y:externalLeft	; left copy for modulation
	MOVE			R0,X:ADCBufferPtr
	MOVE			B,Y:maxInR		; max right for level meter

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;	Voice Loop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	DO	#N_VOICES_PROC,VoiceLoop

	MOVE			X:Ksine512,R1		; Osc sine LUT
	MOVE			X:Ksine512,R2		

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; 	Oscillators
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	; Incremental oscillator parameter addressing
	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	;
	; 	R3 incr	   	   X		   Y	           L
	; 	-------        	  ---		  ---	          ---
	;
	; 	   0	 	O1zfmsrc	  O1a
	; 	   1				     	        O1phaseAcc   
	; 	   2				                O1freq   
	; 	   3		O1zfm		  O1PM
	; 	   4		O1frqshift 	  O1b
	; 	   5				  O1            
	;
	; 	R4 incr	  	   X		 
	; 	-------           ---
	;
	;    	   0		O1zshape
	;
	; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	; XY:oscPhzCtrl uses its X portion for temporary storage of phase,
	; but its Y portion contains a constant which is the control word
	; for the EXTRACT instruction. 

	; See java/oscillator/Oscillator.java for prototype

	DUP	3						; Three oscillators	

	MOVE			X:(R3)+,R0			; 		Oxzfmsrc (address of FM signal)
	MOVE			L:(R3)+,A			;		OxphaseAcc
	MOVE			L:(R3)+,X			;		Oxfreq	

	ADD	X,A		X:(R3)-N3,X0			; Add phase	Oxzfm
	MOVE			         	Y:(R0)+,Y0	;					FM signal 
	MPY	X0,Y0,B		X:(R4)+,X1	Y:(R3)+,Y1	; FM		Oxzshape		Oxa 	
	ADDL	A,B		A10,L:(R3)+N3			; +2.FM		-> OxphaseAcc (store unmodulated phase) 
	MOVE			X:(R3)-,X0			; 		Oxfrqshift (1 for square, 0 for saw)	

	TFR	X0,A		B1,X:oscPhzCtrl			; Oxfrqshift	copy base phase
	ASL	X0,B,B						; ASL if sq		
	SUB	X1,B				Y:(R3)+,X0	; -Oxzshape		OxPM
	MAC	X0,Y1,B				R3,R5		; PM			addr(Oxb)
	MOVE			Y:oscPhzCtrl,Y0			;			shift control word

	EXTRACTU Y0,B,B						; 48-bit LSR #15

	TST	A				Y:(R3),Y0	; Square?		Oxb (previous output)	
	MPY	X0,Y1,A		IFNE				; PM, if sq, use Oxa as modulator
	MPY	X0,Y0,A		IFEQ				; PM, else use Oxb
	ADD	X1,A		L:oscPhzCtrl,X			; +Oxzshape		base phase & shift control word
	ADD	X1,A				B1,N1
	
	EXTRACTU X0,A,A						; 48-bit LSR #15

	MOVE					B0,Y0		;			fractional address
	MOVE					A1,N2

	MOVE			L:(R1+N1),X			;			nb diff must be halved in LUT
	MPYsu	X0,Y0,B						; Linear interpolation
	ADD	X1,B				A0,Y0		;			fractional address

	MOVE			L:(R2+N2),X			;			NB: 2 cycles for (Rx+Nx)
	MPYsu	X0,Y0,A						; Linear interpolation
	ADD	X1,A				(R5)-N5		;			addr(Oxa)		

	MOVE			X:(R3),B	B,Y:(R5)	; Oxfrqshift (1 for square, 0 for saw)		-> Oxa
	TST	B		A,X0		Y:(R5),B	;			Oxb			Oxa
	ADD	B,A		IFEQ				; if saw, add Oxa and Oxb together
	ASR	A		IFEQ				; and divide by two

	MOVE					X0,Y:(R3)+	;			-> Oxb
	MOVE					A,Y:(R3)+	; The combined output (or same as Oxb if square)

	ENDM

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; 	Prepare address registers
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	MOVE			X:(R3)+,R1			; X:srca Y:RM
	MOVE			X:(R3),R2			; X:srcb Y:NS
	MOVE			R5,R0
	MOVE			#O3-O2,N0			; gap between O1, O2, O3 outputs

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; 	Ring Modulator & Noise
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	; The ring modulator and noise sources are interleaved in 
	; this section of code, but are functionally independent.

	; Stereo gaussian noise is supplied to the main filter as an
	; input signal, but has a minimum (inaudible) signal level 
	; that dithers the pre-filter mix (this minimum is defined
	; in parameterGenerator.asm). A separate (mono) gaussian 
	; noise source is provided as a modulation source.

	MOVE			X:(R6)+N6,X0	Y:(R1),Y0	; 	noise (mod src)		source a
	MOVE					Y:(R2),Y1	;      				source b
	MPYR	Y0,Y1,A				X0,Y:(R3)-	; ring modulation		NS (mod src)
	MOVE					(R0)-		; 	
	MOVE					(R0)-N0		;				[+P] (O1)
	MOVE			X:(R6)+N6,X1	A,Y:(R3)	; 	[+P] noise (mix L)	RM
	MOVE			X:(R6)+N6,Y1			; 	[+P] noise (mix R)

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; 	Pre-filter Mixer
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
	
	MOVE			X:(R4)+,X0	Y:(R0)+N0,Y0	; 		O1L	O1
	MPY	X0,Y0,A		X:(R4)+,X0	Y:(R0)+N0,Y0	;		O2L	O2
	MAC	X0,Y0,A		X:(R4)+,X0	Y:(R0),Y0	;		O3L	O3
	MAC	X0,Y0,A		X:(R4)+,X0	Y:(R3),Y0	;		RML	RM
	MAC	X0,Y0,A		X:(R4)+,X0			;		NSLR	
	MAC	X0,X1,A		X:(R4)+,X1			; dither/noise	EXLR	
	MOVE					  (R0)-N0
	MOVE				   	  (R0)-N0
	MPY	X0,Y1,B		X:(R4)+,X0	Y:(R0)+N0,Y0	; dither/noise	O1R	O1
	MAC	X0,Y0,B		X:(R4)+,X0	Y:(R0)+N0,Y0	;		O2R	O2
	MAC	X0,Y0,B		X:(R4)+,X0	Y:(R0),Y0	;		O3R	O3
	MAC	X0,Y0,B		X:(R4)+,X0	Y:(R3)+,Y0	;		RMR	RM
	MAC	X0,Y0,B		L:external,Y

	MAC	Y1,X1,A		          	  (R3)+
	MAC	Y1,X1,A				   R3,R2
	MAC	Y0,X1,B				#3,N4    			
	MAC	Y0,X1,B					
 
	MOVE			A,Y:gp1				;		Filter input (left)
	MOVE			B,Y:gp2				;		Filter input (right)

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Stereo Filter
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; This is a derivative of the original Entity filter, having a reduced 
; number of instruction cycles. Careful application of dither at each 
; truncation from 48 to 24 bits avoids the introduction of digital 
; artefacts. This is necessary when calculating integrator feedback; 
; otherwise, integration of digital artefacts occurs.
;
; On entry:
;
; Y:gp1 and Y:gp2 contain the left & right input signals
;
; Interpolated control parameters (the same used for left and right):
;
; R4 -> X: fltRs, fltFa, fltFb, fltOn  
;
; fltRs is the feedback level that defines resonance. fltFa and fltFb
; are the filter frequency coefficients (proportional to cutoff), and 
; fltOn is the parameter defining the distortion transfer function. 
;
; The remainder of the data are audio samples:
;
; R2 -> Y: fltD0L, fltD1L, fltD0R, fltD1R (filter out and delayed out)
; R3 -> X: fltDaL, fltDbL, fltDaR, fltDbR (integrator delays)
; 
; R6 -> next noise sample in noise array 
;
; Each respective Nx must contain appropriate values (see code)
;
; On exit: 
; 
; Filter samples updated. 
; Registers changed: A, B, X, Y, R2, N2, R3, R4, R6.
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE			X:KP000001,Y1			; Filter dither level

	; - - - - - - - - - - - - - - - -
	; Resonance (left channel)
	; - - - - - - - - - - - - - - - -

	MOVE			X:(R4)+,X1	Y:(R2)+,Y0	;				res		fltD0x
	MOVE			X:(R4)+,X0	Y:(R2),A	;				1-K = fltFa	fltD1x
	SUB	Y0,A		X:K7FFFFF,B			; fb = fltD1x-fltD0x		1
	SUB	X0,B				Y0,Y:(R2)-	; K = 1-(1-K)				     -> fltD1x

	MOVE					A,Y0		;						fb
	MPY	-X1,Y0,A	X:K7FFFFF,X1			; -res.fb			1
	MOVE			X:(R6)+N6,Y0			; 				noise

	; The gaussian noise added to each integrator stage has two purposes: firstly to linearise (dither)
	; the truncation of the 48 bit integrator feedback to 24 bits, and secondly to impart roughness.

	; - - - - - - - - - - - - - - - -
	; First stage
	; - - - - - - - - - - - - - - - -

	MAC	Y1,Y0,A				Y:gp1,Y0	; + noise					in
	MAC	X0,Y0,A		X:(R3),X0	B,Y0		;  (1-K).ov.in - res.fb	z'	z'		K
	MAC	X0,Y0,A		X:(R6)+N6,Y0			; ((1-K).ov.in - res.fb+z'.K	[+P] noise

	; - - - - - - - - - - - - - - - -
	; Second stage 
	; - - - - - - - - - - - - - - - -
	
	TFR	X1,B		X:(R4)+,X0			; 1				1-K = fltFd
	SUB	X0,B				#2,N2		; 1-(1-K)			[+P]
	MPY	Y1,Y0,A		A,X:(R3)+	A,Y0		; noise				[-P] -> z'	in
	MAC	X0,Y0,A		X:(R3),X0	B,Y0		; 1-K . in			z'		K	
	MAC	X0,Y0,A		X:(R6)+N6,Y0			;				[+P] noise 
	
	ASL	#3,A,B						; increase signal to distort (also increases resonance)
	MOVE			A,X:(R3)+			; 				-> z'

	; - - - - - - - - - - - - - - - -
	; Distortion: f(x) = x - 2(nx) 
	; - - - - - - - - - - - - - - - -

	MPY	Y1,Y0,A		X:(R4)-N4,X0	B,Y0		; noise				n		x
	MAC	X0,Y0,A		B,B				; (nx) 				x
	MOVE			X:(R6)+N6,Y0			;				noise
	MPY	Y1,Y0,A		A,X0				; noise
	MAC	X0,X0,A		X:(R6)+N6,Y0			; (nx)				noise
	MOVE					R2,R0
	MAC	Y1,Y0,B		A,Y0				; noise
	MAC    -X0,Y0,B				  (R2)+N2	; x - (nx)
	MAC    -X0,Y0,B		X:(R4)+,X1	Y:(R2)+,Y0	; x - 2(nx)			[+P] res	fltD0R
	MOVE			X:(R4)+,X0	Y:(R2),A	; 				[+P] 1-K=fltFa	fltD1R

	MOVE					B,Y:(R0)	; 				D0. (R)+ -> fltD0L

	; - - - - - - - - - - - - - - - -
	; Resonance (right channel)
	; - - - - - - - - - - - - - - - -

	SUB	Y0,A		X:K7FFFFF,B			; fb = fltD1x-fltD0x		1
	SUB	X0,B				Y0,Y:(R2)-	; K = 1-(1-K)				     -> fltD1x

	MOVE					A,Y0		;						fb
	MPY	-X1,Y0,A	X:K7FFFFF,X1			; -res.fb			1
	MOVE			X:(R6)+N6,Y0			; 				noise
	
	; - - - - - - - - - - - - - - - -
	; First stage
	; - - - - - - - - - - - - - - - -

	MAC	Y1,Y0,A				Y:gp2,Y0	; + noise					in
	MAC	X0,Y0,A		X:(R3),X0	B,Y0		;  (1-K).ovx.in - res.fb	z'		K
	MAC	X0,Y0,A		X:(R6)+N6,Y0			; ((1-K).ovx.in - res.fb+z'.K	[+P] noise

	; - - - - - - - - - - - - - - - -
	; Second stage 
	; - - - - - - - - - - - - - - - -
	
	TFR	X1,B		X:(R4)+,X0			; 1				1-K = fltFd
	SUB	X0,B						; 1-(1-K) 
	MPY	Y1,Y0,A		A,X:(R3)+	A,Y0		; noise				[-P] -> z'	in
	MAC	X0,Y0,A		X:(R3),X0	B,Y0		; 1-K . in			z'		K	
	MAC	X0,Y0,A		X:(R6)+N6,Y0			;				[+P] noise 
	
	ASL	#3,A,B						; increase signal to distort (also increases resonance)
	MOVE			A,X:(R3)+			; 				-> z'

	; - - - - - - - - - - - - - - - -
	; Distortion: f(x) = x - 2(nx) 
	; - - - - - - - - - - - - - - - -

	MPY	Y1,Y0,A		X:(R4)+,X0	B,Y0		; noise				n		x
	MAC	X0,Y0,A		B,B				; (nx) 				x
	MOVE			X:(R6)+N6,Y0			;				noise
	MPY	Y1,Y0,A		A,X0				; noise
	MAC	X0,X0,A		X:(R6)+N6,Y0			; (nx)				noise
	NOP
	MAC	Y1,Y0,B		A,Y0				; noise
	MAC    -X0,Y0,B		L:vAccL,A			; x - (nx)			[+P] Voice acc. left		
	MAC    -X0,Y0,B		X:(R4)+,X0	Y:(R0),Y1	; x - 2(nx)			[+P] VLL	[+P] fltD0L
	MOVE			X:(R4)+,Y0			; 				[+P] VRL
	MOVE			B,X1		B,Y:(R2)+N2	; 				D0. (R)+ -> fltD0R

; Filter = 55 cycles (LR)

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; 	Accumulate voice left and right signals
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MAC	X0,Y1,A		L:vAccR,B			; vAccL += VLL x fltD0L		Voice acc. right	
	MAC	Y0,X1,A		X:(R4)+,X0			; vAccL += VRL x fltD0R		VLR	
	MAC	X0,Y1,B		X:(R4)+,X0			; vAccR += VLR x fltD0L 	VRR
	MAC	X0,X1,B						; vAccR += VRR x fltD0R

	MOVE			A,L:vAccL
	MOVE			B,L:vAccR

VoiceLoop
	
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Limiter
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; A correction is calculated for the output signal, which is less than 
; 1 if limiting is required. This correction is then applied in the 
; final output stage. The level meters are driven from the 
; uncorrected output, not the actual output, so it's possible 
; to see when limiting occurs.
;
; The limiter LUT has integer and fractional address portions. The
; integer portion is supplied from the extension register (A2 or B2)
; after the signal sum has been calculated. The LUT contains 1/x
; values, so any signal that uses the extension register is given
; a corresponding 1/x correction value. The reason that an LUT is
; used, rather than a simple 1/x divide, is that the LUT contains
; a soft transition into the 1/x curve, starting at x = 0.8. Also
; a divide takes longer than linear interpolation.
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	; Define amount of left shift applied to voice accumulators 
	; (More is safer, but precision of voice accumulators is reduced)
	; The effective accumulator precision is (48-_accShift) bits

_accShift	equ		3

	; Calculate uncorrected maximum and left & right meter levels

;	MOVE			L:vAccL,A			; [should be unnecessary]
	ASL	#_accShift,A,A
	MOVE			Y:maxOutL,B
	MAXM	A,B
	ASR	#8,A,A
	MOVE			B,Y:maxOutL
	MOVE			A,X:gp

	MOVE			L:vAccR,A
	ASL	#_accShift,A,A
	MOVE			Y:maxOutR,B
	MAXM	A,B
	ASR	#8,A,A
	MOVE			B,Y:maxOutR
	MOVE			X:gp,B

	MAXM	A,B						; max(left,right)

	; Calculate correction

 	ABS	B		X:KP999813,X0			; Limiter decay time = 0.7s, k = e^(-2PI/(t.Fs)
	MOVE			Y:limMaxLR,Y0			; Previous peak
	MPYR	X0,Y0,A		B,B				; limiter decay	(limit B)
	MAX	A,B		X:Klim256LUT64K,X0		; max(limMaxLR,left,right)
	ASR	#7,B,A						; LUT address offset

	ADD	X0,A						; LUT base address
	MOVE			B,Y:limMaxLR			; Store new peak
	MOVE			A1,R0
	MOVE			A0,X1
	NOP
	NOP
	
	; Linear interpolate correction factor

	MOVE			Y:(R0)+,B
	MOVE			Y:(R0)+,A
	SUB	B,A						; Calculate data difference
	ASL	B						; pre-compensate mixed multiply
	MOVE			A,X0                    	; Copy difference for multiply
	MACsu	X0,X1,B						; Linear interpolate
	ASR	B						; compensate mixed multiply
	RND	B
	
	MOVE			X:MVOL,X0			;		panel volume
	MOVE			B,X1				; 		correction

	; Include front panel volume

	MPYR	X0,X1,A		X:DACBufferPtr,R0		; x MVOL	[+P]
	MOVE			L:vAccL,Y			;		[+P]
	MOVE			A,X0				;		c

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;	Finalise level and store in output buffer
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MPYsu	X0,Y0,A
	DMACss	X0,Y1,A

	ASL	#_accShift,A,A
	MOVE			X:(R6)+N6,X1
	MAC	X1,#22,A					; + dither
	MOVE			L:vAccR,Y			;		[+P]
	MOVE			A,X:(R0)+			;		truncate to 24 bits

	MPYsu	X0,Y0,A
	DMACss	X0,Y1,A

	ASL	#_accShift,A,A
	MOVE			X:(R6)+N6,X1
	MAC	X1,#22,A					; + dither
	MOVE			R6,X:noisePtr
	MOVE			A,X:(R0)+			;		truncate to 24 bits

	MOVE			R0,X:DACBufferPtr

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; 	Parameter Interpolation
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	MOVE		   	#interpolatedParas,R1
	MOVE		  	#interpolatedParas,R4
	MOVE		   	#interpolatedParas,R2
	NOP

	MOVE		   	#1.0/N_CYCLES,Y1		; increment factor
	MOVE		   	X:(R1)+,A	Y:(R4)+,Y0	; 		Pa	dPa

	DO	#nInterpolatedParas/2,_loop

	MACR	Y0,Y1,A	   	X:(R1)+,B	Y:(R4)+,Y0	; Pa + dPa	Pb	dPb
	MACR	Y0,Y1,B    	X:(R1)+,X0	Y:(R4)+,Y0	; Pb + dPb	Pc	dPc
	MOVE		  	A,X:(R2)+			;		new Pa ->
	TFR	X0,A	   	B,X:(R2)+			; Pc		new Pb ->

_loop	NOP

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

EndSampleLoop

	MOVE			X:KFFFFFF,M6

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Audio Buffering & MIPS Measurement
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Swap the active ADC and DAC buffers, waiting if the codec is still 
; working with it. This is the point at which idle cycles are 
; measured with the DSP timer - optionally a code fragment is 
; included or excluded alternately with each double buffer cycle 
; to separately measure its cycle count.
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

SwapAudioBuffers

	BSET	#TCSR_TE,X:<<TCSR0			; Start idle timer

	MOVE			X:DACBufferPtr,A
	MOVE			X:KendDACBuffer,Y0
	CMP	Y0,A		X:KDACBuffer2,X0
	BEQ	_changeToBuffer1

_changeToBuffer2

	; Idle until buffer ready

	MOVEP			X:<<DSR0,A
	CMP	X0,A
	BGE	_changeToBuffer2

	BCLR	#TCSR_TE,X:<<TCSR0			; Stop idle timer

	MOVE			X0,X:DACBufferPtr
	MOVE			X:KADCBuffer2,X0
	MOVE			X0,X:ADCBufferPtr

	; Calculate fragment MIPS used

	MOVEP			X:<<TCR0,B		; Get idle time interval
	ASL	B		X:KcyclesAvailable,A	; Timer increments at clock/2

	SUB	B,A		Y:totalMips,B		; MIPS used without code fragment
	SUB	A,B		Y:maxFragMips,A		; MIPS used by code fragment alone
	
	MAX	A,B					
	NOP
	MOVE			B,Y:maxFragMips		; Store largest value

	BRA	MainLoop

_changeToBuffer1

	; Idle until buffer ready

	MOVEP			X:<<DSR0,A	
	CMP	X0,A		X:KDACBuffer1,X1
	BLT	_changeToBuffer1

	BCLR	#TCSR_TE,X:<<TCSR0			; Stop idle timer

	MOVE			X1,X:DACBufferPtr
	MOVE			X:KADCBuffer1,X1
	MOVE			X1,X:ADCBufferPtr

	; Calculate total MIPS used

	MOVEP			X:<<TCR0,B		; Get idle time interval
	ASL	B		X:KcyclesAvailable,A	; Timer increments at clock/2
	SUB	B,A
	MOVE			Y:maxTotalMips,B
	MAX	A,B		A,Y:totalMips
	NOP
	MOVE			B,Y:maxTotalMips	; Store largest value
	
	BRA	MainLoop

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        MSG     'INFO: ',@CVI(*)-SampleGenerator,' / 1024 words of cache used by SampleGenerator'
