// File: MidiLFOSyncSeq.java

// Program to generate sequence numbers for LFO midi 
// clock sync state machine - see midiLFOSync.txt 

public class MidiLFOSyncSeq {
 
	public static void main(String[] args) {

		int phaseResolution = 65536;	 	// Set this to the precision of LFO phase 

		int [] denominators = {

			3072, 2304, 1536, 1152, 864,
			768, 672, 576, 480, 384, 288, 192, 96, 
			48, 36, 24, 18, 12, 8, 6, 4,  
                        3, 2, 1
		};	

		System.out.printf("Sequence values assume 360 degree phase change given by %d\n\n", phaseResolution);

		for(int x = 0; x < denominators.length; x++) {

			Factors f = new Factors(denominators[x]);
			
			System.out.printf("Sequence for 1/%d -> 1/(%dx%d):\n\n", f.denominator, f.a, f.b);
		
			int 	clockCycles = f.b;	// Number of clock cycles for sequence
			
			double 	Denominator = f.denominator;
			double 	IncPerClock = (double)phaseResolution / Denominator;
			double	Fraction = IncPerClock - Math.floor(IncPerClock);
		
			int	baseSeqVal = (int)Math.floor(IncPerClock);
			double	Carry = 0;			
		
			int	seqTotal = 0;
			
			for(int c = 0; c < clockCycles; c++) {

				int seqVal = baseSeqVal;
				Carry += Fraction;
				
				if(Carry >= 0.9999998) { // NB Accuracy fails if 1.0 is comparison value

					Carry -= 1.0;
					seqVal += 1;
				}

				seqTotal += seqVal;

				System.out.printf("\t%d\n", seqVal);
			}
			
			int total = phaseResolution / f.a; 
			
			if(seqTotal != total) {

				System.out.printf("\n\tERROR: Difference of %d\n\n", seqTotal - total);

			}
			
			else {
				
				System.out.printf("\n\tSequence total of %d\n", seqTotal);
				System.out.printf("\tover %d midi clock cycles\n\n", f.b);
			}
		}
	}
}

class Factors {

	public int a;
	public int b;
	public int denominator;
	
	Factors(int denominator_) {

		a = 1;
		b = denominator_;
		denominator = denominator_;
			
		do {	// until b is not an even number
			
			int x = b >> 1;
			if((x << 1) != b) break;

			a <<= 1;
			b >>= 1;

		} while(true);
	}
}

